#
# config.mk
# There are essentially 3 configuration files:
# config.mk	This defines variables that are needed in all cases
# std.conf	This is customized for a particular distribution
#		to have knowledge about all the packages used by
#		the distribution.
# <foo>.conf	Customization of std.conf specific to a particular
#		environment.  e.g., specifing path locations and
#		the particular compilier to use.

ifndef Package
You-must-define-Package 
endif
ifndef PackageType
You-must-define-PackageType 
endif

ifndef MakeDir
MakeDir :=	 $(strip $(wildcard ~$(MakeUser)/etc/make))
endif

ifndef MAKEFILE
MAKEFILE :=	Makefile
endif

AllArchs :=	sun4 sun4.dbg sun4.opt sun4.pur \
		mips mips.dbg mips.opt mips.pur

#
# Get the archeture to build
#
ifndef ARCH
ARCH := $(shell $(MakeDir)/arch.sh)
endif
ifndef MakeArch
MakeArch := $(ARCH)
endif

include $(MakeDir)/$(MakeArch).arch

#
# Get the config and sub-config
#
include $(MakeDir)/std.conf
ifdef MakeConf
ifneq ($(MakeConf),std)
include $(MakeDir)/$(MakeConf).conf
endif
endif

# We explicitly reset stuff so that we dont inhierent for our parent
DependFile	:= DependInfo
ArchDependFile	:= $(MakeArch)/$(DependFile)
UseFileListB	:= 1
UseArchB	:= 1
LibName		:=
LibFile		:=
BinName		:=
BinFile		:=
TestBinName	:=
TestBinFile	:=
DocName		:=

ifeq ($(PackageType),library)
LibName		:= lib$(Package).a
LibFile		:= $(MakeArch)/$(LibName)
all:		$(LibFile)
endif

ifeq ($(PackageType),binary)
BinName		:= $(Package)
BinFile		:= $(MakeArch)/$(BinName)
all:		$(BinFile)
endif

ifeq ($(PackageType),libbin)
LibName		:= $(Package).a
LibFile		:= $(MakeArch)/$(LibName)
BinName		:= $(Package)
BinFile		:= $(MakeArch)/$(BinName)
all:		$(LibFile) $(BinFile)
endif

ifeq ($(PackageType),testbin)
TestBinName	:= $(Package)
TestBinFile	:= $(TestBinName)
PNT_INCLUDE	:= ..
PNT_LIBFILES	:= ../$(MakeArch)/lib$(PntPackage).a
all:		$(TestBinFile)
endif

ifeq ($(PackageType),document)
UseArchB	:= 0
DocName		:= $(Package)
all:		$(DocName).ps $(InfoFile)
ifdef BibFiles
BblFile		:= $(Package).bbl
$(DocName).dvi: $(BblFile)
endif
endif

ifeq ($(PackageType),other)
UseArchB	:= 0
all:		UserAll
endif

ifeq ($(PackageType),hierarchy)
UseFileListB	:= 0
UseArchB	:= 0
ifndef Packages
You-must-define-Packages
endif
all: ;
endif

All:		all
		@MakeTargets=$@ $(MakeSubPackages)      

SubMake := Package=$(Package) MAKE=$(MAKE) $(MakeDir)/submake.sh

# Siblings to ourselves
SibIncs :=	$(addprefix -I../, $(UseSiblings))
SibLibFiles :=  $(foreach f, $(UseSiblings), ../$(f)/$(MakeArch)/lib$(notdir $(f)).a)
# $(patsubst %, ../%/$(MakeArch)/lib$(notdir %).a, $(UseSiblings))
SibLibs :=	$(SibLibFiles)

# These are done afterwords so that XXX_INCLUDE is defined by *.conf
PkgIncs :=	$(foreach f, $(UsePackages), -I$($(f)_INCLUDE))
PkgLibFiles :=	$(foreach f, $(UsePackages), $($(f)_LIBFILES))
PkgLibs	:=	$(foreach f, $(UsePackages), \
		  $($(f)_LIBFILES) $($(f)_DASHLIBS))

# Must be =, not :=
CPPFLAGS =	$(UserCDefs) -I. $(SibIncs) $(PkgIncs)
CFLAGS =	$(UserCFlags) $(ArchCFlags) $(CPPFLAGS)
CXXFLAGS =	$(UserCFlags) $(ArchCFlags) $(CPPFLAGS)
LdLibFiles =	$(SibLibFiles) $(PkgLibFiles)
LdLibs =	$(SibLibs) $(PkgLibs) -lm
