#!/bin/sh

# This script traverses a source directory (tree?) and adds a copywrite 
# notice to all source files.
# It first removes the existing copyright and then adds the copyright
# located in copyright.txt.  For this to work, the copyright notice
# must start with 'Copyright (c)', and the last line
# of copyright.inc must have the keyword "COPYRIGHTENDKEY".
# Backup copies of the files before modification are stored
# in a subdirectory incase this program creams the source files somehow.
# The copyright is inserted at the same place as the current copyright,
# or the top if there is no current copyright.
#

BEGINKEY='Copyright (c)'
ENDKEY='COPYRIGHTENDKEY'
ALTENDKEY='All rights res'
CUSTOMKEY='CUSTOMCOPYRIGHT'
MakeDir=${MakeDir-$HOME/etc/make}

CopyrightFile=${CopyrightFile-copyright.inc}
if [ ! -r $CopyrightFile ] ; then
    CopyrightFile=$MakeDir/../copyright.inc
    if [ ! -r $CopyrightFile ] ; then
	echo Cant find copyright notice $CopyrightFile
	exit 1
    fi
fi

if [ "$1" = "" ] ; then
    echo Adding copyrights to all .h, .c, and .cc files.
    exec $0 *.h *.c *.cc
    exit 0
fi

SAVE=\#save$$
mkdir $SAVE

while [ $1x != x ] ; do
    F=$1
    FP=$1.$$
    shift
    # we dont want to remove the existing file, or the perms&ownerships
    # will get mixed up
    if [ $F = copyright.h ] ; then
	continue;
    fi
    if [ ! -r "$F" ] ; then
	echo $F: Doesnt exist: skipping...
	continue
    fi
    if grep -s $CUSTOMKEY $F ; then
	echo $F: Custom copyright: skipping...
	continue
    fi
    echo Processing $F

    sccsflag=0; rcsflag=0
    if [ ! -w "$F" ] ; then
	if [ -d SCCS ] ; then
	    sccs edit $F
	    sccsflag=1
	elif [ -d RCS ] ; then
	    co -l $F
	    rcsflag=1
	fi
        if [ ! -w $F ] ; then
	    echo $F: is not writable: cant insert copyright
	    continue
	fi
        echo $F: checked out to add copyright
    fi
    if [ -d $SAVE ] ; then
        cp $F $SAVE/$F
	if [ ! -s $SAVE/$F ] ; then
	    echo $F: Save file empty...disk may be full
	    continue
	fi
    fi
    cp $F $FP
    if [ ! -s $FP ] ; then
	echo $F: Save file empty...disk may be full
	rm $FP
	continue
    fi
    startline=`grep -n -i "$BEGINKEY" $FP | sed 's/:.*//'`
#    echo startline is $startline
    if echo $startline | grep -s ' ' ; then
	echo "$F: Contains multiple copyrights.  Skipping..."
	rm $FP
	continue
    fi
    if [ "$startline" = "" ] ; then
	echo "$F: No pre-existing copyright.  Inserting at top..."
	echo '/* '	> $F
	cat $CopyrightFile	>> $F
	echo '*/'	>> $F
	cat $FP		>> $F
    else
	if grep -s "$ENDKEY" $FP ; then
	    endline="/$ENDKEY/"
	else
	    endline=`grep -n -i "$ALTENDKEY" $FP | sed 's/:.*//' | head -1`
	    if [ "$endline" = "" ] ; then
		endline=$startline
	    fi
	fi
#	echo from $startline to $endline
	sed -e "$startline r $CopyrightFile" -e "$startline,$endline d" $FP	> $F
    fi
    rm $FP
    if [ "$sccsflag" = "1" ] ; then
	sccs delget $F -ycopyright
    elif [ "$rcsflag" = "1" ] ; then
	ci -mcopyright $F
    fi

done
