#!/bin/sh
#
# usage: create.sh pkgtype
# Recreate the Makefile in the current directory
# 
PKGTYPE=${1-library}

MakeDir=${MakeDir-$HOME/etc/make}

if [ -f "$MAKEFILE" ] ; then
    :
elif [ -f GNUmakefile ] ; then
    MAKEFILE=GNUmakefile
elif [ -f Makefile ] ; then
    MAKEFILE=Makefile
elif [ -f makefile ] ; then
    MAKEFILE=makefile
else
    echo "Warning: No pre-existing Makefile"
    MAKEFILE=${MAKEFILE-Makefile}
fi

if [ -f ${MAKEFILE}.new ] ; then
    echo "Error: Pre-existing file ${MAKEFILE}.new"
    echo "Error: ... remove and try again."
    exit 1
fi

##############################################################################
#
#	Create the system's variables here.  These depend on the type
#	of makefile involved.
#
##############################################################################
SYSVARS=.mkmfsvars
echo "#===========================SYS_VARS_BEGIN" > $SYSVARS
if [ "$PKGTYPE" = testbin ] ; then
    sed -n \
      -e '/^Package / s/^/Pnt/p' \
      -e '/^UseSiblings/ {' -e 's/^/Pnt/' -e 's@[ 	]\([a-zA-Z0-9]\{1,\}\)@ ../\1@g' -e p -e } \
      -e '/^UsePackages/ s/^/Pnt/p' \
      ../$MAKEFILE >> $SYSVARS
    cat <<'END_MAKE' >> $SYSVARS
Package :=	$(PntPackage)test
UsePackages :=  $(PntUsePackages)
UseSiblings :=  ../$(PntPackage) $(PntUseSiblings)
END_MAKE
fi
echo "#===========================SYS_VARS_END" >> $SYSVARS

##############################################################################
#
#	Extract the user's variable section.  This lets the user configure
#	all the important variables.  If doesn't exist, then provide
#	a template user's var section.
#
##############################################################################
USERVARS=.mkmfuvars
if [ -f "$MAKEFILE" ] ; then
    sed -n -e '/=====USER_VARS_BEGIN$/,/========USER_VARS_END$/ p' \
      $MAKEFILE > $USERVARS
fi
if [ ! -s $USERVARS ] ; then
    cat <<'END_MAKE' > $USERVARS
#===========================USER_VARS_BEGIN
Package :=	packageName
PackageType :=	library
#UseSiblings :=
#UsePackages :=	
#UserCDefs :=	
#UserCFlags :=
#Packages :=	
#TestPackages :=
#===========================USER_VARS_END
END_MAKE
fi

##############################################################################
#
#	Extract the user's rules section.  This lets the user make his
#	own rules.  It is seperate from the user's var section because
#	the rules might make use of stuff defined in config.mk.
#
##############################################################################
USERRULES=.mkmfurules
if [ -f "$MAKEFILE" ] ; then
    sed -n -e '/=====USER_RULES_BEGIN$/,/========USER_RULES_END$/ p' \
      $MAKEFILE > $USERRULES
fi
if [ ! -s $USERRULES ] ; then
    cat <<'END_MAKE' > $USERRULES
#===========================USER_RULES_BEGIN
#Insert whatever custom rules you want here.
#===========================USER_RULES_END
END_MAKE
fi


##############################################################################
#
#	Build the actual makefile.  The template below is used, and
#	certain keywords are replaced with the sections we built above.
#
##############################################################################
sed \
   -e "/^#SYS_VARS_SECTION/ {" -e "r $SYSVARS" -e d -e "}" \
   -e "/^#USER_VARS_SECTION/ {" -e "r $USERVARS" -e d -e "}" \
   -e "/^#USER_RULES_SECTION/ {" -e "r $USERRULES" -e d -e "}" \
 <<'END_MAKE' > ${MAKEFILE}.new
#
# Makefile
#
.PHONY: ENTRYPOINT all All
ENTRYPOINT:	all

ifndef MakeDir
MakeDir	:=	$(strip $(wildcard ~$(MakeUser)/etc/make))
endif

#SYS_VARS_SECTION

# PackageType must be one of: library, binary, testbin, hierarchy, other
#USER_VARS_SECTION

include $(MakeDir)/config.mk
#USER_RULES_SECTION
include $(MakeDir)/depend.mk
include $(MakeDir)/rules.mk

END_MAKE

if [ -f "${MAKEFILE}" ] ; then
    rm -f ${MAKEFILE}.old
    mv ${MAKEFILE} ${MAKEFILE}.old
fi
mv ${MAKEFILE}.new ${MAKEFILE}
rm -f $SYSVARS $USERVARS $USERRULES
exit 0
