#!/bin/sh
#
# Shell script to make directory "neat".  It deals with emacs
# files and warns the user about junk files.
# This take from the environment:
#	Packages
#	TestPackages
#	FlatSrcFiles
#

OkFiles="Makefile DependInfo tags TAGS OneLineDesc Backup RCS Junk junk lib doc ToDo Ideas Theory Changes"
Files=`ls | tr \\\12 \\\40`

#
# Deal with emacs check point files
#
EmacsJunk=
for f in "#*#"; do
    if [ -f "$f" ] ; then
	EmacsJunk="$EmacsJunk $f"
	Files=`echo $Files | sed -e "s@$f@@"`
    fi
done
if [ "$EmacsJunk" != '' ] ; then
    echo "Emacs check point files...clean up manually:"
    ls -l $EmacsJunk
fi

#
# Deal with emacs backup files
#
BackupFiles=
for f in *~; do
    if [ -f "$f" ] ; then
	BackupFiles="$BackupFiles $f"
    fi
done
if [ "$BackupFiles" != '' ] ; then
    mkdir Backup > /dev/null 2>&1
    chmod ug+w Backup
    mv -f $BackupFiles Backup
fi

if [ -d RCS ] ; then
    RLOGFILE=/tmp/neatrlog$$
    rlog -L -R $FlatSrcFiles > $RLOGFILE.co 2> $RLOGFILE.norcs
    if [ -s $RLOGFILE.norcs ] ; then
	echo "The following files are not RCS'ed:"
	cat $RLOGFILE.norcs
    fi
    if [ -s $RLOGFILE.co ] ; then
	echo "The following files are checked out (use ci -u filename):"
	ls -l `cat $RLOGFILE.co`
    fi
    rm -f $RLOGFILE $RLOGFILE.norcs $RLOGFILE.co
fi

Files=`echo " $Files " | sed -e "s/ sun4.* / /g" -e "s/ mips.* / /g"` 

for f in $OkFiles; do
    Files=`echo " $Files " | sed -e "s@ $f @ @"`
done

if [ "$Packages" != "" -o "$TestPackages" != "" ] ; then
    for f in $Packages $TestPackages ; do
        Files=`echo " $Files " | sed -e "s@ $f @ @"`
    done
fi

for f in $FlatSrcFiles ; do
    if [ -f $f ] ; then
	Files=`echo " $Files " | sed -e "s@ $f @ @"`
    else
	echo "Missing source file \`\`$f''"
    fi
done

Files=`echo $Files`	# Gets rid of excess spaces
if [ "$Files" != '' ] ; then
    echo 'The following files are extra:'
    ls -l -d $Files
fi
