#
# rules.mk
# GNUMake rules to compile stuff
#

ifdef DocName
include $(MakeDir)/rulesdoc.mk
else
include $(MakeDir)/rulesobj.mk
endif

# Just verify that the makefile system is alive
make-ping:	FORCE
		@echo $(Package) $(MAKEFILE) is alive

Make-ping:	make-ping
		@MakeTargets=$@ $(MakeAllPackages)

ifdef MakeExpert
Makefile makefile:	$(MakeDir)/create.sh
		@echo $@ is out of date, remaking in $(Package)...
		$(MakeDir)/create.sh $(PackageType)
		$(MAKE) $(DependFile) $(ArchDependFile)
		@echo "--Makefile-- Done."
endif

.PHONY: newMakefile NewMakefile pnt

newMakefile:	FORCE
		$(MakeDir)/create.sh $(PackageType)
		-rm -f $(DependFile)
		$(MAKE) $(DependFile)
		@echo "--newMakefile-- Done."

NewMakefile:	newMakefile
		@MakeTargets=$@ $(MakeAllPackages)


pnt:		FORCE
		chdir ..; $(MAKE)


.PHONY: depend Depend neat_clean clean Clean pristine Pristine neat Neat wrap Wrap

depend:		FORCE
		@rm -f $(ArchDependFile)
		$(MAKE) $(ArchDependFile) archDepend
		@echo "--depend-- Done."

Depend:		depend
		@MakeTargets=$@ $(MakeAllPackages)


# clean and pristine targets are declared in one of the sub-rules files
Clean:		clean
		@MakeTargets=$@ $(MakeAllPackages)

Pristine:	pristine
		@MakeTargets=$@ $(MakeAllPackages)

# Neaten up the source directory, dont care about archs
neat:		neat_clean
		@echo 'neat.sh'
		@FlatSrcFiles="$(SrcFiles)" Packages="$(Packages)" \
		  TestPackages="$(TestPackages)" $(MakeDir)/neat.sh

Neat:		neat
		@MakeTargets=$@ $(MakeAllPackages)


# Properly "wrap" all the source code.
wrap:		FORCE
		@echo 'wrapcode.sh'
		$(MakeDir)/wrapcode.sh $(SrcFiles)

Wrap:		wrap
		@MakeTargets=$@ $(MakeAllPackages)

.PHONY: $(AllArchs) Tags

$(AllArchs):	FORCE
		@echo "Use the \"MakeArch\" environment variable

tags:		$(C_HdrFiles) $(C_SrcFiles) $(Tcl_SrcFiles)
		@rm -f tags; touch tags
ifneq "$(C_HdrFiles) $(C_SrcFiles)" " "
		-$(CTAGS) $(CTAGSFLAGS) $(C_HdrFiles) $(C_SrcFiles)
endif
ifneq "$(Tcl_SrcFiles)" ""
		$(MakeDir)/tcltags.sh $(Tcl_SrcFiles) >> tags
endif
		@echo "tags made."
Tags:		tags
		@MakeTargets=$@ $(MakeSubPackages)
		
TAGS:		$(C_HdrFiles) $(C_SrcFiles)
ifneq "$(C_HdrFiles) $(C_SrcFiles)" ""
		-$(ETAGS) $(ETAGSFLAGS) $(C_HdrFiles) $(C_SrcFiles)
endif
		@echo "TAGS made."

.PHONY: rcd rcdg rce rcg rdist

# rce     sccs edit       co -l   Check out for editing.
# rcg     sccs get        co      Check out for compilation.
# rcd     sccs delta      ci      Check in for delta.
# rcdg    sccs delget     ci -u   Check in a change and leave a copy.
# rcdf    sccs delget     ci -f -u   Check in a change and leave a copy.
# rcdi    sccs diffs      rcsdiff Generate diffs between versions.

rce:		FORCE
		-$(CO) -l $(SrcFiles)
Rce:		rce
		@MakeTargets=$@ $(MakeAllPackages)
rcg:		FORCE
		-$(CO) $(SrcFiles)
Rcg:		rcg
		@MakeTargets=$@ $(MakeAllPackages)
rcd rcdg:	FORCE
		-$(CI) '-m$(MSG)' -tOneLineDesc -u $(SrcFiles)
Rcd Rcdg:	rcd
		@MakeTargets=$@ $(MakeAllPackages)
rcdf:		FORCE
		-$(CI) '-m$(MSG)' -tOneLineDesc -u -f $(SrcFiles)
Rcdf:		rcdf
		@MakeTargets=$@ $(MakeAllPackages)

ifeq ($(UseArchB),1)

srcdist:	FORCE
		$(MakeDir)/srcdist.sh

else

srcdist:	FORCE
		$(MakeDir)/srcdist.sh Makefile Depend $(Packages)

endif

ifdef XResName
include $(MakeDir)/xres.mk
endif

#############################################################################
#
#	 Support for makefile hierarchy (recursion)
#
#############################################################################

.PHONY: makepackages

#
# GNUMake passes all vars down through the environment; this can
# screwup the child make.  There is no easy way to stop this, so
# we must make sure to reset all critical vars upon entry.
#
DontPassVars =	Package PackageType Packages TestPackages \
		LibFile BinFile TestBinFile XResName \
		MakeSubPackages MakeTestPackages MakeAllPackages \
		UseArchB

#
# This is generally not invoked directly, but rather recursively.
# The caller should have set $(MakeTargets) and $(MakePackages)
#
#
makepackages:	FORCE
		@$(MakeDir)/submake.sh

#############################################################################
#
#	 Turd targets
#
#############################################################################

FORCE:
