#
# GNUMake rules to objects, libraries, binaries, and related files
# These are sufficiently complicated that these rules get there own file.
#

$(MakeArch)/%.o: %.c
	$(CC) -o $@ -c $(CFLAGS) $<

$(MakeArch)/%.o: %.cc
	$(CXX) -o $@ -c $(CXXFLAGS) $<

%.s: %.c
	$(CC) -S $(CFLAGS) $<

%.s: %.cc
	$(CXX) -S $(CXXFLAGS) $<

%.E: %.c
	$(CC) -E $(CFLAGS) $< > $@

%.E: %.cc
	$(CXX) -E $(CXXFLAGS) $< > $@

.PHONY: objs Objs

#
# Build all the object files
#
objs:		$(ObjFiles)
		@echo "All object files made."

Objs:		objs
		@MakeTargets=$@ $(MakeSubPackages)

#
# Build a library
#
ifdef LibFile
$(LibName):	$(LibFile)
$(LibFile):	$(ObjFiles)
		-@mv -f $@ $@.old > /dev/null 2>&1
		ar cq $@ $(ObjFiles)
		ranlib $@
		-@chmod g=u $(LibFile) $(ObjFiles) > /dev/null 2>&1
		@ls -lagF $@
endif

ifdef BinFile
$(BinName):	$(BinFile)
$(BinFile):	$(ObjFiles) $(LdLibFiles)
		-@mv -f $@ $@.old > /dev/null 2>&1
		$(LD) -o $@ $(LDFLAGS) $(ObjFiles) $(LdLibs)
		-@chmod g=u $(BinFile) $(ObjFiles) > /dev/null 2>&1
		@ls -lagF $@
endif

ifdef TestBinFile
# $(TestBinName):	$(TestBinFile)
$(TestBinFile):	$(ObjFiles) $(LdLibFiles)
		-@mv -f $@ $@.old > /dev/null 2>&1
		$(LD) -o $@ $(LDFLAGS) $(ObjFiles) $(LdLibs)
		@ls -lagF $@

saber:		FORCE
		#load $(CFLAGS) $(C_SrcFiles)
		#load $(LDFLAGS) $(LdLibs)
endif

# kill off old and junk files
neat_clean:	FORCE
		-rm -f $(MAKEFILE).old core *.E *.s

# kill all objs,lib,bin of the current arch
clean:		FORCE
		-rm -f $(MAKEFILE).old \
		  $(LibFile) $(LibFile).old $(BinFile) $(BinFile).old \
		  $(TestBinFile) $(TestBinFile).old \
		  $(ObjFiles) core *.E *.s

# kill all objs,lib,bin of ALL archs
pristine:	FORCE
		-rm -rf $(AllArchs)
		-rm -f $(MAKEFILE).old *.E *.s *.o \
	          $(LibFile) $(LibFile).old $(BinFile) $(BinFile).old \
		  $(TestBinFile) $(TestBinFile).old \
