/* 
    Basic math functions on EEMComplex.

    Author: Kennard White

    Copyright (c) 1990-93 The Regents of the University of California.
    All rights reserved.  See the file "./copyright.h" for full copyright
    notice, limitation of liability, and disclaimer of warranty provisions.
#include "copyright.h"				COPYRIGHTENDKEY
*/
#if !defined(lint) && !defined(SABER)
static char rcsid[] = "$Header: /vol/cooley/cooley1/kennard/src/eem/RCS/cplx.c,v 1.17 93/08/27 14:29:43 kennard Exp $";
#endif

#include "topFixup.h"
#include <stdio.h>
#include <math.h>

#include "topStd.h"
#include "eemDflt.h"
#include "eemCplx.h"

/*global*/ char _EemCplxSubPackage[] = "eem.cplx";
#define SPKG _EemCplxSubPackage

void eemCplxZero( EEMComplex *pC) {
    pC->r = 0;
    pC->i = 0;
}

void eemCplxSet( EEMComplex *pC, double R, double I) {
    pC->r = R;
    pC->i = I;
}

TOPLogical eemCplxSameB( EEMComplex *a, EEMComplex *b) {
    return eemDfltSameB( a->r, b->r) && eemDfltSameB( a->i, b->i);
}
TOPLogical eemCplxNegSameB( EEMComplex *a, EEMComplex *b) {
    return eemDfltSameB( a->r, - b->r) && eemDfltSameB( a->i, - b->i);
}
TOPLogical eemCplxConjSameB( EEMComplex *a, EEMComplex *b) {
    return eemDfltSameB( a->r, b->r) && eemDfltSameB( a->i, - b->i);
}
TOPLogical eemCplxZeroB( EEMComplex *a) {
    return eemDfltSameB( a->r, 0.0) && eemDfltSameB( a->i, 0.0);
}
TOPLogical eemCplxOneB( EEMComplex *a) {
    return eemDfltSameB( a->r, 1.0) && eemDfltSameB( a->i, 0.0);
}

void eemCplxNeg1( EEMComplex *pD) {
    pD->r = - pD->r;
    pD->i = - pD->i;
}
void eemCplxNeg2( EEMComplex *pD, EEMComplex *pS) {
    pD->r = - pS->r;
    pD->i = - pS->i;
}

void 
eemCplxPrint(EEMComplex *pP, char* title) {
    if ( title )	printf("%s: ", title);
    printf("(%.20g,%.20g) ", pP->r, pP->i);
}

void 
eemCplxPrint2(EEMComplex *pP, EEMComplex *pC, char* title){
    printf("%s: (%3g,%3g) is (%3g,%3g)", title,
      pC->r,pC->i,  pP->r,pP->i);
}
void
eemRealToCplx(EEMComplex *pC, double R){
    pC->r=R;
    pC->i=0.0;
}

void eemCplxAdd2( EEMComplex *pD, EEMComplex *pS) {
    pD->r += pS->r;
    pD->i += pS->i;
}
void eemCplxAdd3( EEMComplex *pS1, EEMComplex *pS2, EEMComplex *pD) {
    pD->r = pS1->r + pS2->r;
    pD->i = pS1->i + pS2->i;
}

void eemCplxSub2( EEMComplex *pD, EEMComplex *pS) {
    pD->r -= pS->r;
    pD->i -= pS->i;
}
void eemCplxSub3( EEMComplex *pS1, EEMComplex *pS2, EEMComplex *pD) {
    pD->r = pS1->r - pS2->r;
    pD->i = pS1->i - pS2->i;
}

void eemCplxMul2( EEMComplex *pD, EEMComplex *pS) {
    double dr = pD->r;
    double di = pD->i;
    pD->r = dr * pS->r - di * pS->i;
    pD->i = di * pS->r + dr * pS->i;
}

void eemCplxMul3( EEMComplex *pS1, EEMComplex *pS2, EEMComplex *pD) {
    double dr = pS1->r;
    pD->r = pS1->r * pS2->r - pS1->i * pS2->i;
    pD->i = pS1->i * pS2->r + dr * pS2->i;
}

void eemCplxDiv2( EEMComplex *pD, EEMComplex *pS) {
    double denom = pS->r * pS->r + pS->i * pS->i;
    double dr = pD->r;
    if ( denom == 0 )	denom = 1;	/* XXX: hack */
    pD->r = (pD->r * pS->r + pD->i * pS->i) / denom;
    pD->i = (pD->i * pS->r - dr * pS->i) / denom;
}

void eemCplxDiv3( EEMComplex *pS1, EEMComplex *pS2, EEMComplex *pD) {
    double denom = pS2->r * pS2->r + pS2->i * pS2->i;
    double dr = pS1->r;
    if ( denom == 0 )	denom = 1;	/* XXX: hack */
    pD->r = (pS1->r * pS2->r + pS1->i * pS2->i) / denom;
    pD->i = (pS1->i * pS2->r - dr * pS2->i) / denom;
}

void 
eemCplxInv1( EEMComplex *pD) {
    double denom = EEM_CPLX_MAGSQ(pD);
    pD->r /= denom;
    pD->i /= - denom;
}

void
eemCplxInv2( EEMComplex *pD, EEMComplex *pS) {
    double denom = EEM_CPLX_MAGSQ(pS);
    pD->r = pS->r / denom;
    pD->i = - pS->i / denom;
}

double
eemCplxMag( EEMComplex *pS) {
    return sqrt( EEM_CPLX_MAGSQ( pS));
}

void 
eemCplxCopy(EEMComplex *pD, EEMComplex *pS){
    pD->r = pS->r;
    pD->i = pS->i;
}
void 
eemCplxCopyNeg(EEMComplex *pD, EEMComplex *pS){
    pD->r = -pS->r;
    pD->i = -pS->i;
}


void eemCplxPowDflt2( EEMComplex *pD, double power) {
    double	magsq, theta, powmag;
    /*
     * polar expansion: method works for non-integer powers also
     * (sr+j si) = r e^(jw)
     * (r e^(jw))^n = r^n e^(jnw)
     */
    magsq = EEM_CPLX_MAGSQ(pD);
    theta = atan2( pD->i, pD->r);
    powmag = pow( magsq, power/2.0);
    pD->r = powmag * cos( power * theta);
    pD->i = powmag * sin( power * theta);
}

void eemCplxPow2( EEMComplex *pD, int power) {
    double		sr = pD->r, dr;
    double		si = pD->i, di;
    int			i, abspow;

    if (power == 0) {
	EEM_CPLX_SET(pD, 1.0, 0.0);
	return;
    }
    if ( pD->i==0.0 ) {
	pD->r = pow( pD->r, (double)power);
	pD->i = 0.0;
	return;
    }
    abspow = TOP_ABS(power);
    if ( abspow >= 50 ) {
	eemCplxPowDflt2( pD, (double)power);
	return;
    }

    /*
     * Brute force.  Just do abspow complex multiplications.
     */
    dr = sr; di = si;
    for ( i=1; i < abspow; i++) {
	double		pre_dr = dr, pre_di = di;
     	dr = pre_dr * sr - pre_di * si;
    	di = pre_di * sr + pre_dr * si;
    }
    pD->r = dr;
    pD->i = di;

    if ( power < 0 ) 
	eemCplxInv1(pD);

#ifdef binomial_way
    /*
     * binomial expansion: I dont think this is a good method
     * (sr+j si)^n = sum(k=0..n) (n choose k) * sr^(n-k) * si^k
     * note that (n ch k+1) = (n ch k) * (n-k)/(k+1)
     * so	 (n ch k) = (n ch k-1) * (n-k-1)/k
     */
    srinv = 1.0 / sr;
    srp = pow( sr, (double)abspow); sri = 1.0; binom = 1.0;
    for ( k=0; ; sip = 0 - sip) {
	dr += binom * srp * sip;
	if ( ++k == abspow )	break;
	srp *= srinv; sip *= si; binom *= (abspow-k-1) / k;

	di += binom * srp * sip;
	if ( ++k == abspow )	break;
	srp *= srinv; sip *= si; binom *= (abspow-k-1) / k;
    }
#endif

}

void eemCplxPow3( EEMComplex *pS1, int power, EEMComplex *pD) {
    *pD = *pS1;
    eemCplxPow2( pD, power);
}

double
eemCplxPhase( double rval, double ival) {
    return EEM_CPLX_PHASE(rval,ival);
}

/* real/imaj ----> mag/phase */
void eemCplxMagPhase( EEMComplex *pD, EEMComplex *pS) {
    double sr = pS->r;
    double si = pS->i;
    pD->r = sqrt( sr * sr + si * si);
    pD->i = EEM_CPLX_PHASE(sr,si);
}

/* mag/phase  ---> real/imaj */
void eemCplxRealImag( EEMComplex *pD, EEMComplex *pS) {
    double sr = pS->r;
    double si = pS->i;
    pD->r = sr * cos(si);
    pD->i = sr * sin(si);
}

void eemCplxSqr1( EEMComplex *pS) {
    double	sr = pS->r;
    double	si = pS->i;
    pS->r = sr*sr - si * si;
    pS->i = 2 * sr * si;
}

void eemCplxSqr2( EEMComplex *pD, EEMComplex *pS) {
    double	sr = pS->r;
    double	si = pS->i;
    pD->r = sr*sr - si * si;
    pD->i = 2 * sr * si;
}

void eemCplxSqrt2( EEMComplex *pS, EEMComplex *pD) {
    double	sr = pS->r;
    double	si = pS->i;

    if ( si == 0.0 ) {
	/* the orgin or pure real */
	if ( sr == 0.0 ) {
	    pD->r = 0.0;
	    pD->i = 0.0;
	} else if ( sr < 0 ) {
	    pD->r = 0.0;
	    pD->i = sqrt( -sr);
	} else {
	    pD->r = sqrt( sr);
	    pD->i = 0.0;
	}
    } else if ( sr == 0.0 ) {
	double ang45;
	/* pure imaginary */
	if ( si < 0 ) {
	    ang45 = sqrt( -si) * M_SQRT1_2;
	    pD->r = ang45; pD->i = - ang45;
	} else {
	    ang45 = sqrt( si) * M_SQRT1_2;
	    pD->r = ang45; pD->i =   ang45;
	}
    } else {
        double	mag, theta, rootmag;

	mag = sqrt( sr*sr + si*si);
	theta = atan2( si, sr);
	rootmag = sqrt( mag);
	pD->r = rootmag * cos( theta/2.0);
	pD->i = rootmag * sin( theta/2.0);
    }
}

void eemCplxSqrt1( EEMComplex *pVal) {
    eemCplxSqrt2( pVal, pVal);	/* this happens to be is safe */
}


double eemCplxDistSq( EEMComplex *pS1, EEMComplex *pS2) {
    double distr = pS1->r - pS2->r;
    double disti = pS1->i - pS2->i;
    return( distr*distr + disti*disti );
}

void
eemCplxMaxRect( EEMComplex *pL, EEMComplex *pLo, EEMComplex *pHi) {
    if ( pL->r < pLo->r)	pLo->r = pL->r;
    if ( pL->r > pHi->r)	pHi->r = pL->r;
    if ( pL->i < pLo->i)	pLo->i = pL->i;
    if ( pL->i > pHi->i)	pHi->i = pL->i;
}

void
eemCplxQuadraticRoots( EEMComplex *pA, EEMComplex *pB, EEMComplex *pC,
  EEMComplex *pR1, EEMComplex *pR2) {
    EEMComplex modB, modC, acc;

    if ( pB==NULL || (pB->r == 0.0 && pB->i == 0.0) ) {
	/* ax^2 + c = 0		==> x^2 = c / a */
        if ( pA==NULL || (pA->r == 1.0 && pA->i == 0.0) ) {
	    modC = *pC;
	} else {
	    eemCplxDiv3( pC, pA, &modC);
	}
	modC.r = - modC.r;
	modC.i = - modC.i;
        eemCplxSqrt2( &modC, pR1);
	pR2->r = - pR1->r;
	pR2->i = - pR1->i;
	return;
    }
    if ( pA==NULL || (pA->r == 1.0 && pA->i == 0.0) ) {
	modB = *pB; modC = *pC;
    } else {
	eemCplxDiv3( pB, pA, &modB);
	eemCplxDiv3( pC, pA, &modC);
    }
    acc.r = modB.r*modB.r - modB.i * modB.i;
    acc.i = 2 * modB.r * modB.i;
    acc.r -= 4 * modC.r;
    acc.i -= 4 * modC.i;
    eemCplxSqrt1( &acc);

    pR1->r = (- modB.r + acc.r) / 2.0;
    pR1->i = (- modB.i + acc.i) / 2.0;
    pR2->r = (- modB.r - acc.r) / 2.0;
    pR2->i = (- modB.i - acc.i) / 2.0;
}

/*
    Calculates pD = exp(pS * A);
*/
void
eemCplxExp( double A, EEMComplex *pS, EEMComplex *pD) {
    double	rexp;
    sincos( pS->i * A, &pD->i, &pD->r);
    rexp = exp( pS->r * A);
    pD->r *= rexp; pD->i *= rexp;
}
