/* 
    Author: Kennard White

    Copyright (c) 1990-93 The Regents of the University of California.
    All rights reserved.  See the file "./copyright.h" for full copyright
    notice, limitation of liability, and disclaimer of warranty provisions.
#include "copyright.h"				COPYRIGHTENDKEY
*/
#if !defined(lint) && !defined(SABER)
static char rcsid[] = "$Header: /vol/cooley/cooley1/kennard/src/eem/RCS/dflt.c,v 1.14 93/08/27 14:29:39 kennard Exp $";
#endif

#include "topFixup.h"
#include <stdio.h>
#include <math.h>

#include "topStd.h"
#include "eemDflt.h"

static TOPLogical EEMParamsInited = TOP_FALSE;
/*global*/ EEMParamsStruct EEMParams;


#define EEM_PARAMINIT() {			\
    if ( EEMParamsInited == TOP_FALSE ) {	\
	eemParamsInit();			\
    }						\
}

/*
 * Calculate by brute force the Unit in the Last Place
 * We just bi-sect our way down.  You can find smaller
 * ULP's by using a factor of (2/3) rather than (1/2).
 * For Sparc IEEE, this returns 1.11E-16, the same as paranoia.
 * DEC's Ultrix defines DFLT_EPSILON to be 2.22E-16 for some reason.
 */
double eemParamCalcUlp() {
    double ulp, new_ulp = 1.001, oneANDulp;
    do {
	ulp = new_ulp;
	new_ulp = .5 * new_ulp;
	oneANDulp = 1.0+new_ulp;
	/* printf( "ulp=%25.20g 1+ulp=%25.20g\n", new_ulp, oneANDulp); */
    } while ( oneANDulp > 1.0 );
    return ulp;
}

/*
 * Calculate the smallest positive number
 */
double eemParamCalcSmallno() {
#if defined(__mips)
#include <values.h>
    /* This is bad, b/c MINDOUBLE is min_subnormal, not min_normal */
    return MINDOUBLE;
#else
    extern double min_normal();
    return min_normal();
#endif
}

void eemParamsCalc() {
    EEM_PARAMINIT();
    EEMParams.Abstol = EEMParams.CurAddErr * EEMParams.Smallno;
    EEMParams.Reltol = EEMParams.CurMulErr * EEMParams.Ulp;
    EEMParams.HalfReltol = EEMParams.Reltol/2.0;
    EEMParams.Polytol = EEMParams.Reltol / 100;
}

void eemParamsInit() {
    if ( EEMParamsInited )
	return;
    EEMParams.Smallno = eemParamCalcSmallno();
    EEMParams.Ulp = eemParamCalcUlp();
    EEMParams.DefAddErr = 10;
    EEMParams.DefMulErr = 10;
    strcpy( EEMParams.DfltFmtStr, "%#.12g");
    strcpy( EEMParams.CplxFmtStr, "%#.12g %#.12g");
    EEMParamsInited = TOP_TRUE;

    eemParamsSet( EEMParams.DefAddErr, EEMParams.DefMulErr);
}


void eemParamsSet( int addErr, int mulErr) {
    EEM_PARAMINIT();
    EEMParams.CurAddErr = addErr;
    EEMParams.CurMulErr = mulErr;
    eemParamsCalc();
}

void eemParamsSetDfltFmtStr( char *fmt) {
    strcpy( EEMParams.DfltFmtStr, fmt);
}
void eemParamsSetCplxFmtStr( char *fmt) {
    strcpy( EEMParams.CplxFmtStr, fmt);
}


double eemParamGetUlp() {
    EEM_PARAMINIT();
    return EEMParams.Ulp;
}

TOPLogical eemDfltSameB( double a, double b) {
    double	bound;
    if ( a == b )
	return TOP_TRUE;
    EEM_PARAMINIT();
    bound = TOP_ABS(a+b)*EEMParams.HalfReltol + EEMParams.Abstol;
    return TOP_ABS(a-b) < bound;
}

/**
    Factorials up to 12 fit in long.
**/
static long _EemFactorialTbl[13] = { 
   1,		1,	2,		6,		/* 0 - 3 */
   24,		120,	720,		5040, 		/* 4 - 7 */
   40320,	362880,	3628800,	39916800,	/* 8 - 11 */
   479001600						/* 12 */
};

double eemFactorial( int n) {
    double	p;
    if ( n==0 )
	return 1.0;
    else if ( n <= 12 ) {
	if ( n < 0 ) {
	    return -1;
	}
	return (double) _EemFactorialTbl[n];
    }
    p = _EemFactorialTbl[12];
    for ( ; n > 12; n--) {
	p *= n;
    }
    return p;
}


/**
    Compute the binomial (n choose k), as defined by n! / k! / (n-k)!
    See "src/misc/bintbl.c".
**/

#define EEM_BTS 11
static u_char _EemBinomialTbl[EEM_BTS][EEM_BTS] = { 	/* [n][k] */
/*n=0*/                                   { 1, },
/*n=1*/                               { 1,      1, },
/*n=2*/                           { 1,      2,      1, },
/*n=3*/                       { 1,      3,      3,     1, },
/*n=4*/                    { 1,     4,      6,      4,     1, },
/*n=5*/                { 1,     5,     10,     10,     5,     1, },
/*n=6*/             { 1,     6,    15,     20,     15,     6,    1, },
/*n=7*/          { 1,    7,    21,     35,     35,    21,     7,   1, },
/*n=8*/        { 1,   8,    28,    56,     70,     56,    28,    8,   1, },
/*n=9*/     { 1,   9,   36,    84,    126,    126,    84,    36,   9,   1, },
/*n=10*/  { 1,  10,  45,   120,   210,    252,    210,   120,   45,  10,  1, },
};

double eemBinomial( int n, int k) {
    int		t;
    double	p;

    if ( n < 0 || k < 0 )
	return 0.0;
    if ( n < EEM_BTS && k < EEM_BTS ) {
	return (double) _EemBinomialTbl[n][k];
    }
    /*
     * The test below is not just an optimzation;
     * the algorithm fails for these values because we've unwrapped loop
     */
    if ( k==0 || k==n )
	return 1.0;

    if ( k < n/2 )	k = n - k;
    for ( p=n, t=n-1; t > k; t--)
	p *= t;
    return( p / eemFactorial( n-k) );
}
