/* 
    dfltTcl.c :: EEM Library

    Tcl interface to EEM double-float functions

    Author: Kennard White

    Copyright (c) 1990-93 The Regents of the University of California.
    All rights reserved.  See the file "./copyright.h" for full copyright
    notice, limitation of liability, and disclaimer of warranty provisions.
#include "copyright.h"				COPYRIGHTENDKEY
*/
#if !defined(lint) && !defined(SABER)
static char rcsid[] = "$Header: /vol/cooley/cooley1/kennard/src/eem/RCS/dfltTcl.c,v 1.13 93/08/27 14:29:38 kennard Exp $";
#endif

#include "topFixup.h"
#include <math.h>
#include <tcl.h>

#include "topStd.h"

#include "eemDflt.h"

/**************************************************************************
		int functions
**************************************************************************/

int
eemInt2PrsTcl( Tcl_Interp *ip, char *str, int *pX, int *pY) {
    char	*s = str, *strEnd;

    *pX = strtol( s, &strEnd, 0);
    if ( s == strEnd ) {
	Tcl_AppendResult(ip,"Int2: bad x coord in ``",str,"''",NULL);
	return TCL_ERROR;
    }
    s = strEnd;
    *pY = strtol( s, &strEnd, 0);
    if ( s == strEnd ) {
	Tcl_AppendResult(ip,"Int2: bad y coord in ``",str,"''",NULL);
	return TCL_ERROR;
    }
    if ( *strEnd != '\0' ) {
	Tcl_AppendResult(ip,"Int2: trailing junk in ``",str,"''",NULL);
	return TCL_ERROR;
    }
    return TCL_OK;
}

int
eemInt4PrsTcl( Tcl_Interp *ip, char *str, int *pX1,int *pY1, int *pX2,int *pY2){
    char	*s = str, *strEnd;

    *pX1 = strtol( s, &strEnd, 0);
    if ( s == strEnd ) {
	Tcl_AppendResult(ip,"Int4: bad x1 coord in ``",str,"''",NULL);
	return TCL_ERROR;
    }
    s = strEnd;
    *pY1 = strtol( s, &strEnd, 0);
    if ( s == strEnd ) {
	Tcl_AppendResult(ip,"Int4: bad y1 coord in ``",str,"''",NULL);
	return TCL_ERROR;
    }
    s = strEnd;
    *pX2 = strtol( s, &strEnd, 0);
    if ( s == strEnd ) {
	Tcl_AppendResult(ip,"Int4: bad x2 coord in ``",str,"''",NULL);
	return TCL_ERROR;
    }
    s = strEnd;
    *pY2 = strtol( s, &strEnd, 0);
    if ( s == strEnd ) {
	Tcl_AppendResult(ip,"Int4: bad y2 coord in ``",str,"''",NULL);
	return TCL_ERROR;
    }
    if ( *strEnd != '\0' ) {
	Tcl_AppendResult(ip,"Int4: trailing junk in ``",str,"''",NULL);
	return TCL_ERROR;
    }
    return TCL_OK;
}

/**************************************************************************
		dflt functions
**************************************************************************/

void
eemDfltFmtTcl( Tcl_Interp *ip, double val, TOPLogical NestB) {
    char	buf[100];

    /* Ignore NestB...never need to nest a single float */
    sprintf( buf, EEMParams.DfltFmtStr, val);
    Tcl_AppendElement( ip, buf, 0);
}

int
eemDflt2PrsTcl( Tcl_Interp *ip, char *str, double *pX, double *pY) {
    char	*s = str, *strEnd;

    *pX = strtod( s, &strEnd);
    if ( s == strEnd ) {
	Tcl_AppendResult(ip,"Dflt2: bad x coord in ``",str,"''",NULL);
	return TCL_ERROR;
    }
    s = strEnd;
    *pY = strtod( s, &strEnd);
    if ( s == strEnd ) {
	Tcl_AppendResult(ip,"Dflt2: bad y coord in ``",str,"''",NULL);
	return TCL_ERROR;
    }
    if ( *strEnd != '\0' ) {
	Tcl_AppendResult(ip,"Dflt2: trailing junk in ``",str,"''",NULL);
	return TCL_ERROR;
    }
    return TCL_OK;
}

int 
_eemDfltCmds( ClientData cld, Tcl_Interp *ip, int aC, char **aV) {
    double	op1, op2, res;
    TOPLogical	CastIntB = TOP_FALSE;
    char	*fn = aV[0], *cmd;

    if ( aC >= 2 && strcmp(aV[1], "-int")==0 ) {
	CastIntB = TOP_TRUE;
	--aC; ++aV;
    }
    if ( aC < 3 ) {
	Tcl_AppendResult(ip,"usage: ",fn," sqr|sqrt|cos|sin|tan|floor|ceil op1",
	  NULL);
	return TCL_ERROR;
    }
    if ( Tcl_ExprDouble( ip, aV[2], &op1) != TCL_OK ) {
	Tcl_AppendResult(ip,"\n",fn," ",aV[1],": op1 bad",NULL);
	return TCL_ERROR;
    }

    cmd = aV[1];
    /*IF*/ if ( strcmp( cmd, "sqrt")==0 ) {
	res = sqrt(op1);
    } else if ( strcmp( cmd, "sqr")==0 ) {
	res = op1 * op1;
    } else if ( strcmp( cmd, "cos")==0 ) {
	res = cos(op1);
    } else if ( strcmp( cmd, "sin")==0 ) {
	res = sin(op1);
    } else if ( strcmp( cmd, "tan")==0 ) {
	res = tan(op1);
    } else if ( strcmp( cmd, "atan")==0 ) {
	res = atan(op1);
    } else if ( strcmp( cmd, "exp")==0 ) {
	res = exp(op1);
    } else if ( strcmp( cmd, "exp10")==0 ) {
	res = exp10(op1);
    } else if ( strcmp( cmd, "log")==0 ) {
	res = log(op1);
    } else if ( strcmp( cmd, "log10")==0 ) {
	res = log10(op1);
    } else if ( strcmp( cmd, "floor")==0 ) {
	res = floor(op1);
    } else if ( strcmp( cmd, "ceil")==0 ) {
	res = ceil(op1);
    } else if ( strcmp( cmd, "rint")==0 ) {
	res = rint(op1);
    } else if ( strcmp( cmd, "factorial")==0 ) {
	res = eemFactorial((int)op1);
    } else {
	if ( aC < 4 || Tcl_ExprDouble( ip, aV[3], &op2) != TCL_OK ) {
	    Tcl_AppendResult(ip,"\n",fn," ",cmd,": op2 bad",NULL);
	    return TCL_ERROR;
	}
	/*IF*/ if ( strcmp( cmd, "pow")==0 ) {
	    res = pow(op1,op2);
	} else if ( strcmp( cmd, "binomial")==0 ) {
	    res = eemBinomial((int)op1,(int)op2);
	} else if ( strcmp( cmd, "atan2")==0 ) {
	    res = atan2(op1,op2);
	} else {
	    Tcl_AppendResult(ip,fn,": bad opcode: ``",cmd,"''",NULL);
	    return TCL_ERROR;
	}
    }
    if ( CastIntB ) {
	Tcl_ResetResult( ip);
	sprintf( ip->result, "%d", (int)res);
    } else {
        eemDfltFmtTcl( ip, res, TOP_FALSE);
    }
    return TCL_OK;
}

/**************************************************************************
		params
**************************************************************************/

int 
_eemParamsCmds( ClientData cld, Tcl_Interp *ip, int aC, char **aV) {
    int		cae, cme, len;
    char	*cmd, c, buf[100];

    eemParamsInit();

    if ( aC < 2 ) {
	Tcl_AppendResult(ip,"usage: ",aV[0]," adderr|mulerr|dfltfmt|cplxfmt value",NULL);
	return TCL_ERROR;
    }
    cmd = aV[1]; len = strlen(cmd); c = cmd[0];

    cae = EEMParams.CurAddErr;
    cme = EEMParams.CurMulErr;

    /*IF*/ if ( c=='a' && strncmp(cmd,"adderror",len)==0 ) {
	if ( aC == 3 ) {
	    if ( Tcl_GetInt(ip, aV[2], &cae) != TCL_OK ) {
	        Tcl_AppendResult(ip,"\n",aV[0],": bad value",NULL);
	        return TCL_ERROR;
	    }
	    eemParamsSet( cae, cme);
	} else {
    	    sprintf(buf,"%d", cae);
    	    Tcl_SetResult(ip,buf,TCL_VOLATILE);
	}
    } else if ( c=='m' && strncmp(cmd,"mulerror",len)==0 ) {
	if ( aC == 3 ) {
	    if ( Tcl_GetInt(ip, aV[2], &cme) != TCL_OK ) {
	        Tcl_AppendResult(ip,"\n",aV[0],": bad value",NULL);
	        return TCL_ERROR;
	    }
	    eemParamsSet( cae, cme);
	} else {
    	    sprintf(buf,"%d", cme);
    	    Tcl_SetResult(ip,buf,TCL_VOLATILE);
	}
    } else if ( c=='d' && strncmp(cmd,"dfltfmt",len)==0 ) {
	if ( aC == 3 ) {
	    eemParamsSetDfltFmtStr( aV[2]);
	} else {
    	    Tcl_SetResult(ip,EEMParams.DfltFmtStr,TCL_VOLATILE);
	}
    } else if ( c=='c' && strncmp(cmd,"cplxfmt",len)==0 ) {
	if ( aC == 3 ) {
	    eemParamsSetCplxFmtStr( aV[2]);
	} else {
    	    Tcl_SetResult(ip,EEMParams.CplxFmtStr,TCL_VOLATILE);
	}
    } else {
	Tcl_AppendResult(ip,aV[0],": unknown command ``",cmd,"''",NULL);
	return TCL_ERROR;
    }
    return TCL_OK;
}

void
eemDfltRegisterCmds( Tcl_Interp *ip) {
    eemParamsInit();
    Tcl_CreateCommand( ip, "mathparams", _eemParamsCmds,
      (ClientData)NULL, NULL);
    Tcl_CreateCommand( ip, "dflt", _eemDfltCmds, (ClientData)NULL, NULL);
}
