#if !defined(_EEMLOCI_H_)
#define _EEMLOCI_H_ 1
/**
    eemLoci.h :: EEM Library

    Functions and data structures for
    manipulating singularities in the complex plane (poles/zeros).
**/

#if !defined(_TOPVEC_H_) && __TOP_AUTOINCLUDE__
#include "topVec.h"
#endif


#if !defined(_EEMCPLX_H_) && __TOP_AUTOINCLUDE__
#include "eemCplx.h"
#endif

#if !defined(_EEMPOLYCPLX_H_) && __TOP_AUTOINCLUDE__
#include "eemPoly.h"
#endif

typedef	    	 EEMComplex	EEMLocusPt;	

typedef struct _EEMLociGrp EEMLociGrp;
typedef struct _EEMArray EEMArray;
typedef struct _EEMLocus EEMLocus;

struct _EEMArray {
    TOPVector		Num;		    /* of double */
};

#define EEM_ARRAY_Num(pGrp)        TOPVEC_GETNUM(&(pGrp)->Num)
#define EEM_ARRAY_Get(pGrp,i)  	   TOPVEC_GET(&(pGrp)->Num,(i),double)

struct _EEMLocus {
    EEMLociGrp	        *PntGrp;
    EEMLociGrp	        *MasterGrp;
    EEMComplex		Pole;
    TOPVector		*CurLocus;
    TOPVector           LocusPos;           /* of EEMComplex */
    TOPVector		LocusNeg;	    /* of EEMComplex */ 
};


struct _EEMLociGrp {
    EEMLociGrp	        *MasterGrp;
    EEMSilarGrp	 	*SG;	
    EEMSilarGrp	 	Sing;	
    EEMPolyCplxRat 	Rat;
    TOPVector		Locus;		    /* of EEMLocus */
    EEMPolyCplx         ADiff;
    EEMPolyCplx         BDiff;
    EEMPolyCplx         KDiff;
    EEMArray		Klist;
    double		DeltaH;
    double		DiffEpsilon;
};

#define EEMLocusF_NegInf	   TOP_BIT(0)
#define EEMLocusF_PosInf	   TOP_BIT(1)
#define EEMLocusF_Mark		   TOP_BIT(2)
#define EEM_LOCUS_Num(pGrp)    	   TOPVEC_GETNUM((pGrp)->CurLocus)
#define EEM_LOCUSPOS_Num(pGrp)     TOPVEC_GETNUM(&(pGrp)->LocusPos)
#define EEM_LOCUSNEG_Num(pGrp)     TOPVEC_GETNUM(&(pGrp)->LocusNeg)
#define EEM_LOCUS_Get(pGrp,i)  	   TOPVEC_GET((pGrp)->CurLocus,(i),EEMComplex)
#define EEM_LOCUSPOS_Get(pGrp,i)   TOPVEC_GET(&(pGrp)->LocusPos,(i),EEMComplex)
#define EEM_LOCUSNEG_Get(pGrp,i)   TOPVEC_GET(&(pGrp)->LocusNeg,(i),EEMComplex)
#define EEM_LOCUS_Append(pGrp)	TOPVEC_APPEND((pGrp)->CurLocus,EEMComplex)

#if __TOP_DEBUG__
#define EEM_LOCUS_Print(pkg, pLocus, title) {  		\
    if (topMsgPkgDbgOnB(pkg) ) {			\
	eemLocusPrint(pLocus, title);			\
    }							\
}
#define EEM_LOCIGRP_Print(pkg, pLociGrp, title) {  	\
    if (topMsgPkgDbgOnB(pkg) ) {			\
	eemLociGrpPrint(pLociGrp, title);			\
    }							\
}
#define EEM_ARRAY_Print(pkg,pArray,title) {		\
    if ( topMsgPkgDbgOnB(pkg) ) {			\
	eemArrayPrint(pArray,title);			\
    }							\
}
#else
#define EEM_LOCUS_Print(pkg, pLocus, title) /* */
#define EEM_LOCIGRP_Print(pkg, pLociGrp, title) /* */
#define EEM_ARRAY_Print(pkg,pArray,title) /* */
#endif

#define EEM_LOCI_Num(pGrp)     TOPVEC_GETNUM(&(pGrp)->Locus)
#define EEM_LOCI_Get(pGrp,i)   TOPVEC_GET(&(pGrp)->Locus,(i),EEMLocus)

extern void		eemLocusPtConstructor( EEMLocusPt *pLocusPt);
extern void 		eemLocusPtDestructor( EEMLocusPt *pLocusPt);
extern void 		eemLocusConstructor( EEMLocus *pLocus, EEMLociGrp *pGrp);
extern void		eemLocusDestructor( EEMLocus *pLocus);
extern void		eemLociGrpConstructor( EEMLociGrp *pLGrp, EEMLociGrp *pMasterGrp);
extern void 		eemLociGrpDestructor( EEMLociGrp *pLGrp);
extern void		eemLociGrpSetNum( EEMLociGrp *pGrp, int n);
extern void 		eemLocusSetNum( EEMLocus *pGrp, int n);
extern void 		eemLocusPosSetNum( EEMLocus *pGrp, int n);
extern void 		eemLocusNegSetNum( EEMLocus *pGrp, int n);
extern void		eemLociGrpInsureNum( EEMLociGrp *pGrp, int n);
extern void		eemLocusInsureNum( EEMLocus *pGrp, int n);
extern void		eemLocusPosInsureNum( EEMLocus *pGrp, int n);
extern void		eemLocusNegInsureNum( EEMLocus *pGrp, int n);
extern void		eemLociGrpPrint( EEMLociGrp *pGrp, char *title);
extern void		eemLocusPrint( EEMLocus *pGrp, char *title);
extern void 		eemLocusPosPrint( EEMLocus *pGrp, char *title);
extern void		eemLocusNegPrint( EEMLocus *pGrp, char *title);

extern void		eemArrayConstructor( EEMArray *pArray );
extern void		eemArrayDestructor( EEMArray *pArray);



extern TOPLogical 	eemBSearchK(EEMLocus *pLocus, double k, EEMComplex *pOut);
extern void    		eemRootLociMaster( EEMLociGrp *pLGrp, EEMBox *pBox, EEMSilarGrp *pSGrp, double k0, TOPLogical Drag);
extern void		eemResizeBox (EEMSilarGrp *pSGprp, EEMBox *pBox);
extern void		eemStoreK(EEMSilarGrp *pSing, EEMPolyCplxRat *pRat, EEMBox *pBox);
extern void		eemSilarToArrayCompact(EEMSilarGrp *pSing, EEMArray *pKlist);
extern void		eemDrawMaster(EEMLociGrp *pLGrp, EEMSilarGrp *pSGrp, EEMBox *pBS, EEMBox *pBox);
extern TOPLogical	eemInsideBox(EEMLocusPt *pLocusPt, EEMBox *pB);
extern void		eemFindQ(EEMPolyCplxRat *pRat, EEMBox *pDB, EEMLocusPt *pLocusPt, int *q);
extern void		eemNewtonIteration(EEMLocusPt *pLocusPt,EEMLociGrp *pLGrp, double k);
extern double		eemEvalK(EEMPolyCplxRat *pRat, EEMLocusPt *pLocusPt);
extern void		eemRootLociKRange(EEMLociGrp *pLGrp, EEMLociGrp *pLGP, EEMBox *pB, double KStart, double KEnd, TOPMask InfFlag);
extern void 		eemRootLociZoomIn( EEMLociGrp *pLGrp, EEMLociGrp *pLGP, EEMBox *pB);
extern void 		eemRootLociZoomInK( EEMLociGrp *pLGrp, EEMLociGrp *pLGP, EEMBox *pB, double KStart, double KEnd);
extern void		eemDrawLocusPartK(EEMLocus *pLocus, EEMLocus *pLP, EEMBox *pBS, EEMBox *pB, double KStart, double KEnd);
extern void		eemDrawLocusPart(EEMLocus *pLocus, EEMLocus *pLP, EEMBox *pBS, EEMBox *pB, TOPLogical Sign);

extern void		eemDrawLocus( EEMLocus *pLocus, EEMLocusPt *pLocusPtNew, EEMBox *pBox, EEMBox *pDrawBox, double *pk0, double KStart, double KEnd);
extern void 		eemMeetSingAndCorrect(EEMLocusPt *pLocusPtNew, int v, EEMLocus *pLocus, EEMBox *pDB, EEMComplex *pAtemp, EEMLocusPt *pLastTwo); 

#endif /* _EEMLOCI_H_ */
