#include "topFixup.h"
#include <stdio.h>
#include <math.h>

#include "topStd.h"
#include "topMem.h"
#include "topVec.h"

#include "eemDflt.h"
#include "eemCplx.h"
#include "eemSilar.h"
#include "eemPoly.h"
#include "eemLoci.h"

char _EemLociSubPackage[] = "eem.loci";
#define SPKG _EemLociSubPackage

void
eemLocusPtConstructor( EEMLocusPt *pLocusPt){
    eemCplxZero(pLocusPt);
}	

void
eemLocusPtDestructor( EEMLocusPt *pLocusPt){
    eemCplxZero(pLocusPt);
    ;	/* this function exists just for completeness */
}

void 
eemLocusConstructor( EEMLocus *pLocus, EEMLociGrp *pGrp){
    pLocus->PntGrp = pGrp;
    pLocus->MasterGrp = pGrp->MasterGrp;
    topVecConstructor( &pLocus->LocusPos, sizeof(EEMLocusPt));
    topVecConstructor( &pLocus->LocusNeg, sizeof(EEMLocusPt));
}

void
eemLocusDestructor( EEMLocus *pLocus){
    topVecDestructor( &pLocus->LocusPos);
    topVecDestructor( &pLocus->LocusNeg);
}

void 
eemArrayConstructor( EEMArray *pArray ){
    topVecConstructor( &pArray->Num, sizeof(double));
}

void
eemArrayDestructor( EEMArray *pArray){
    topVecDestructor( &pArray->Num);
}

void
eemLociGrpConstructor( EEMLociGrp *pLGrp, EEMLociGrp *pMasterGrp) {
    pLGrp->MasterGrp =  pMasterGrp ? pMasterGrp : pLGrp;
    eemSilarGrpConstructor( &pLGrp->Sing);
    eemPolyCplxRatConstructor( &pLGrp->Rat);
    eemPolyCplxConstructor( &pLGrp->ADiff);
    eemPolyCplxConstructor( &pLGrp->BDiff);
    eemPolyCplxConstructor( &pLGrp->KDiff);
    eemArrayConstructor( &pLGrp->Klist);
    topVecConstructor( &pLGrp->Locus, sizeof(EEMLocus));
}

void
eemLociGrpDestructor( EEMLociGrp *pLGrp) { 
    eemSilarGrpDestructor( &pLGrp->Sing);
    eemPolyCplxRatDestructor( &pLGrp->Rat);
    eemPolyCplxDestructor( &pLGrp->ADiff);
    eemPolyCplxDestructor( &pLGrp->BDiff);
    eemPolyCplxDestructor( &pLGrp->KDiff);
    eemArrayDestructor( &pLGrp->Klist);
    topVecDestructor( &pLGrp->Locus );
}    

void
eemLociGrpSetNum( EEMLociGrp *pGrp, int n) {
    int i, curnum = EEM_LOCI_Num(pGrp);

    topVecSetNum( &pGrp->Locus, n);
    for (i=curnum; i < n; i++)         /* if expanding, init the new ones */
        eemLocusConstructor( &EEM_LOCI_Get(pGrp,i), pGrp);
}

void
eemLocusSetNum( EEMLocus *pGrp, int n) {
    int i, curnum = EEM_LOCUS_Num(pGrp);

    topVecSetNum( pGrp->CurLocus, n);
    for (i=curnum; i < n; i++)         /* if expanding, init the new ones */
        eemLocusPtConstructor( &EEM_LOCUS_Get(pGrp,i));
}

void
eemArraySetNum( EEMArray *pGrp, int n) {
    int i, curnum = EEM_ARRAY_Num(pGrp);
    topVecSetNum( &pGrp->Num, n);
    for (i=curnum; i< n; i++)
        EEM_ARRAY_Get(pGrp,i) = 0.0;
}
void
eemLocusPosSetNum( EEMLocus *pGrp, int n) {
    int i, curnum = EEM_LOCUSPOS_Num(pGrp);

    topVecSetNum( &pGrp->LocusPos, n);
    for (i=curnum; i < n; i++)         /* if expanding, init the new ones */
        eemLocusPtConstructor( &EEM_LOCUSPOS_Get(pGrp,i));
}

void
eemLocusNegSetNum( EEMLocus *pGrp, int n) {
    int i, curnum = EEM_LOCUSNEG_Num(pGrp);

    topVecSetNum( &pGrp->LocusNeg, n);
    for (i=curnum; i < n; i++)         /* if expanding, init the new ones */
        eemLocusPtConstructor( &EEM_LOCUSNEG_Get(pGrp,i));
}


void
eemArrayInsureNum( EEMArray *pGrp, int n) {
    if ( n > EEM_ARRAY_Num(pGrp) )
        eemArraySetNum( pGrp, n);
}
void
eemLociGrpInsureNum( EEMLociGrp *pGrp, int n) {
    if ( n > EEM_LOCI_Num(pGrp) )
        eemLociGrpSetNum( pGrp, n);
}


void
eemLocusInsureNum( EEMLocus *pGrp, int n) {
    if ( n > EEM_LOCUS_Num(pGrp) )
        eemLocusSetNum( pGrp, n);
}
void
eemLocusPosInsureNum( EEMLocus *pGrp, int n) {
    if ( n > EEM_LOCUSPOS_Num(pGrp) )
        eemLocusPosSetNum( pGrp, n);
}


void
eemLocusNegInsureNum( EEMLocus *pGrp, int n) {
    if ( n > EEM_LOCUSNEG_Num(pGrp) )
        eemLocusNegSetNum( pGrp, n);
}


void
eemArrayPrint( EEMArray *pArray, char *title) {
    int i;

    if ( title != NULL )
        printf("\n%s:\n", title);
    printf("\n");
    for (i=0; i<EEM_ARRAY_Num(pArray) ; i++)  
	printf("\t%d:%g",i, EEM_ARRAY_Get(pArray, i));
    printf("\n");
 }
void
eemLociGrpPrint( EEMLociGrp *pGrp, char *title) {
    int i;

    if ( title != NULL )
        printf("\n%s:\n", title);
    for (i=0; i<EEM_LOCI_Num(pGrp) && i < 10000 ; i++) { 
	printf("\nThis is Locus #%d\n",i);
        eemLocusPosPrint( &EEM_LOCI_Get( pGrp, i), "Positve Locus:");
        eemLocusNegPrint( &EEM_LOCI_Get( pGrp, i), "Negative Locus:");
    }
}

void
eemLocusPrint( EEMLocus *pGrp, char *title){
    int i;
    if ( title != NULL )
        printf("\n%s:\n", title);

    for (i=0; i<EEM_LOCUS_Num(pGrp) && i < 10000 ; i++) 
	printf("\n%d (%.10g, %.10g)", i, EEM_LOCUS_Get(pGrp, i).r, EEM_LOCUS_Get(pGrp, i).i);
}

void
eemLocusPosPrint( EEMLocus *pGrp, char *title){
    int i;
    if ( title != NULL )
        printf("\n%s:\n", title);
    for (i=0; i<EEM_LOCUSPOS_Num(pGrp) && i < 10000 ; i++) 
	printf("\n%d (%.10g, %.10g)", i, EEM_LOCUSPOS_Get(pGrp, i).r, EEM_LOCUSPOS_Get(pGrp, i).i);
}


void
eemLocusNegPrint( EEMLocus *pGrp, char *title){
    int i;
    if ( title != NULL )
        printf("\n%s:\n", title);
    for (i=0; i<EEM_LOCUSNEG_Num(pGrp) && i < 10000 ; i++) 
	printf("\n%d: (%.10g, %.10g)", i, EEM_LOCUSNEG_Get(pGrp, i).r, EEM_LOCUSNEG_Get(pGrp, i).i);
}
