/* 
    Author: Mei-Tjng Huang & Kennard White

    Copyright (c) 1990-93 The Regents of the University of California.
    All rights reserved.  See the file "./copyright.h" for full copyright
    notice, limitation of liability, and disclaimer of warranty provisions.
#include "copyright.h"				COPYRIGHTENDKEY
*/
#if !defined(lint) && !defined(SABER)
static char rcsid[] = "$Header: /vol/cooley/cooley1/kennard/src/eem/RCS/poly.c,v 1.24 93/08/27 14:29:33 kennard Exp $";
#endif

#include "topFixup.h"
#include <math.h>

#include "topStd.h"
#include "topMem.h"
#include "topVec.h"

#include "eemCplx.h"
#include "eemPoly.h"

char _EemPolySubPackage[] = "eem.poly";
#define SPKG _EemPolySubPackage

/**************************************************************************
 *
 *		 PolyCplx funcs
 *
 **************************************************************************/

/**
	xsdf
**/

void
eemPolyCplxConstructor( EEMPolyCplx *pPoly) {
   topVecConstructor( &pPoly->Coef, sizeof(EEMComplex));
}

void
eemPolyCplxDestructor( EEMPolyCplx *pPoly) {
   topVecDestructor( &pPoly->Coef);
}

EEMPolyCplx*
eemPolyCplxCreate() {
    EEMPolyCplx	*pDst;

    pDst = MEM_ALLOC(EEMPolyCplx);
    eemPolyCplxConstructor( pDst);
    return pDst;
}

void
eemPolyCplxDestroy( EEMPolyCplx *pPoly) {
    eemPolyCplxDestructor( pPoly);
    MEM_FREE(pPoly);
}

EEMPolyCplx*
eemPolyCplxCopy( EEMPolyCplx *pPoly) {
    EEMPolyCplx	*pNew;

    pNew = eemPolyCplxCreate();
    topVecCopy( &pNew->Coef, &pPoly->Coef);
    return pNew;
}

void
eemPolyCplxCopy2( EEMPolyCplx *pSrc, EEMPolyCplx *pDst) {
    if ( pSrc != pDst ) 
    	topVecCopy( &pDst->Coef, &pSrc->Coef);
}

void
eemPolyCplxZero( EEMPolyCplx *pPoly, int N) {
    topVecSetNum( &pPoly->Coef, N);
    TOPVEC_LOOP_BACK_BEGIN( &pPoly->Coef, EEMComplex, pVal) {
	pVal->r = 0.0; pVal->i = 0.0;
    } TOPVEC_LOOP_BACK_END();
}

void
eemPolyCplxOne( EEMPolyCplx *pPoly) {
    EEMComplex	*pCoef;
    eemPolyCplxSetNum( pPoly, 1);
    pCoef = &EEM_POLYCPLX_Get(pPoly,0);
    EEM_CPLX_SET( pCoef, 1.0, 0.0);
}

TOPLogical
eemPolyCplxZeroB( EEMPolyCplx *pPoly) {
    if ( EEM_POLYCPLX_Num(pPoly) == 0 )
	return TOP_TRUE;
    return EEM_POLYCPLX_Num(pPoly) == 1
      && eemCplxZeroB( &EEM_POLYCPLX_Get(pPoly,0));
}

TOPLogical
eemPolyCplxOneB( EEMPolyCplx *pPoly) {
    return EEM_POLYCPLX_Num(pPoly) == 1
      && eemCplxOneB( &EEM_POLYCPLX_Get(pPoly,0));
}

void
eemPolyCplxSetNum( EEMPolyCplx *pPoly, int N) {
    int	curN = EEM_POLYCPLX_Num(pPoly);
    topVecSetNum( &pPoly->Coef, N);
    TOPVEC_SUBIDX_LOOP_FORW_BEGIN( &pPoly->Coef, curN, N, i, EEMComplex, pVal) {
	pVal->r = 0.0; pVal->i = 0.0;
    } TOPVEC_SUBIDX_LOOP_FORW_END();
}

void
eemPolyCplxInsureNum( EEMPolyCplx *pPoly, int N) {
    if ( EEM_POLYCPLX_Num(pPoly) < N )
	eemPolyCplxSetNum( pPoly, N);
}

void 
eemPolyCplxShorten( EEMPolyCplx *pPoly) {
    int		useLen = 0;
    double	magsq,v, max_mag, max_magsq = 0.0;

    EEM_POLYCPLX_Print(SPKG, pPoly, "Unshortened poly");
    EEM_POLYCPLX_IDX_LOOP_BACK_BEGIN(pPoly,idx,pCoef) {
	 magsq = EEM_CPLX_MAGSQ(pCoef);
	 if ( magsq > max_magsq )
	     max_magsq = magsq;
    } EEM_POLYCPLX_IDX_LOOP_BACK_END();
    max_mag = sqrt(max_magsq);
    v = max_mag * EEMParams.Polytol;
    TOP_PDBG((SPKG, "\nIn Shorten: v=%3g",v));

    EEM_POLYCPLX_IDX_LOOP_BACK_BEGIN(pPoly,idx,pCoef) {
        if ( TOP_ABS(pCoef->r) >= v || TOP_ABS(pCoef->i) >= v ) {
	    useLen = idx+1;
	    break;
	}
    } EEM_POLYCPLX_IDX_LOOP_BACK_END();
    eemPolyCplxSetNum(pPoly, useLen);
    EEM_POLYCPLX_Print(SPKG, pPoly, "Shortened poly"); 
}      

EEMPolyCplx*
eemPolyCplxAdd( EEMPolyCplx *pSrc1, EEMPolyCplx *pSrc2) {
    EEMPolyCplx *pDst;

    pDst = eemPolyCplxCopy(pSrc1);
    eemPolyCplxAdd2(pDst, pSrc2);
    return pDst;
}

void
eemPolyCplxAdd2(EEMPolyCplx *pDst, EEMPolyCplx *pSrc) {
    int i, nsrc;

    nsrc = EEM_POLYCPLX_Num(pSrc);
    eemPolyCplxInsureNum( pDst, nsrc);
    for ( i=0; i < nsrc; i++) {
        eemCplxAdd2( &EEM_POLYCPLX_Get(pDst,i), &EEM_POLYCPLX_Get(pSrc,i));
    }
    eemPolyCplxShorten(pDst);
}

void
eemPolyCplxAdd3(EEMPolyCplx *pSrc1, EEMPolyCplx *pSrc2, EEMPolyCplx *pDst) {
    int i, n1, n2;

    /* Make Src1 the shorter vec */
    n1 = EEM_POLYCPLX_Num(pSrc1);
    n2 = EEM_POLYCPLX_Num(pSrc2);
    if ( n1 > n2 ) {
        TOP_SWAPTYPE(int, n1, n2);
        TOP_SWAPTYPE(EEMPolyCplx*, pSrc1, pSrc2);
    }
    eemPolyCplxSetNum( pDst, n2);
    for ( i=0; i < n1; i++) {
        eemCplxAdd3( &EEM_POLYCPLX_Get(pSrc1,i), &EEM_POLYCPLX_Get(pSrc2,i),
          &EEM_POLYCPLX_Get(pDst,i));
    }
    for ( ; i < n2; i++) {
        EEM_POLYCPLX_Get(pDst,i) = EEM_POLYCPLX_Get(pSrc2,i);
    }
    eemPolyCplxShorten(pDst);
}
void
eemPolyCplxSub2(EEMPolyCplx *pDst, EEMPolyCplx *pSrc) {
    eemPolyCplxSub3(pDst, pSrc, pDst);
}

void
eemPolyCplxSub3(EEMPolyCplx *pSrc1, EEMPolyCplx *pSrc2, EEMPolyCplx *pDst) {
    EEMPolyCplx pPoly;

    eemPolyCplxConstructor( &pPoly);
    eemPolyCplxMulConst3(pSrc2, -1.0, &pPoly);
    eemPolyCplxAdd3(pSrc1, &pPoly, pDst);
    eemPolyCplxShorten(pDst);
    eemPolyCplxDestructor( &pPoly);
}

void
eemPolyCplxMulConst2(EEMPolyCplx *pDst, double val) {
    eemPolyCplxMulConst3( pDst, val, pDst);
}

void
eemPolyCplxMulConst3(EEMPolyCplx *pSrc, double val, EEMPolyCplx *pDst) {
    int nsrc = EEM_POLYCPLX_Num(pSrc);

    if ( pSrc != pDst )
        eemPolyCplxSetNum( pDst, nsrc);
    EEM_POLYCPLX_IDX_LOOP_FORW_BEGIN(pSrc, i, pSrcCoef) {
        EEM_POLYCPLX_Get(pDst, i).r = pSrcCoef->r * val;
        EEM_POLYCPLX_Get(pDst, i).i = pSrcCoef->i * val;
    } EEM_POLYCPLX_IDX_LOOP_FORW_END();
    eemPolyCplxShorten(pDst);
}

void
eemPolyCplxMulConstCplx2(EEMPolyCplx *pDst, EEMComplex *pVal) {
    EEM_POLYCPLX_IDX_LOOP_FORW_BEGIN(pDst, i, pCoef) {
	eemCplxMul2( pCoef, pVal);
    } EEM_POLYCPLX_IDX_LOOP_FORW_END();
}

void
eemPolyCplxMulConstCplx3(EEMPolyCplx *pSrc, EEMComplex *pVal, EEMPolyCplx *pDst){
    int nsrc = EEM_POLYCPLX_Num(pSrc);
    if ( pSrc != pDst )
        eemPolyCplxSetNum( pDst, nsrc);
    EEM_POLYCPLX_IDX_LOOP_FORW_BEGIN(pSrc, i, pSrcCoef) {
        eemCplxMul3( pSrcCoef, pVal, &EEM_POLYCPLX_Get(pDst, i));
    } EEM_POLYCPLX_IDX_LOOP_FORW_END();
    eemPolyCplxShorten(pDst);
}

void
eemPolyCplxDotVec3( EEMPolyCplx *pSrc, TOPVector *pVec, EEMPolyCplx *pDst) {
    int N1 = EEM_POLYCPLX_Num(pSrc);
    int N2 = TOPVEC_Num(pVec);
    int N = TOP_MIN(N1,N2);

    eemPolyCplxSetNum( pDst, N);
    EEM_POLYCPLX_IDX_LOOP_FORW_BEGIN(pDst, i, pDstCoef) {
	EEMComplex	*pSrcCoef = &EEM_POLYCPLX_Get(pSrc, i);
	double		vecCoef = TOPVEC_Get(pVec,i,double);
	pDstCoef->r = pSrcCoef->r * vecCoef;
	pDstCoef->i = pSrcCoef->i * vecCoef;
    } EEM_POLYCPLX_IDX_LOOP_FORW_END();
}

void
eemPolyCplxMul2(EEMPolyCplx *pD, EEMPolyCplx *pS) {
    eemPolyCplxMul3( pS, pD, pD);
}

/**
    As always, we must allow any of the pointers to be equal.
    Note that pS1==pS2 is not a problem; but pS1==pD or pS2==pD
    require making copies.
**/
void
eemPolyCplxMul3( EEMPolyCplx *pS1_in, EEMPolyCplx *pS2_in, EEMPolyCplx *pD) {
    int		n1 = EEM_POLYCPLX_Num(pS1_in);
    int		n2 = EEM_POLYCPLX_Num(pS2_in);
    int		i, j;
    EEMPolyCplx	s1, s2, *pS1 = pS1_in, *pS2 = pS2_in;
    EEMComplex	temp;

    if ( n1==0 || n2==0 ) {
	/* not just optimzation: alg below fails */
        eemPolyCplxZero(pD, 0);
	return;
    }
	
    if ( pS1_in == pD ) {
	eemPolyCplxConstructor( &s1);
	eemPolyCplxCopy2( pS1_in, &s1);
	pS1 = &s1;
    }
    if ( pS2_in == pD ) {
	if ( pS2_in == pS1_in ) {
	    pS2 = &s1;
	} else {
	    eemPolyCplxConstructor( &s2);
	    eemPolyCplxCopy2( pS2, &s2);
	    pS2 = &s2;
	}
    }

    eemPolyCplxZero(pD, n1+n2-1);
    for (i=0; i < EEM_POLYCPLX_Num(pS1); i++) {
         for(j=0; j < EEM_POLYCPLX_Num(pS2); j++) {
             eemCplxMul3 (& EEM_POLYCPLX_Get(pS1, i),
                          & EEM_POLYCPLX_Get(pS2, j),
                          & temp);
             eemCplxAdd2(&EEM_POLYCPLX_Get(pD, i+j), &temp);
         }
    }
    if ( pS1 == &s1 )		eemPolyCplxDestructor( &s1);
    if ( pS2 == &s2 )		eemPolyCplxDestructor( &s2);
    eemPolyCplxShorten(pD);
}


/**
    Multiply {pPoly} by x^{k}.  Just a shift.  Works for any integer k.
**/
void
eemPolyCplxMulX(EEMPolyCplx *pPoly, int k) {
    int i;
    topVecShift( EEM_POLYCPLX_Vec(pPoly), k);
    if ( k > 0 ) {
	for (i=k-1; i>=0; i--) {
	    EEM_CPLX_ZERO(&EEM_POLYCPLX_Get(pPoly,i));
	}
    }
}


/**
    Copies the coef from {pPoly} into the two arrays {pR} and {pI},
    which are assumed to be large enough.  The data is "flipped" from
    the native representation, so that (pR,pI)[0] are the leading
    coeficient.  Copying begins at the coef of order {first}; that is,
    the lower order {first} coefs of {pPoly} will be dropped.
**/
void 
eemPolyCplxToArrays( EEMPolyCplx *pPoly, double *pR, double *pI, int first) {
    int i, n;

    n = EEM_POLYCPLX_Num(pPoly)-1;
    for (i=first; i<=n; i++) {
    	pR[n-i] = EEM_POLYCPLX_Get(pPoly,i).r;
    	pI[n-i] = EEM_POLYCPLX_Get(pPoly,i).i;
    }
}

void
eemPolyCplxPower2(EEMPolyCplx *pD, int power) {
    eemPolyCplxPower3( pD, power, pD);
}


/**
    As allways, we must allow pS==pD case.  Because there are so many
    special cases, this can be challengeing.
**/
void
eemPolyCplxPower3(EEMPolyCplx* pS, int power, EEMPolyCplx *pD) {
    int		i, sn;
    double	binom;
    EEMComplex	a0;

    /*IF*/ if ( power < 0 ) {
	topProgMsg("eemPolyCplxPower2: bad args: power=%d",power);
	return;
    } else if ( power == 0 ) {
	eemPolyCplxOne( pD);
	return;
    } else if ( power == 1 ) {
	eemPolyCplxCopy2( pS, pD);
	return;
    }

    sn = EEM_POLYCPLX_Num(pS);
    switch ( sn ) {
    case 0:
	eemPolyCplxZero( pD, 0);
	return;
    case 1:
	eemPolyCplxSetNum( pD, 1);
        eemCplxPow3( &EEM_POLYCPLX_Get(pS,0), power, &EEM_POLYCPLX_Get(pD, 0));
	return;
    case 2:
	if ( eemCplxOneB( &EEM_POLYCPLX_Get(pS,1)) ) {
	    a0 = EEM_POLYCPLX_Get(pS,0);
	    eemPolyCplxSetNum(pD, power+1);
	    EEM_POLYCPLX_IDX_LOOP_FORW_BEGIN( pD, di, pCoef) {
		eemCplxPow3( &a0, power-di, pCoef);
		binom = eemBinomial(power, power-di);
		pCoef->r *= binom;
		pCoef->i *= binom;
	    } EEM_POLYCPLX_IDX_LOOP_FORW_END();
	} else {
	    topAbortMsg("not implemented.");
	}
	break;
    default:
	/* XXX: brute force...theres got to be a better way */
	if ( pS == pD ) {
	    EEMPolyCplx tsrc;
	    eemPolyCplxConstructor(&tsrc);
	    eemPolyCplxCopy2( pS, &tsrc);
	    for ( i=1; i < power; i++) {
		eemPolyCplxMul2( pD, &tsrc);
	    }
	    eemPolyCplxDestructor(&tsrc);
	} else {
	    eemPolyCplxCopy2( pS, pD);
	    for ( i=1; i < power; i++) {
		eemPolyCplxMul2( pD, pS);
	    }
	}
	break;
    }
    eemPolyCplxShorten(pD);
}


/** 
  Evaluates the polynomial {pPoly} at {pVal}, returning the result in {pDst}.
**/

void
eemPolyCplxEval(EEMPolyCplx *pPoly, EEMComplex *pVal, EEMComplex *pDst) {
    EEM_CPLX_ZERO( pDst);
    EEM_POLYCPLX_IDX_LOOP_BACK_BEGIN(pPoly, i, pCoef) {
        eemCplxMul2( pDst, pVal);
	eemCplxAdd2( pDst, pCoef);
    } EEM_POLYCPLX_IDX_LOOP_BACK_END();
}

void
eemPolyCplxDiff2(EEMPolyCplx *pSrc, EEMPolyCplx *pDst) {
    int i,n;

    n  = EEM_POLYCPLX_Num(pSrc);
    if ( n == 0 ) {
	eemPolyCplxZero( pDst, 0);
	return;
    }
    if ( pSrc != pDst )
    	eemPolyCplxSetNum(pDst, n-1);
    for (i=0 ; i<n-1; i++) {
    	EEM_POLYCPLX_Get(pDst, i).r = EEM_POLYCPLX_Get(pSrc, i+1).r * (i+1);
    	EEM_POLYCPLX_Get(pDst, i).i = EEM_POLYCPLX_Get(pSrc, i+1).i * (i+1);
    }
    if ( pSrc == pDst )
	eemPolyCplxSetNum(pDst, n-1);
}

void
eemPolyCplxDiff1(EEMPolyCplx *pSD) {
    eemPolyCplxDiff2(pSD, pSD);
}


void
eemPolyCplxPrint(EEMPolyCplx *pPoly, char* title) {
    printf("%s:\t { ", title);
    EEM_POLYCPLX_LOOP_BEGIN(pPoly,pCoef) {
	eemCplxPrint( pCoef, NULL);
    } EEM_POLYCPLX_LOOP_END();
    printf("}\n");
}

void
eemPolyCplxZeroImag( EEMPolyCplx *pPoly) {
    EEM_POLYCPLX_LOOP_BEGIN(pPoly,pCoef) {
	pCoef->i = 0.0;
    } EEM_POLYCPLX_LOOP_END();
}

void
eemPolyCplxReverse( EEMPolyCplx *pPoly) {
    int		n = EEM_POLYCPLX_Num(pPoly);
    EEMComplex	*pOther;
    EEM_POLYCPLX_IDX_LOOP_FORW_BEGIN(pPoly,idx,pCoef) {
	if ( idx >= n/2 )
	    break;
	pOther = &EEM_POLYCPLX_Get(pPoly,n-idx-1);
	TOP_SWAPTYPE(EEMComplex, *pCoef, *pOther);
    } EEM_POLYCPLX_IDX_LOOP_FORW_END();
}

void
eemPolyCplxRI2MP( EEMPolyCplx *pRI, EEMPolyCplx *pMP) {
    topVecSetNum( &pMP->Coef, EEM_POLYCPLX_Num(pRI));
    EEM_POLYCPLX_IDX_LOOP_FORW_BEGIN(pRI,idx,pRIval) {
	EEMComplex *pMPval = &EEM_POLYCPLX_Get(pMP,idx);
	eemCplxMagPhase( pMPval, pRIval);
    } EEM_POLYCPLX_IDX_LOOP_FORW_END();
}


/**************************************************************************
 *
 *		 PolyCplxRat funcs
 *
 **************************************************************************/

void
eemPolyCplxRatConstructor( EEMPolyCplxRat *pRat) {
    eemPolyCplxConstructor( &pRat->Whole);
    eemPolyCplxConstructor( &pRat->Numer);
    eemPolyCplxConstructor( &pRat->Denom);
}

void
eemPolyCplxRatDestructor( EEMPolyCplxRat *pRat) {
    eemPolyCplxDestructor( &pRat->Whole);
    eemPolyCplxDestructor( &pRat->Numer);
    eemPolyCplxDestructor( &pRat->Denom);
}

void
eemPolyCplxRatCopy(EEMPolyCplxRat *pDst, EEMPolyCplxRat *pSrc) {
    eemPolyCplxSetNum(&pDst->Numer, EEM_POLYCPLX_Num(&pSrc->Numer));
    eemPolyCplxSetNum(&pDst->Denom, EEM_POLYCPLX_Num(&pSrc->Denom));
    eemPolyCplxSetNum(&pDst->Whole, EEM_POLYCPLX_Num(&pSrc->Whole));

    eemPolyCplxCopy2(&pSrc->Numer, &pDst->Numer);
    eemPolyCplxCopy2(&pSrc->Denom, &pDst->Denom);
    eemPolyCplxCopy2(&pSrc->Whole, &pDst->Whole);
}

void
eemPolyCplxRatEval(EEMPolyCplxRat *pRat, EEMComplex *pIn, EEMComplex *pOut) {
    EEMComplex n,d;
    eemPolyCplxEval(&pRat->Numer, pIn, &n);
    eemPolyCplxEval(&pRat->Denom, pIn, &d);
    eemCplxDiv3(&n, &d, pOut);
}


void eemPolyCplxRatDiff1(EEMPolyCplxRat *pDst) {
    eemPolyCplxRatDiff2( pDst, pDst);
}


/**
   As always, we must allows for pSrc==pDst.  By doing operations
   is the correct order, we can avoid extra temporaries.
**/
void
eemPolyCplxRatDiff2(EEMPolyCplxRat *pSrc, EEMPolyCplxRat *pDst) {
    EEMPolyCplx nDiff, dDiff, P1, P2;

    eemPolyCplxRatNormDen(pSrc);

    eemPolyCplxConstructor( &nDiff);
    eemPolyCplxConstructor( &dDiff);
    eemPolyCplxDiff2( &pSrc->Numer, &nDiff);
    eemPolyCplxDiff2( &pSrc->Denom, &dDiff);

    eemPolyCplxConstructor( &P1);
    eemPolyCplxConstructor( &P2);
    eemPolyCplxMul3( &pSrc->Denom, &nDiff, &P1);
    eemPolyCplxMul3( &pSrc->Numer, &dDiff, &P2);
    eemPolyCplxSub3( &P1, &P2, &pDst->Numer);
    eemPolyCplxDestructor( &P1);
    eemPolyCplxDestructor( &P2);

    eemPolyCplxPower3( &pSrc->Denom, 2, &pDst->Denom);

    /* eemPolyCplxRatNormDen( pDst); */

    eemPolyCplxDestructor( &nDiff);
    eemPolyCplxDestructor( &dDiff);
}

void
eemPolyCplxRatNormDen(EEMPolyCplxRat *pDst) {
    int		n;
    EEMComplex	lead;

    eemPolyCplxShorten( &pDst->Denom);	/* this is crucial */
    n = EEM_POLYCPLX_Num(&pDst->Denom);
    if ( n==0 )
	return;
    lead = EEM_POLYCPLX_Get(&pDst->Denom,n-1);
    if ( ! eemCplxOneB(&lead) ) {
	eemCplxInv1( &lead);
        eemPolyCplxMulConstCplx2(&pDst->Numer, &lead);
        eemPolyCplxMulConstCplx2(&pDst->Denom, &lead);
    }
}



/**
    Make {pRat} a proper rational polynomial.  {pRat.Denom} must
    have have non-zero leading cooef.  The denom will not be modified.
    The resulting polynomial will be ``proper'', meaning either:
    - The order of the numerator will be strictly less than the denominator; or,
    - The numerator will be zero.  (This case can be views as
      a 0 order numer with a 1 order denom).
**/
void 
eemPolyCplxRatProperize(EEMPolyCplxRat *pRat) {
    int		d = EEM_POLYCPLX_Num(&pRat->Denom);
    int		n = EEM_POLYCPLX_Num(&pRat->Numer);
    int		i;
    EEMComplex	dlead, scaleval;
    EEMPolyCplx	whole;

    if ( d == 0 ) {
	topAbortMsg("eemPolyCplxRatProperize: denom is zero");
	return;
    }
    dlead = EEM_POLYCPLX_Get(&pRat->Denom,d-1);
    if ( eemCplxZeroB(&dlead) ) {
	topAbortMsg("eemPolyCplxRatProperize: leading denom cooef is zero");
	return;
    }
    if ( n < d || eemPolyCplxZeroB(&pRat->Numer) )
	return;
    if ( d == 1 ) {
	/*
	 * Denominator is just a scaling factor.  Multiply it through,
	 * and then move into whole part.
	 */
	if ( ! eemPolyCplxOneB(&pRat->Denom) ) {
	    eemCplxInv1( &dlead);
	    eemPolyCplxMulConstCplx2( &pRat->Numer, &dlead);
	}
	eemPolyCplxAdd2( &pRat->Whole, &pRat->Numer);
	eemPolyCplxZero( &pRat->Numer, 0);
	return;
    }

    /*
     * Standard Long Division
     * Must do exactly n-d+1 steps to obtain proper fraction, generating
     * whole-parts with powers x^(n-d)...x^0.  (Just before the last step,
     * the order of the denom will equal the order of the remainder.)
     * Note that {pRat.Numer} becomes the remainder.
     */
    eemPolyCplxConstructor(&whole);
    eemPolyCplxZero(&whole, n-d+1);
    for ( i=0; i < n-d+1; i++) {
	eemCplxDiv3( &EEM_POLYCPLX_Get(&pRat->Numer, n-i-1), &dlead, &scaleval);
	EEM_POLYCPLX_Get(&whole,n-d-i) = scaleval;
	/*
	 * Do a division step: remain -= scaleval * pRat->Denom
	 * The number of terms affected is controled by the size of
	 * the denominator (which is smaller than the remainder).
	 */
	EEM_POLYCPLX_IDX_LOOP_FORW_BEGIN( &pRat->Denom, di, pDenomCoef) {
	    EEMComplex temp1;
	    eemCplxMul3( pDenomCoef, &scaleval, &temp1);
	    eemCplxSub2( &EEM_POLYCPLX_Get(&pRat->Numer, n-d-i+di), &temp1);
	} EEM_POLYCPLX_IDX_LOOP_FORW_END();
	/*
	 * At this point, remain[n-i-1] should be very close to zero.
	 * In future, might want to compute an error measure here.
	 */
    }
    eemPolyCplxAdd2( &pRat->Whole, &whole);
    eemPolyCplxDestructor(&whole);

    eemPolyCplxSetNum( &pRat->Numer, d-1);
    eemPolyCplxShorten( &pRat->Numer);
}

void
eemPolyCplxRatPrint(EEMPolyCplxRat *pRat, char *title) {
    if ( title != NULL )
        printf("%s:\n", title);
    eemPolyCplxPrint(&pRat->Numer, "Numer");
    eemPolyCplxPrint(&pRat->Denom, "Denom");
    eemPolyCplxPrint(&pRat->Whole, "Whole");
}
