/* 
    Author: Mei-Tjng Huang & Kennard White

    Copyright (c) 1990-93 The Regents of the University of California.
    All rights reserved.  See the file "./copyright.h" for full copyright
    notice, limitation of liability, and disclaimer of warranty provisions.
#include "copyright.h"				COPYRIGHTENDKEY
*/
#if !defined(lint) && !defined(SABER)
static char rcsid[] = "$Header: /vol/cooley/cooley1/kennard/src/eem/RCS/polyTcl.c,v 1.13 93/08/27 14:29:31 kennard Exp $";
#endif

#include "topFixup.h"
#include <stdio.h>
#include <math.h>
#include <tcl.h>

#include "topStd.h"
#include "topMem.h"
#include "topVec.h"

#include "eemDflt.h"
#include "eemCplx.h"
#include "eemSilar.h"
#include "eemPoly.h"
#include "eemPfe.h"



/************************************************************************
 *	PolyCplx funcs (Fmt,Prs,Cmd)
 ************************************************************************/

int
eemPolyCplxFmtTcl( Tcl_Interp *ip, EEMPolyCplx *pPoly, TOPLogical nestB, 
  char *fmtStr) {
    if ( eemPolyCplxZeroB( pPoly) ) {
	EEMComplex	zero;
	EEM_CPLX_ZERO( &zero);
	eemCplxFmtTcl( ip, &zero, TOP_TRUE, fmtStr);
	return TCL_OK;
    }
    if ( nestB )	Tcl_AppendResult(ip," {",NULL);
    EEM_POLYCPLX_LOOP_BEGIN( pPoly, pCoef) {
	eemCplxFmtTcl( ip, pCoef, TOP_TRUE, fmtStr);
    } EEM_POLYCPLX_LOOP_END();
    if ( nestB )	Tcl_AppendResult(ip,"}",NULL);
    return TCL_OK;
}

int
eemPolyCplxPrsTcl( Tcl_Interp *ip, EEMPolyCplx *pPoly, int aC, char **aV) {
    int		i;

    eemPolyCplxZero( pPoly, aC);
    if ( aC == 0 ) {
	Tcl_AppendResult(ip,"\nPolyCplx format: c0 c1 c2 ... cN",NULL);
	return TCL_ERROR;
    }
    for (i=0; i < aC; i++) {
	if ( eemCplxStrPrsTcl( ip, aV[i], &EEM_POLYCPLX_Get(pPoly,i)) != TCL_OK ) {
	    Tcl_AppendResult(ip,"\nPolyCplx: bad term: ``",aV[i],"''",NULL);
	    return TCL_ERROR;
	}
    }
    return TCL_OK;
}

int
eemPolyCplxStrPrsTcl( Tcl_Interp *ip, EEMPolyCplx *pPoly, char *polyStr) {
    int		sC;
    char	**sV;

    if ( Tcl_SplitList( ip, polyStr, &sC, &sV) != TCL_OK ) {
    	Tcl_AppendResult(ip,"\nPolyCplx: bad list: ``",polyStr,"''",NULL);
	return TCL_ERROR;
    }
    if ( eemPolyCplxPrsTcl( ip, pPoly, sC, sV) != TCL_OK ) {
	Tcl_AppendResult(ip,"\nPolyCplx: bad poly: ``",polyStr,"''",NULL);
	free( (char*)sV);
	return TCL_ERROR;
    }
    free( (char*)sV);
    return TCL_OK;
}

static int
_eemPolyCplxCmds( ClientData cld, Tcl_Interp *ip, int aC, char **aV) {
    int		i;
    EEMPolyCplx	src, dst;
    char	*cmd = aV[1];
    int		c, len;
    int		r = TCL_OK;
    char	buf[10];

    if ( aC < 2 ) {
	Tcl_AppendResult(ip,"usage: ",aV[0]," add|derivative ?options?",NULL);
	return TCL_ERROR;
    }

    eemPolyCplxConstructor( &dst);
    c = *cmd; len = strlen(cmd);
    /*IF*/ if ( c == 'i' && strncmp(cmd,"identity",len)==0 ) {
	if ( aC != 3 ) {
	    Tcl_AppendResult(ip,"usage: ",aV[0]," ",aV[1]," poly",NULL);
	    goto error;
	}
	if ( eemPolyCplxStrPrsTcl( ip, &dst, aV[2]) != TCL_OK ) {
	    Tcl_AppendResult(ip,"\n",aV[0]," ",aV[1],": bad poly",NULL);
	    goto error;
	}
	eemPolyCplxFmtTcl( ip, &dst, TOP_FALSE, NULL);
    } else if ( c == 'a' && strncmp(cmd,"add",len)==0 ) {
	for (i=2; i < aC; i++) {
	    eemPolyCplxConstructor( &src);
	    if ( eemPolyCplxStrPrsTcl( ip, &src, aV[i]) != TCL_OK ) {
		sprintf( buf, "%d", i-2);
		Tcl_AppendResult(ip,"\n",aV[0]," ",aV[1],": bad poly #",buf,
		  NULL);
		goto error;
	    }
	    eemPolyCplxAdd2( &dst, &src);
	    eemPolyCplxDestructor( &src);
	}
	eemPolyCplxFmtTcl( ip, &dst, TOP_FALSE, NULL);
    } else if ( c == 'f' && strncmp(cmd,"factor",len)==0 ) {
	EEMSilarGrp	sg;
	if ( aC != 3 ) {
	    Tcl_AppendResult(ip,"usage: ",aV[0]," ",cmd," poly",NULL);
	    goto error;
	}
	if ( eemPolyCplxStrPrsTcl( ip, &dst, aV[2]) != TCL_OK ) {
	    Tcl_AppendResult(ip,"\n",aV[0]," ",aV[1],": bad poly",NULL);
	    goto error;
	}
	eemSilarGrpConstructor( &sg);
	eemPolyCplxToSilarGrp( &dst, &sg.gain, &sg, 1);
	eemSilarGrpFmtTcl( ip, &sg, TOP_FALSE);
	eemSilarGrpDestructor( &sg);
    } else if ( c == 'm' && strncmp(cmd,"mul",len)==0 ) {
        eemPolyCplxOne( &dst);
	for (i=2; i < aC; i++) {
	    eemPolyCplxConstructor( &src);
	    if ( eemPolyCplxStrPrsTcl( ip, &src, aV[i]) != TCL_OK ) {
		sprintf( buf, "%d", i-2);
		Tcl_AppendResult(ip,"\n",aV[0]," ",aV[1],": bad poly #",buf,
		  NULL);
		goto error;
	    }
	    eemPolyCplxMul2( &dst, &src);
	    eemPolyCplxDestructor( &src);
	}
	eemPolyCplxFmtTcl( ip, &dst, TOP_FALSE, NULL);
    } else if ( c == 's' && strncmp(cmd,"sub",len)==0 ) {
	if ( aC != 4 ) {
	    Tcl_AppendResult(ip,"usage: ",aV[0]," ",aV[1]," poly1 poly2",NULL);
	    goto error;
	}
	if ( eemPolyCplxStrPrsTcl( ip, &dst, aV[2]) != TCL_OK ) {
	    Tcl_AppendResult(ip,"\n",aV[0]," ",aV[1],": bad poly #1",NULL);
	    goto error;
	}
        eemPolyCplxConstructor( &src);
	if ( eemPolyCplxStrPrsTcl( ip, &src, aV[3]) != TCL_OK ) {
	    Tcl_AppendResult(ip,"\n",aV[0]," ",aV[1],": bad poly #2",NULL);
	    eemPolyCplxDestructor( &src);
	    goto error;
	}
	eemPolyCplxSub2( &dst, &src);
	eemPolyCplxDestructor( &src);
	eemPolyCplxFmtTcl( ip, &dst, TOP_FALSE, NULL);
    } else if ( (c == 'd' && strncmp(cmd,"derivative",len)==0)
             || (c == 'd' && strncmp(cmd,"differentiate",len)==0) ) {
	if ( aC != 3 ) {
	    Tcl_AppendResult(ip,"usage: ",aV[0]," ",aV[1]," poly");
	    goto error;
	}
	if ( eemPolyCplxStrPrsTcl( ip, &dst, aV[2]) != TCL_OK ) {
	    Tcl_AppendResult(ip,"\n",aV[0]," ",aV[1], ": bad poly: ``",aV[2],
	      "''",NULL);
	    goto error;
	}
	eemPolyCplxDiff1( &dst);
	eemPolyCplxFmtTcl( ip, &dst, TOP_FALSE, NULL);
    } else {
	Tcl_AppendResult(ip,aV[0],": bad command: ``",cmd,"''",NULL);
	goto error;
    }
/*ok:*/
    eemPolyCplxDestructor( &dst);
    return r;
error:
    eemPolyCplxDestructor( &dst);
    return TCL_ERROR;
}

/************************************************************************
 *	PolyCplxRat funcs (Fmt,Prs,Cmd)
 ************************************************************************/

int
eemPolyCplxRatFmtTcl( Tcl_Interp *ip, EEMPolyCplxRat *pRat, TOPLogical nestB) {
    if ( nestB )	Tcl_AppendResult(ip,"{",NULL);
    eemPolyCplxFmtTcl( ip, &pRat->Numer, TOP_TRUE, NULL);
    eemPolyCplxFmtTcl( ip, &pRat->Denom, TOP_TRUE, NULL);
    if ( ! eemPolyCplxZeroB(&pRat->Whole) ) {
        eemPolyCplxFmtTcl( ip, &pRat->Whole, TOP_TRUE, NULL);
    }
    if ( nestB )	Tcl_AppendResult(ip,"}",NULL);
    return TCL_OK;
}

int
eemPolyCplxRatPrsTcl( Tcl_Interp *ip, EEMPolyCplxRat *pRat, int aC, char **aV) {
    if ( aC != 1 && aC != 2 && aC != 3 ) {
	Tcl_AppendResult(ip,"\nPolyCplxRat format: ",
	  "numerPoly ?denomPoly? ?wholePoly?",NULL);
	return TCL_ERROR;
    }
    if ( eemPolyCplxStrPrsTcl( ip, &pRat->Numer, aV[0]) != TCL_OK ) {
	Tcl_AppendResult(ip,"\nPolyCplxRat: bad numerPoly: ``",aV[0],"''",NULL);
	return TCL_ERROR;
    }
    if ( aC >= 2 ) {
	if ( eemPolyCplxStrPrsTcl( ip, &pRat->Denom, aV[1]) != TCL_OK ) {
	    Tcl_AppendResult(ip,"\nPolyCplxRat: bad denomPoly: ``",aV[1],"''",
	      NULL);
	    return TCL_ERROR;
	}
    } else {
        eemPolyCplxOne( &pRat->Denom);
    }
    if ( aC >= 3 ) {
	if ( eemPolyCplxStrPrsTcl( ip, &pRat->Whole, aV[1]) != TCL_OK ) {
	    Tcl_AppendResult(ip,"\nPolyCplxRat: bad wholePoly: ``",aV[1],"''",
	      NULL);
	    return TCL_ERROR;
	}
    } else {
        eemPolyCplxZero( &pRat->Whole, 0);
    }
    return TCL_OK;
}

int
eemPolyCplxRatStrPrsTcl( Tcl_Interp *ip, EEMPolyCplxRat *pRat, char *ratStr) {
    int		sC;
    char	**sV;

    if ( Tcl_SplitList( ip, ratStr, &sC, &sV) != TCL_OK ) {
    	Tcl_AppendResult(ip,"\nPolyCplxRat: bad list: ``",ratStr,"''",NULL);
	return TCL_ERROR;
    }
    if ( eemPolyCplxRatPrsTcl( ip, pRat, sC, sV) != TCL_OK ) {
	Tcl_AppendResult(ip,"\nPolyCplxRat: bad ratpoly: ``",ratStr,"''",NULL);
	free( (char*)sV);
	return TCL_ERROR;
    }
    free( (char*)sV);
    return TCL_OK;
}

static int
_eemPolyCplxRatCmds( ClientData cld, Tcl_Interp *ip, int aC, char **aV) {
    EEMPolyCplxRat	dst;
    char		*cmd = aV[1];
    int			c, len;
    int			r = TCL_OK;

    if ( aC < 2 ) {
	Tcl_AppendResult(ip,"usage: ",aV[0]," identity|derivative ?options?",
	  NULL);
	return TCL_ERROR;
    }

    eemPolyCplxRatConstructor( &dst);
    c = *cmd; len = strlen(cmd);
    /*IF*/ if ( c == 'i' && strncmp(cmd,"identity",len)==0 ) {
	if ( aC != 3 ) {
	    Tcl_AppendResult(ip,"usage: ",aV[0]," ",aV[1]," ratpoly");
	    goto error;
	}
	if ( eemPolyCplxRatStrPrsTcl( ip, &dst, aV[2]) != TCL_OK ) {
	    Tcl_AppendResult(ip,"\n",aV[0]," ",aV[1],": bad ratpoly",NULL);
	    goto error;
	}
	eemPolyCplxRatFmtTcl( ip, &dst, TOP_FALSE);
#ifdef notdef
    } else if ( c == 'p' && strncmp(cmd,"pfe",len)==0 ) {
	if ( aC != 3 ) {
	    Tcl_AppendResult(ip,"usage: ",aV[0]," ",aV[1]," ratpoly");
	    goto error;
	}
	if ( eemPolyCplxRatStrPrsTcl( ip, &dst, aV[2]) != TCL_OK ) {
	    Tcl_AppendResult(ip,"\n",aV[0]," ",aV[1],": bad ratpoly",NULL);
	    goto error;
	}
	eemPolyCplxRatToPfe( &dst, NULL);
#endif
    } else if ( (c == 'd' && strncmp(cmd,"derivative",len)==0)
             || (c == 'd' && strncmp(cmd,"differentiate",len)==0) ) {
	if ( aC != 3 ) {
	    Tcl_AppendResult(ip,"usage: ",aV[0]," ",aV[1]," ratpoly");
	    goto error;
	}
	if ( eemPolyCplxRatStrPrsTcl( ip, &dst, aV[2]) != TCL_OK ) {
	    Tcl_AppendResult(ip,"\n",aV[0]," ",aV[1],": bad ratpoly",NULL);
	    goto error;
	}
	eemPolyCplxRatDiff1( &dst);
	eemPolyCplxRatFmtTcl( ip, &dst, TOP_FALSE);
    } else if ( (c == 'p' && strncmp(cmd,"proper",len)==0) ) {
	if ( aC != 3 ) {
	    Tcl_AppendResult(ip,"usage: ",aV[0]," ",aV[1]," ratpoly");
	    goto error;
	}
	if ( eemPolyCplxRatStrPrsTcl( ip, &dst, aV[2]) != TCL_OK ) {
	    Tcl_AppendResult(ip,"\n",aV[0]," ",aV[1],": bad ratpoly",NULL);
	    goto error;
	}
	eemPolyCplxRatProperize( &dst);
	eemPolyCplxRatFmtTcl( ip, &dst, TOP_FALSE);
    } else {
	Tcl_AppendResult(ip,aV[0],": bad command: ``",cmd,"''",NULL);
	goto error;
    }
/*ok:*/
    eemPolyCplxRatDestructor( &dst);
    return r;
error:
    eemPolyCplxRatDestructor( &dst);
    return TCL_ERROR;
}

/************************************************************************
 *	Pfe funcs (Fmt,Prs,Cmd)
 ************************************************************************/

int
eemPfeFmtTcl( Tcl_Interp *ip, EEMPfe *pTerm, TOPLogical nestB) {
    char	buf[10];

    if ( nestB )	Tcl_AppendResult(ip," {",NULL);
    eemCplxFmtTcl( ip, &pTerm->gain, TOP_TRUE, NULL);
    eemCplxFmtTcl( ip, &pTerm->pole, TOP_TRUE, NULL);
    sprintf( buf, " %d %d", pTerm->power, pTerm->numerpow);
    Tcl_AppendResult(ip,buf,NULL);
    if ( nestB )	Tcl_AppendResult(ip,"}",NULL);
    return TCL_OK;
}


int
eemPfeGrpFmtTcl( Tcl_Interp *ip, EEMPfeGrp *pGrp, TOPLogical nestB) {
    if ( nestB )	Tcl_AppendResult(ip," {",NULL);
    eemPolyCplxFmtTcl( ip, &pGrp->poly, TOP_TRUE, NULL);
    EEM_PFE_LOOP_BEGIN(pGrp,pPfe) {
	if ( pPfe->power == 0 )
	    continue;
    	eemPfeFmtTcl( ip, pPfe, TOP_TRUE);
    } EEM_PFE_LOOP_END();
    if ( nestB )	Tcl_AppendResult(ip," }",NULL);
    return TCL_OK;
}

int
eemPfePrsTcl( Tcl_Interp *ip, EEMPfe *pPfe, int aC, char **aV) {
    if ((aC != 3 && aC != 4) || strcmp(aV[0],"help")==0 ) {
	Tcl_AppendResult(ip, "PfeFormat: gain pole power", NULL);
	return TCL_ERROR;
    }
    if ( eemCplxStrPrsTcl( ip, aV[0], &pPfe->gain) != TCL_OK ) {
   	Tcl_AppendResult( ip, "\nPfeTerm: bad gain", NULL);
    	return TCL_ERROR;
    }
    if ( eemCplxStrPrsTcl( ip, aV[1], &pPfe->pole) != TCL_OK ) {
   	Tcl_AppendResult( ip, "\nPfeTerm: bad pole loc", NULL);
    	return TCL_ERROR;
    }
    if ( Tcl_GetInt( ip, aV[2], &pPfe->power) != TCL_OK ) {
   	Tcl_AppendResult( ip, "\nPfeTerm: bad power", NULL);
    	return TCL_ERROR;
    }
    pPfe->numerpow = 0;
    if ( aC >= 4 ) {
	if ( Tcl_GetInt( ip, aV[3], &pPfe->numerpow) != TCL_OK ) {
	    Tcl_AppendResult( ip, "\nPfeTerm: bad numer power", NULL);
	    return TCL_ERROR;
	}
    }
    return TCL_OK;
}

int
eemPfeStrPrsTcl( Tcl_Interp *ip, EEMPfe *pPfe, char *pfeStr) {
    int			sC;
    char		**sV;

    if ( Tcl_SplitList( ip, pfeStr, &sC, &sV) != TCL_OK ) {
	Tcl_AppendResult(ip,"\nPfeTermStr: bad list: ``",pfeStr,"''",NULL);
	return TCL_ERROR;
    }
    if ( eemPfePrsTcl( ip, pPfe, sC, sV) != TCL_OK ) {
	Tcl_AppendResult(ip,"\nPfeTermStr: bad silar: ``",pfeStr,"''",NULL);
        free( (char*)sV);
	return TCL_ERROR;
    }
    free( (char*)sV);
    return TCL_OK;
}

int
eemPfeGrpPrsTcl( Tcl_Interp *ip, EEMPfeGrp *pGrp, int aC, char **aV) {
    char	buf[100];

    if ( aC < 1 ) {
	Tcl_AppendResult(ip,"PfeGrp format: poly pfeterm ...",NULL);
	return TCL_ERROR;
    }
    if ( eemPolyCplxStrPrsTcl( ip, &pGrp->poly, aV[0]) != TCL_OK ) {
	Tcl_AppendResult( ip, "\nPfeGrp: bad gain", NULL);
	return TCL_ERROR;
    }
    eemPfeGrpSetNum( pGrp, aC-1);
    EEM_PFE_IDX_LOOP_FORW_BEGIN( pGrp, i, pPfeTerm) {
	if ( eemPfeStrPrsTcl( ip, pPfeTerm, aV[i+1]) != TCL_OK) {
	    sprintf( buf, "%d", i+1);
	    Tcl_AppendResult(ip,"\nPfeGrp: bad pfeterm #",buf,NULL);
	    return TCL_ERROR;
	}
    } EEM_PFE_IDX_LOOP_FORW_END();
    return TCL_OK;
}

int
eemPfeGrpStrPrsTcl( Tcl_Interp *ip, EEMPfeGrp *pPfe, char *pfeStr) {
    int		sC;
    char	**sV;

    if ( Tcl_SplitList( ip, pfeStr, &sC, &sV) != TCL_OK ) {
    	Tcl_AppendResult(ip,"\nPFE Grp: bad list: ``",pfeStr,"''",NULL);
	return TCL_ERROR;
    }
    if ( eemPfeGrpPrsTcl( ip, pPfe, sC, sV) != TCL_OK ) {
	Tcl_AppendResult(ip,"\nPFE Grp: bad ratpoly: ``",pfeStr,"''",NULL);
	free( (char*)sV);
	return TCL_ERROR;
    }
    free( (char*)sV);
    return TCL_OK;
}

static int
_eemSilarCmds( ClientData cld, Tcl_Interp *ip, int aC, char **aV) {
    EEMSilarGrp		dst;
    char		*cmd = aV[1];
    int			c, len;
    int			r = TCL_OK;
    EEMPfeGrp          pfe;
    if ( aC < 2 ) {
	Tcl_AppendResult(ip,"usage: ",aV[0]," identity|pfe ?options?",
	  NULL);
	return TCL_ERROR;
    }

    eemSilarGrpConstructor( &dst);
    eemPfeGrpConstructor(&pfe);
    c = *cmd; len = strlen(cmd);
    /*IF*/ if ( c == 'i' && strncmp(cmd,"identity",len)==0 ) {
	if ( aC != 3 ) {
	    Tcl_AppendResult(ip,"usage: ",aV[0]," ",aV[1]," silarGrp",NULL);
	    goto error;
	}
	if ( eemSilarGrpStrPrsTcl( ip, &dst, aV[2]) != TCL_OK ) {
	    Tcl_AppendResult(ip,"\n",aV[0]," ",aV[1],": bad silarGrp",NULL);
	    goto error;
	}
	eemSilarGrpFmtTcl( ip, &dst, TOP_FALSE);
    } else if ( c == 'p' && strncmp(cmd,"pfe",len)==0 ) {
	if ( aC != 3 ) {
	    Tcl_AppendResult(ip,"usage: ",aV[0]," ",aV[1]," silarGrp",NULL);
	    goto error;
	}
	if ( eemSilarGrpStrPrsTcl( ip, &dst, aV[2]) != TCL_OK ) {
	    Tcl_AppendResult(ip,"\n",aV[0]," ",aV[1],": bad silarGrp",NULL);
	    goto error;
	}
	eemSilarGrpToPfeGrp( &dst, &pfe, 0);
        eemPfeGrpFmtTcl( ip, &pfe, TOP_FALSE);
    } else if ( c == 'i' && strncmp(cmd,"invpfe",len)==0 ) {
	int numzeros = -1;
	if ( (aC != 3) && (aC!=4) ) {
	    Tcl_AppendResult(ip,"usage: ",aV[0]," ",cmd," pfeGrp ?numzeros?",
	      NULL);
	    goto error;
	}
	if ( eemPfeGrpStrPrsTcl( ip, &pfe, aV[2]) != TCL_OK ) {
	    Tcl_AppendResult(ip,"\n",aV[0]," ",cmd,": bad pfeGrp",NULL);
	    goto error;
	}
	if ( aC>=4 ) {
	    if ( Tcl_GetInt( ip, aV[3], &numzeros) != TCL_OK ) {
		Tcl_AppendResult( ip, "\n",aV[0]," ",cmd,": bad numzeros",NULL);
		goto error;
	    }
	}
	eemPfeGrpToSilarGrp( &pfe, &dst, numzeros);
        eemSilarGrpFmtTcl( ip, &dst, TOP_FALSE);
    } else {
	Tcl_AppendResult(ip,aV[0],": bad command: ``",cmd,"''",NULL);
	goto error;
    }
/*ok:*/
    eemSilarGrpDestructor( &dst);
    eemPfeGrpDestructor(&pfe);
    return r;
error:
    eemSilarGrpDestructor( &dst);
    eemPfeGrpDestructor(&pfe);
    return TCL_ERROR;
}

int
eemPolyCplxRegisterCmds( Tcl_Interp *ip) {
    Tcl_CreateCommand( ip, "poly", _eemPolyCplxCmds,
      (ClientData)NULL, NULL);
    Tcl_CreateCommand( ip, "ratpoly", _eemPolyCplxRatCmds,
      (ClientData)NULL, NULL);
    Tcl_CreateCommand( ip, "silar", _eemSilarCmds,
      (ClientData)NULL, NULL);
    return TCL_OK;
}
