/* 
    Author: Kennard White

    Copyright (c) 1990-93 The Regents of the University of California.
    All rights reserved.  See the file "./copyright.h" for full copyright
    notice, limitation of liability, and disclaimer of warranty provisions.
#include "copyright.h"				COPYRIGHTENDKEY
*/
#if !defined(lint) && !defined(SABER)
static char rcsid[] = "$Header: /vol/cooley/cooley1/kennard/src/eem/RCS/silarTcl.c,v 1.14 93/08/27 14:29:27 kennard Exp $";
#endif

#include "topFixup.h"
#include <tcl.h>

#include "topStd.h"
#include "topMem.h"
#include "topVec.h"
#include "topBuf.h"
#include "topTcl.h"

#include "eemDflt.h"
#include "eemCplx.h"
#include "eemSilar.h"

extern char _EemSilarSubPackage[];
#define SPKG _EemSilarSubPackage


/*****************************************************************************
 *
 *		EEMSilar
 *
 *****************************************************************************/

void
eemSilarFmtTcl( Tcl_Interp *ip, EEMSilar *pSilar, TOPLogical nestB) {
    char	buf[10];

    if ( nestB )		Tcl_AppendResult(ip, " {", NULL);
    eemCplxFmtTcl( ip, &pSilar->loc, TOP_TRUE, NULL);
    sprintf( buf, " %d ", pSilar->power);
    Tcl_AppendResult( ip, buf,
      TOP_IsOn(pSilar->flags,EEMSilarF_Conj) ? "conj " : "",
      TOP_IsOn(pSilar->flags,EEMSilarF_Sel) ? "sel " : "",
      NULL);
    if ( nestB )		Tcl_AppendResult(ip, "}", NULL);
}

#define EEMSilarF_Modes (EEMSilarF_Conj|EEMSilarF_Sel)
#define EEMSilarC_Modes (EEMSilarC_Exist|EEMSilarC_Sel)


static int
_eemSilarFlagsPrs( Tcl_Interp *ip, EEMSilarFlags *pMask, int aC, char **aV) {
    int		i;

    for (i=0; i < aC; i++) {
	char *flgStr = aV[i];
	TOPLogical NegateB = TOP_FALSE;
	TOPMask flgBit = 0;
	if ( *flgStr == '!' ) {
	    ++flgStr;	NegateB = TOP_TRUE;
	}
	/*IF*/ if ( strcasecmp( flgStr, "reset")==0 ) {
	    *pMask &= ~(EEMSilarF_Modes);
	    *pMask |= EEMSilarC_Modes;
	} else if ( strcasecmp( flgStr, "conj")==0 ) {
	    flgBit = EEMSilarF_Conj;
	    *pMask |= EEMSilarC_Exist;
	} else if ( strcasecmp( flgStr, "select")==0
	         || strcasecmp( flgStr, "sel")==0 ) {
	    flgBit = EEMSilarF_Sel;
	    *pMask |= EEMSilarC_Sel;
	} else {
	    Tcl_AppendResult(ip,"SilarFlag: unknown flag: ``",flgStr,"''",NULL);
	    return TCL_ERROR;
	}
	if ( NegateB )		*pMask = (*pMask & ~flgBit);
	else			*pMask = (*pMask |  flgBit);
	flgBit = 0;
    }
    return TCL_OK;
}

static int
_eemSilarFlagsStrPrs( Tcl_Interp *ip, TOPMask *pMask, char *flagsStr) {
    int		sC;
    char	**sV;

    if ( *flagsStr == '@' ) {
	*pMask &= ~(EEMSilarF_Modes);
	*pMask |= EEMSilarC_Modes;
	++flagsStr;
    }
    if ( Tcl_SplitList( ip, flagsStr, &sC, &sV) != TCL_OK ) {
    	Tcl_AppendResult(ip,"\nbad list: ``",flagsStr,"''",NULL);
	return TCL_ERROR;
    }
    if ( _eemSilarFlagsPrs( ip, pMask, sC, sV) != TCL_OK ) {
	Tcl_AppendResult(ip,"\nbad flags: ``",flagsStr,"''",NULL);
	free( (char*)sV);
	return TCL_ERROR;
    }
    free( (char*)sV);
    return TCL_OK;
}

int
eemSilarPrsTcl( Tcl_Interp *ip, EEMSilar *pSilar, int aC, char **aV) {
    int		k = 0;

    if (aC < 1 || strcasecmp(aV[0],"help")==0 ) {
	Tcl_AppendResult(ip, "SilarFormat: loc order flags ...", NULL);
	return TCL_ERROR;
    }
    if ( aC > k && aV[k][0] != '=' ) {	/* nLoc */
	if ( eemCplxStrPrsTcl( ip, aV[k], &pSilar->loc) != TCL_OK ) {
	    Tcl_AppendResult( ip, "\nSilar: bad location", NULL);
	    return TCL_ERROR;
	}
	pSilar->flags |= EEMSilarC_Loc;
    }
    ++k;
    if ( aC > k && aV[k][0] != '=' ) {	/* nPower */
	char *powStr = aV[k], *powStrEnd;
	int power = strtol( powStr, &powStrEnd, 0);
	if ( powStrEnd == powStr || *powStrEnd != NULL ) {
	    Tcl_AppendResult(ip,"\nSilar: bad power: ``",powStr,"''",NULL);
	    return TCL_ERROR;
	}
	pSilar->power = power;
	pSilar->flags |= EEMSilarC_Power;
    }
    ++k;
    for ( ; k < aC; k++) {		/* nFlags */
	if ( aV[k][0] == '=' )
	    continue;
	if ( _eemSilarFlagsStrPrs( ip, &pSilar->flags, aV[k]) != TCL_OK ) {
	    Tcl_AppendResult(ip,"\nSilar: bad flags: ``",aV[k],"''",NULL);
	    return TCL_ERROR;
	}
    }
    return TCL_OK;
}

int
eemSilarStrPrsTcl( Tcl_Interp *ip, EEMSilar *pSilar, char *silarStr) {
    int			sC;
    char		**sV;

    if ( Tcl_SplitList( ip, silarStr, &sC, &sV) != TCL_OK ) {
	Tcl_AppendResult(ip,"\nSilarStr: bad list: ``",silarStr,"''",NULL);
	return TCL_ERROR;
    }
    if ( eemSilarPrsTcl( ip, pSilar, sC, sV) != TCL_OK ) {
	Tcl_AppendResult(ip,"\nSilarStr: bad silar: ``",silarStr,"''",NULL);
        free( (char*)sV);
	return TCL_ERROR;
    }
    free( (char*)sV);
    return TCL_OK;
}

/*****************************************************************************
 *
 *		EEMSilarGrp
 *
 *****************************************************************************/

void
eemSilarGrpFmtTcl( Tcl_Interp *ip, EEMSilarGrp *pGrp, TOPLogical nestB) {
    if ( nestB )		Tcl_AppendResult(ip, " {", NULL);
    eemCplxFmtTcl( ip, &pGrp->gain, TOP_TRUE, NULL);
    EEM_SILAR_LOOP_BEGIN(pGrp,pSilar) {
	if ( pSilar->power == 0 )
	    continue;
	eemSilarFmtTcl( ip, pSilar, TOP_TRUE);
    } EEM_SILAR_LOOP_END();
    if ( nestB )		Tcl_AppendResult(ip, "}", NULL);
}

int
eemSilarGrpPrsTcl( Tcl_Interp *ip, EEMSilarGrp *pGrp, int aC, char **aV) {
    char	buf[100];

    if ( aC < 1 ) {
	Tcl_AppendResult(ip,"SilarGrp format: gain silar ...",NULL);
	return TCL_ERROR;
    }
    if ( eemCplxStrPrsTcl( ip, aV[0], &pGrp->gain) != TCL_OK ) {
	Tcl_AppendResult( ip, "\nSilarGrp: bad gain", NULL);
	return TCL_ERROR;
    }
    eemSilarGrpSetNum( pGrp, aC-1);
    EEM_SILAR_IDX_LOOP_FORW_BEGIN( pGrp, i, pSilar) {
	if ( eemSilarStrPrsTcl( ip, pSilar, aV[i+1]) != TCL_OK) {
	    sprintf( buf, "%d", i+1);
	    Tcl_AppendResult(ip,"\nSilarGrp: bad silar #",buf,NULL);
	    return TCL_ERROR;
	}
    } EEM_SILAR_IDX_LOOP_FORW_END();
    return TCL_OK;
}

int
eemSilarGrpStrPrsTcl( Tcl_Interp *ip, EEMSilarGrp *pGrp, char *grpStr) {
    int			sC;
    char		**sV;

    if ( Tcl_SplitList( ip, grpStr, &sC, &sV) != TCL_OK ) {
	Tcl_AppendResult(ip,"\nSilarGrpStr: bad list: ``",grpStr,"''",NULL);
	return TCL_ERROR;
    }
    if ( eemSilarGrpPrsTcl( ip, pGrp, sC, sV) != TCL_OK ) {
	Tcl_AppendResult(ip,"\nSilarGrpStr: bad group: ``",grpStr,"''",NULL);
        free( (char*)sV);
	return TCL_ERROR;
    }
    free( (char*)sV);
    return TCL_OK;
}

int	/* XXX: unfinished */
eemSilarGrpCmds( Tcl_Interp *ip, EEMSilarGrp *pGrp, int aC, char **aV) {
    char	*cmd, c;
    int		len;
    EEMComplex	g;

    if ( aC < 2 ) {
	Tcl_AppendResult(ip,"usage: ",aV[0],"(SilarGrpCmds) ",
	  "gain|num|clear|insert|delete|get|configure"," ?options?",NULL);
	return TCL_ERROR;
    }
    cmd = aV[1]; c = cmd[0]; len = strlen(cmd);
    /*IF*/ if ( c == 'g' && strncmp(cmd,"gain",len)==0 ) {
	if ( aC < 3 ) {
	    eemCplxFmtTcl( ip, &pGrp->gain, TOP_FALSE, NULL);
	    return TCL_OK;
	}
        if ( eemCplxStrPrsTcl( ip, aV[2], &g) != TCL_OK ) {
	    Tcl_AppendResult(ip,"\nExpected gain term, got ``",aV[2],"''",NULL);
	    return TCL_ERROR;
	}
	pGrp->gain = g;
    } else {
	Tcl_AppendResult(ip,aV[0],"(SilarGrpCmds): invalid subcommand ``",
	  cmd,"''",NULL);
	return TCL_ERROR;
    }
    return TCL_OK;
}

int
eemSilarGrpNotifyTcl( Tcl_Interp *ip, EEMSilarGrp *pGrp, char *basecmd,
   EEMSilarFlags mask) {
    char	*lb, *lbuf;
    int		totPower, pwr, r;

    lb = lbuf = topBufLGet();	*lb = '\0';
    totPower = 0;
    EEM_SILAR_IDX_LOOP_FORW_BEGIN( pGrp, silIdx, pSilar) {
	pwr = pSilar->power;
	if ( EEM_SILAR_ConjB(pSilar) )		pwr *= 2;
	totPower += pwr;
	if ( mask==0 || TOP_IsOn(pSilar->flags,mask) ) {
	    sprintf( lb, "%d ", silIdx);	lb += strlen(lb);
	}
    } EEM_SILAR_IDX_LOOP_FORW_END();
    r = topTclBgEvalFmt( ip, SPKG, "%s {%s} %d", basecmd, lbuf, totPower);
    topBufUnlock(lbuf);
    return r;
}

