/* 
    tclMain.c

    Main loop for tty-based tcl program.
    Based on tcl 6.3 "tclTest.c".

    Author: Kennard White

    Copyright (c) 1990-93 The Regents of the University of California.
    All rights reserved.  See the file "./copyright.h" for full copyright
    notice, limitation of liability, and disclaimer of warranty provisions.
#include "copyright.h"				COPYRIGHTENDKEY
*/
#if !defined(lint) && !defined(SABER)
static char rcsid[] = "$Header: /vol/cooley/cooley1/kennard/src/top/RCS/tclMain.c,v 1.2 93/08/27 14:29:04 kennard Exp $";
#endif

#include "topFixup.h"
#include <stdio.h>
#include <errno.h>
#include <sys/param.h>
#include <tcl.h>

#include "topStd.h"
#include "topTcl.h"

static Tcl_Interp *interp;
static Tcl_CmdBuf buffer;
static int quitFlag = 0;

/**
    Searches a directory path list {dir_path} for {file}.
    (In {dir_path}, dirs are seperated by colons: ``:'').  It will return
    a pointer to a statically allocated string which is a valid
    directory that contains {file}.  If {file} is NULL, the first
    directory in {dir_path} that exists will be returned.
**/
char*
topTclFindDirViaPath( Tcl_Interp *ip, char *dir_path, char *file) {
    char	path[MAXPATHLEN];
    static char	buf[MAXPATHLEN];
    char	*p, *pend;
    int		r;

    strcpy( path, dir_path);
    for ( p=path; *p != '\0'; p = pend) {
	if ( (pend = strchr( p, ':')) == NULL ) {
	    pend = p + strlen(p);
	} else {
	    *pend++ = '\0';
	}
	if ( (p = Tcl_TildeSubst( ip, p)) == NULL )
	    continue;
	if ( file != NULL ) {
	    sprintf( buf, "%s/%s", p, file);
	    r = access( buf, R_OK);
	} else {
	    r = access( p, R_OK);
	}
	if ( r == 0 ) {
	    strcpy( buf, p);
	    return buf;
	}
    }
    return NULL;
}

char*
topTclGetAppLibDir( Tcl_Interp *ip, char *appname, char *defpath,
  char *initfile) {
    int		i;
    char	appUpper[MAXPATHLEN];
    char	appLower[MAXPATHLEN];
    char	buf1[MAXPATHLEN];
    char	initfile_buf[MAXPATHLEN];
    char	*p, *dir;

    for ( i=0; appname[i] != '\0'; i++) {
	appUpper[i] = TOP_ToUpper(appname[i]);
	appLower[i] = TOP_ToLower(appname[i]);
    }
    appUpper[i] = appLower[i] = '\0';

    if ( initfile == NULL ) {
	initfile = initfile_buf;
        sprintf( initfile, "%s.tcl", appLower);
    }

    sprintf( buf1, "%s_library", appname);
    if ( (dir=Tcl_GetVar( ip, buf1, TCL_GLOBAL_ONLY)) != NULL )
	return dir;
    sprintf( buf1, "%s_library", appUpper);
    if ( (dir=Tcl_GetVar( ip, buf1, TCL_GLOBAL_ONLY)) != NULL )
	return dir;
    sprintf( buf1, "%s_library", appLower);
    if ( (dir=Tcl_GetVar( ip, buf1, TCL_GLOBAL_ONLY)) != NULL )
	return dir;

    sprintf( buf1, "%s_LIBRARY", appname);
    if ( (dir=getenv(buf1)) != NULL )
	return dir;
    sprintf( buf1, "%s_LIBRARY", appUpper);
    if ( (dir=getenv(buf1)) != NULL )
	return dir;
    sprintf( buf1, "%s_LIBRARY", appLower);
    if ( (dir=getenv(buf1)) != NULL )
	return dir;

    sprintf( buf1, "%s_LIBPATH", appname);
    if ( (p=getenv(buf1)) != NULL 
      && (dir=topTclFindDirViaPath( ip, p, initfile)) != NULL )
	return dir;
    sprintf( buf1, "%s_LIBPATH", appUpper);
    if ( (p=getenv(buf1)) != NULL 
      && (dir=topTclFindDirViaPath( ip, p, initfile)) != NULL )
	return dir;
    sprintf( buf1, "%s_LIBPATH", appLower);
    if ( (p=getenv(buf1)) != NULL 
      && (dir=topTclFindDirViaPath( ip, p, initfile)) != NULL )
	return dir;

    if ( defpath != NULL
      && (dir=topTclFindDirViaPath( ip, defpath, initfile)) != NULL )
	return dir;

    if ( access( initfile, R_OK) == 0 )
	return ".";

    if ( (dir=Tcl_GetVar( ip, "tk_library", TCL_GLOBAL_ONLY)) != NULL ) {
	sprintf( buf1, "%s/%s", dir, initfile);
	if ( access( buf1, R_OK)==0 )
	    return dir;
    }
    if ( (dir=Tcl_GetVar( ip, "tcl_library", TCL_GLOBAL_ONLY)) != NULL ) {
	sprintf( buf1, "%s/%s", dir, initfile);
	if ( access( buf1, R_OK)==0 )
	    return dir;
    }
    return NULL;
}

char*
topTclSetAppLib( Tcl_Interp *ip, char *appname, char *defpath) {
    char	applibvar[100];
    char	*dir;
    char	*dpath = defpath ? defpath : ".:lib";

    if ( (dir = topTclGetAppLibDir( ip, appname, dpath, NULL)) == NULL )
	return NULL;
    sprintf( applibvar, "%s_library", appname);
    Tcl_SetVar( ip, applibvar, dir, TCL_GLOBAL_ONLY);
    return Tcl_SetVar( ip, "app_library", dir, TCL_GLOBAL_ONLY);
}


char*
topTclGetLib( Tcl_Interp *ip) {
    char	*dir;

    if ( (dir=topTclGetAppLibDir( ip, "tcl", NULL, "init.tcl")) != NULL )
	return dir;
    if ( Tcl_Eval( ip, "info library", 0, NULL) != TCL_OK )
	return NULL;
    return ip->result;
}

char*
topTclSetLib( Tcl_Interp *ip) {
    char	*dir;

    if ( (dir = topTclGetLib( ip)) == NULL )
	return NULL;
    return Tcl_SetVar( ip, "tcl_library", dir, TCL_GLOBAL_ONLY);
}


static char *initCmd1 =
    "if [file exists $tcl_library/init.tcl] {source $tcl_library/init.tcl}";

static char *initCmd2 =
    "if [file exists $tk_library/tk.tcl] {source $tk_library/tk.tcl}";

int
topTclSourceInitFile( Tcl_Interp *ip, char *appname, char *defpath) {
    char	buf[MAXPATHLEN], *applib;

    if ( topTclSetLib( ip) == NULL ) {
	Tcl_SetResult(ip,"Can't find tcl library directory", TCL_STATIC);
	return TCL_ERROR;
    }

    if ( appname != NULL ) {
	if ( (applib = topTclSetAppLib( ip, appname, defpath)) == NULL ) {
	    Tcl_SetResult(ip,"Can't find application library directory",
	      TCL_STATIC);
	    return TCL_ERROR;
	}
        sprintf( buf, "%s/%s.tcl", applib, appname);
	if ( Tcl_EvalFile( ip, buf) != TCL_OK ) {
	    Tcl_AddErrorInfo( ip, "Application initialization file failed");
	    return TCL_ERROR;
	}
    } else {
	strcpy( buf, initCmd1);	/* writable strings */
        if ( Tcl_Eval( ip, buf, 0, (char **) NULL) != TCL_OK ) {
	    Tcl_AddErrorInfo( ip, "Standard TCL initialization file failed");
	    return TCL_ERROR;
	}
	if ( Tcl_GetVar( ip, "tk_library", TCL_GLOBAL_ONLY) != NULL ) {
	    strcpy( buf, initCmd2);	/* writable strings */
	    if ( Tcl_Eval( ip, buf, 0, NULL) != TCL_OK ) {
	        Tcl_AddErrorInfo( ip, "Standard TK initialization file failed");
	        return TCL_ERROR;
	    }
	}
    }
    return TCL_OK;
}

int
topTclMainLoop( Tcl_Interp *ip, char *prompt_in, char *appname, int initB) {
    char line[1000], *cmd, prompt[50];
    int result, gotPartial, ttyB;

    interp = ip ? ip : Tcl_CreateInterp();
    buffer = Tcl_CreateCmdBuf();

    if ( initB ) {
	if ( (result=topTclSourceInitFile( ip, appname, NULL)) != TCL_OK ) {
	    fprintf( stderr, "%s: %s\n", appname ? appname : "???", 
	      topTclGetRetMsg( interp, result));
	    return TCL_ERROR;
	}
    }

    if ( (ttyB = isatty(0)) ) {
	if ( prompt_in == NULL ) {
	    if ( appname )		sprintf( prompt, "%s> ", appname);
	    else			strcpy( prompt, "tcl> ");
	} else {
	    strcpy( prompt, prompt_in);
	}
        printf("Welcome to the TCL interpreter.  Type ``exit'' to leave.\n");
    }

    gotPartial = 0;
    while (1) {
	clearerr(stdin);
	if (!gotPartial) {
	    if ( ttyB ) {
		fputs(prompt, stdout);
	        fflush(stdout);
	    }
	}
	if (fgets(line, sizeof(line)-10, stdin) == NULL) {
	    if (!gotPartial) {
		break;	/* assume EOF */
	    }
	    line[0] = 0;
	}
	cmd = Tcl_AssembleCmd(buffer, line);
	if (cmd == NULL) {
	    gotPartial = 1;
	    continue;
	}

	gotPartial = 0;
	result = Tcl_RecordAndEval(interp, cmd, 0);
	if ( ttyB ) {
	    printf( "%s\n", topTclGetRetMsg( interp, result));
	} else if ( result != TCL_OK ) {
	    fprintf( stderr, "%s: %s\n", appname,
	      topTclGetRetMsg( interp, result));
	}
    }
    Tcl_DeleteCmdBuf(buffer);
    if ( !ip )	Tcl_DeleteInterp(interp);
    return TCL_OK;
}
