/* 
	tclTkMain.c :: TOP Library

	Taken from wish's main.c

    Author: Kennard White

    Copyright (c) 1990-93 The Regents of the University of California.
    All rights reserved.  See the file "./copyright.h" for full copyright
    notice, limitation of liability, and disclaimer of warranty provisions.
#include "copyright.h"				COPYRIGHTENDKEY
*/
#if !defined(lint) && !defined(SABER)
static char rcsid[] = "$Header: /vol/cooley/cooley1/kennard/src/top/RCS/tkMain.c,v 1.2 93/08/27 14:29:06 kennard Exp $";
#endif

#include "topFixup.h"
#include <tk.h>

#include "topStd.h"
#include "topMem.h"
#include "topMisc.h"
#include "topTcl.h"

extern int isatty();

typedef struct _TOPTkMainInfo TOPTkMainInfo;
struct _TOPTkMainInfo {
    Tcl_Interp*		ip;
    Tk_Window		win;
    Tcl_CmdBuf		buffer;
    int			ttyB;
    char*		prompt;
};

static void
_topStdinProc(clientData, mask)
    ClientData		clientData;
    int			mask;
{
    TOPTkMainInfo	*pInfo = (TOPTkMainInfo*) clientData;
    char		line[200];
    static int		gotPartial = 0;
    char		*cmd;
    int			result;

    if (mask & TK_READABLE) {
	if (fgets(line, 200, stdin) == NULL) {
	    if (!gotPartial) {
		if (pInfo->ttyB) {
		    Tcl_Eval(pInfo->ip, "destroy .", 0, (char **) NULL);
		    exit(0);
		} else {
		    Tk_DeleteFileHandler(0);
		}
		return;
	    } else {
		line[0] = 0;
	    }
	}
	cmd = Tcl_AssembleCmd( pInfo->buffer, line);
	if (cmd == NULL) {
	    gotPartial = 1;
	    return;
	}
	gotPartial = 0;
	result = Tcl_RecordAndEval(pInfo->ip, cmd, 0);
#ifdef oustersway
	if (pInfo->ip->result[0] != '\0') {
	    if ((result != TCL_OK) || (pInfo->ttyB)) {
		printf("%s\n", pInfo->ip->result);
	    }
	}
#else
	printf( "%s\n", topTclGetRetMsg(pInfo->ip,result));
#endif
	if (pInfo->ttyB) {
	    printf("%s",pInfo->prompt);
	    fflush(stdout);
	}
    }
}

/* ARGSUSED */
static void
_topStructureProc(clientData, eventPtr)
    ClientData clientData;	/* Information about window. */
    XEvent *eventPtr;		/* Information about event. */
{
    TOPTkMainInfo	*pInfo = (TOPTkMainInfo*) clientData;

    if (eventPtr->type == DestroyNotify) {
	pInfo->win = NULL;
    }
}

/*
 * Information for parsing command-line options:
 */
static int synchronize = 0;
static char *file = NULL;
static char *name = NULL;
static char *display = NULL;
static char *geometry = NULL;
static int _TopTclshB = 0;
static char *_TopWishFile = NULL;

static Tk_ArgvInfo argTable[] = {
    {"-file", TK_ARGV_STRING, (char *) NULL, (char *) &file,
	"File from which to read commands"},
    {"-geometry", TK_ARGV_STRING, (char *) NULL, (char *) &geometry,
	"Initial geometry for window"},
    {"-display", TK_ARGV_STRING, (char *) NULL, (char *) &display,
	"Display to use"},
    {"-name", TK_ARGV_STRING, (char *) NULL, (char *) &name,
	"Name to use for application"},
    {"-sync", TK_ARGV_CONSTANT, (char *) 1, (char *) &synchronize,
	"Use synchronous mode for display server"},
    {"-serverfix", TK_ARGV_CONSTANT, (char *) 1, (char *) &TOPXServerFixB,
	"Work around x server XDrawLines bug"},
    {"-tclsh", TK_ARGV_CONSTANT, (char *) 1, (char *) &_TopTclshB,
	"Emulate tclsh."},
    {"-wish", TK_ARGV_STRING, (char *) NULL, (char *) &_TopWishFile,
	"Emulate wish."},
    {(char *) NULL, TK_ARGV_END, (char *) NULL, (char *) NULL,
	(char *) NULL}
};

static TOPTkMainInfo	info;

int
topTkInitialize1( Tcl_Interp *ip, int *argc, char **argv, Tk_Window *pWin) {
    Tk_3DBorder			border;
    char			*showname;
    Tk_Window			win = NULL;
    int				r;

    info.ip = ip;

    /*
     * Deal with arguments, parsing them and simple using
     */
    if ( *argc >= 2 && argv != NULL ) {
	if ( Tk_ParseArgv(ip, (Tk_Window) NULL, argc, argv, argTable, 0)
	  != TCL_OK ) {
	    Tcl_AppendResult(ip,"\nError parsing argv", NULL);
	    return TCL_ERROR;
	}
    }
    if ( name == NULL )
	name = topBasename( argv[0]);
    topSetAppName( name);
    showname = topBasename( file ? file : name);
    info.prompt = memStrSaveFmt("%s: ", showname);

    if ( _TopTclshB ) {
	if ( file == NULL ) {
	    if ( topTclMainLoop( ip, NULL, name, 0) != TCL_OK )
		exit(1);
	} else {
	    if ( (r = Tcl_EvalFile( ip, file)) != TCL_OK ) {
		fprintf( stderr, "%s: %s\n", name, topTclGetRetMsg( ip, r));
		exit(1);
	    }
	}
	exit(0);
    }

    if ( geometry != NULL ) {
	Tcl_SetVar( ip, "geometry", geometry, TCL_GLOBAL_ONLY);
    }

    /* 
     * Create a window
     */
    if ( win == NULL ) {
    	if ( (win = Tk_CreateMainWindow(ip, display, showname)) == NULL ) {
	    Tcl_AppendResult(ip,"\nError creating main window", NULL);
	    return TCL_ERROR;
	}
        Tk_SetClass(win, "Tk");
    }
    info.win = win;
    Tk_CreateEventHandler(win, StructureNotifyMask, _topStructureProc,
	    (ClientData) &info);
    Tk_GeometryRequest(win, 200, 200);
    border = Tk_Get3DBorder(ip, win, None, "#4eee94");
    if (border == NULL) {
	Tcl_SetResult(ip, (char *) NULL, TCL_STATIC);
	Tk_SetWindowBackground(win, WhitePixelOfScreen(Tk_Screen(win)));
    } else {
	Tk_SetBackgroundFromBorder(win, border);
    }
    XSetForeground(Tk_Display(win), DefaultGCOfScreen(Tk_Screen(win)),
	    BlackPixelOfScreen(Tk_Screen(win)));


    if (synchronize) {
	XSynchronize(Tk_Display(win), True);
    }

    if ( pWin )
	*pWin = info.win;
    return TCL_OK;
}


int
topTkInitialize2( Tcl_Interp *ip, Tk_Window win, int argc, char **argv) {
    char			*args;
    char			buf[20];
    int				result;

    args = Tcl_Merge(argc-1, argv+1);
    Tcl_SetVar(ip, "argv", args, TCL_GLOBAL_ONLY);
    ckfree(args);
    sprintf(buf, "%d", argc-1);
    Tcl_SetVar(ip, "argc", buf, TCL_GLOBAL_ONLY);

    if ( (result=topTclSourceInitFile( ip, name, NULL) != TCL_OK) ) {
	Tcl_AppendResult(ip, "\nError in init file", NULL);
	return TCL_ERROR;
    }
    if (file != NULL) {
	char *command;
	char *args[3];

	args[0] = "source";
	args[1] = file;
	args[2] = (char *) NULL;
	command = Tcl_Merge(2, args);
	result = Tcl_Eval(ip, command, 0, (char **) NULL);
	ckfree(command);
	if (result != TCL_OK) {
	    Tcl_AppendResult( ip, "\nError in command file", NULL);
	    return TCL_ERROR;
	}
	info.ttyB = 0;
    } else {
	info.ttyB = isatty(0);
	Tk_CreateFileHandler(0, TK_READABLE|TK_EXCEPTION, _topStdinProc,
		(ClientData) &info);
	if (info.ttyB) {
	    /* printf("window is %s\n", Tk_Name(info.win)); */
	    printf("%s",info.prompt);
	}
    }
    fflush(stdout);
    info.buffer = Tcl_CreateCmdBuf();
    (void) Tcl_Eval(ip, "update", 0, (char **) NULL);

#ifdef notdef
    /*
     * The window could have been deleted as part of the startup
     * script.  If so, don't try to map it.
     */
    if (info.win != NULL) {
	Tk_MapWindow(info.win);
    }
#endif
    return TCL_OK;
}
