/* 
    Author: Kennard White

    Copyright (c) 1990-93 The Regents of the University of California.
    All rights reserved.  See the file "./copyright.h" for full copyright
    notice, limitation of liability, and disclaimer of warranty provisions.
#include "copyright.h"				COPYRIGHTENDKEY
*/
#if !defined(lint) && !defined(SABER)
static char rcsid[] = "$Header: /vol/cooley/cooley1/kennard/src/xpole/RCS/cplxseq.c,v 1.5 93/08/27 14:28:22 kennard Exp $";
#endif

#include "topFixup.h"
#include <tcl.h>

#include "topStd.h"

#include "pole.h"
#include "filter.h"
#include "cplxseq.h"

/*****************************************************************************

		Operators on Complex Sequences (not the DM)

*****************************************************************************/

static void
_xpCplxSeqConstructor( XPCplxSeqSnap *pSeq, XPCplxSeqView *pSeqView) {
    TOPMask		seqFlags;

    seqFlags = pSeqView->Flags & (XPCSeqF_Discrete|XPCSeqF_Freq);

    pSeq->PntView = pSeqView;
    pSeq->ValidMask = 0;
    pSeq->Flags = seqFlags;
    pSeq->DomStep = pSeqView->DM.SampleTime;
    pSeq->SigDomMin = -1;
    pSeq->SigDomMax = 1;
    topVecConstructor( &pSeq->Index, sizeof(double));
    eemPolyCplxConstructor( &pSeq->valFreqLoc);
    eemPolyCplxConstructor( &pSeq->valRI);
    eemPolyCplxConstructor( &pSeq->valMP);
    topVecConstructor( &pSeq->valPUnwrap, sizeof(double));
    topVecConstructor( &pSeq->valPwr, sizeof(double));
}

static void
_xpCplxSeqDestructor( XPCplxSeqSnap *pSeq) {
    pSeq->ValidMask = 0;
    pSeq->Flags = 0;
    topVecDestructor( &pSeq->Index);
    eemPolyCplxDestructor( &pSeq->valFreqLoc);
    eemPolyCplxDestructor( &pSeq->valRI);
    eemPolyCplxDestructor( &pSeq->valMP);
    topVecDestructor( &pSeq->valPUnwrap);
    topVecDestructor( &pSeq->valPwr);
}

XPCplxSeqSnap*
xpCplxSeqCreate( XPCplxSeqView *pSeqView, int flgs) {
    XPCplxSeqSnap		*pSeq;

    pSeq = (XPCplxSeqSnap*) memHeapObAlloc( pSeqView->SeqHeap);
    _xpCplxSeqConstructor( pSeq, pSeqView);
    return pSeq;
}

static XPViewSnap* _xpCplxSeqCreateCB( XPView *pView) {
    return (XPViewSnap*) xpCplxSeqCreate( (XPCplxSeqView*)pView, 0);
}

void
xpCplxSeqDestroy( XPCplxSeqSnap *pSeq) {
    XPCplxSeqView		*pSeqView = pSeq->PntView;

    _xpCplxSeqDestructor( pSeq);
    memHeapObFree( pSeqView->SeqHeap, pSeq);
}

static void _xpCplxSeqDestroyCB( XPViewSnap *pVS) {
    xpCplxSeqDestroy( (XPCplxSeqSnap*) pVS);
}

void
xpCplxSeqSetNum( XPCplxSeqSnap *pSeq, XPCplxSeqSnapWhich *pWh, int n) {
    TOPVector *pVec = XP_CPLXSEQ_GetVec(pSeq,pWh->VecOfs);
    topVecSetNum( pVec, n);
}

void
xpCplxSeqClear( XPCplxSeqSnap *pSeq, XPCplxSeqSnapWhich *pWh) {
    TOPVector *pVec = XP_CPLXSEQ_GetVec(pSeq,pWh->VecOfs);
    topVecClear( pVec);
}

/**
    Returns FALSE if already configured as requested, TRUE otherwize.
**/
TOPLogical
xpCplxSeqSetIndexLin( XPCplxSeqSnap *pSeq, int n, double domOfs, double domStep) {
    if ( TOPVEC_Num(&pSeq->Index) == n &&
      pSeq->DomOfs == domOfs && pSeq->DomStep == domStep ) {
        pSeq->ValidMask |= XPCSeqVals_Index; 
	return TOP_FALSE;
    }
    topVecSetNum( &pSeq->Index, n);
    pSeq->DomOfs = domOfs;
    pSeq->DomStep = domStep;
    TOPVEC_IDX_LOOP_FORW_BEGIN( &pSeq->Index, idx, double, pVal) {
	*pVal = domOfs + idx * domStep;
    } TOPVEC_IDX_LOOP_FORW_END();
    pSeq->ValidMask = XPCSeqVals_Index; 
    pSeq->PntVP->Flags &= ~XPViewF_Valid;
    return TOP_TRUE;
}

TOPLogical
xpCplxSeqSetIndexXfa( XPCplxSeqSnap *pSeq, XPXfa *pXfa, int n,
  int vdomOfs, int vdomStep) {
    if ( pXfa->Calls->Mode == XPXfaMode_Lin ) {
	double dofs = xpXfaToWin( pXfa, vdomOfs);
	double dstep = xpXfaScaleToWin( pXfa, (double)vdomStep);
	return xpCplxSeqSetIndexLin( pSeq, n, dofs, dstep);
    } 
    pSeq->DomStep = 0.0;
    topVecSetNum( &pSeq->Index, n);
    TOPVEC_IDX_LOOP_FORW_BEGIN( &pSeq->Index, ptNum, double, pVal) {
	*pVal = xpXfaToWin( pXfa, ptNum*vdomStep+vdomOfs);
    } TOPVEC_IDX_LOOP_FORW_END();
    pSeq->ValidMask = XPCSeqVals_Index; 
    pSeq->PntVP->Flags &= ~XPViewF_Valid;
    return TOP_TRUE;
}


/**
    Search the complex sequence for a value a point with index near {val}.
    Return the index of the greatest point whose x-val is less than
    {val}.  In general, the return value is the index where {val} would
    be inserted into the list.  Cases:

      normal ==> array[retidx] <= val < array[retidx+1]
      empty array ==> rets 0
      off lo end (val is less than all others) ==> rets 0
      off hi end (val is greater than all others)  ==> rets len of array

    Note that the equel case really shouldnt happen, so we dont worry
    about it too much.
**/
int
xpCplxSeqFindByIndex( XPCplxSeqSnap *pSeq, double val, XPWhich wh) {
    int		n, idx;

    n = XP_CPLXSEQ_NumIndex(pSeq);
    /*IF*/ if ( n == 0 || val < XP_CPLXSEQ_GetIndex( pSeq, 0) ) {
	idx = 0;
    } else if ( val >= XP_CPLXSEQ_GetIndex( pSeq, n-1) ) {
	idx = n;
    } else {
	/* simple linear search for now; replace with binary later */
	for ( idx=n-1; idx >= 0; idx--) {
	    if ( val > XP_CPLXSEQ_GetIndex(pSeq, idx) ) {
		++idx;	break;
	    }
	}
    }
    /* at this point, we have the "After" index. */
    switch ( wh ) {
    case XPWh_Before:
	return idx-1;
    case XPWh_Near:
	if ( idx > 0 ) {
	    double befVal, aftVal, befDist, aftDist;
	    befVal = XP_CPLXSEQ_GetIndex(pSeq, idx-1);
	    aftVal = XP_CPLXSEQ_GetIndex(pSeq, idx);
	    befDist = befVal - val;
	    aftDist = aftVal - val;
	    if ( TOP_ABS(befDist) < TOP_ABS(aftDist) )
		return idx-1;
	}
    default:;
    }
    return idx;
}

TOPLogical
xpCplxSeqMinMaxCalc( XPCplxSeqSnap *pSeq, XPCplxSeqSnapWhich *pWh, XPFlt *pMin, XPFlt *pMax) {
    XPFlt	minval, maxval;

    if ( XP_CPLXSEQ_Num(pSeq,pWh) == 0 ) {
	*pMin = *pMax = 1;
	return TOP_FALSE;
    }
    minval = maxval = XP_CPLXSEQ_Get( pSeq, pWh, 0);
    XPCPLXSEQ_LOOP_FORW_BEGIN( pSeq, pWh, pVal) {
	if ( *pVal < minval )	minval = *pVal;
	if ( *pVal > maxval )	maxval = *pVal;
    } XPCPLXSEQ_LOOP_FORW_END();
    *pMin = minval;
    *pMax = maxval;
    return TOP_TRUE;
}

/**
    Compute the unwrapped phase adjustment (padj) from {idx1} (with 
    wrapped phase (ph1)}, to {idx2} (with wrapped phase ph2).
    Then, padj is such that padj is a multiple of 2*PI, and:
	unwrap ph2 = wrap ph2 + padj.
**/
#ifdef notdef
static double
_xpCplxSeqResolvePhase( XPCplxSeqSnap *pSeq,
    double idx1, double ph1, double idx2, double ph2) {
    return 0.0;
}
#endif

#ifdef notdef
/**
    Compute the absolute (unwrapped) phase for {pSeq} given
    the wrapped phase (pSeq.valMP) and a function to get samples
    of the unwrapped phase.  Depending on how {pSeq} is defined,
    it may be possible to directly compute the absolute phase,
    but it is very expensive (e.g., SilarFourier).  Thus we
    attempt to simply unwrap the given samples, but retrive
    the absolute samples when we dont trust our unwrapping mechanism. 
    Note: For the algorithm to work right, the intial value of {prevsamp}
    must just be sufficiently out of the wrapped-phase range.
**/
static void
_xpCplxSeqPU_Abs( XPCplxSeqSnap *pSeq) {
    double	thres = 2*M_PI*.10, prevsamp = 1e20;
    double	adj = 0.0, newsamp, delta;

    topVecSetNum( &pSeq->valPUnwrap, EEM_POLYCPLX_Num(&pSeq->valMP));
    EEM_POLYCPLX_IDX_LOOP_FORW_BEGIN( &pSeq->valMP, idx, pMP) {
	newsamp = pMP->i + adj;
	delta = prevsamp - newsamp;
	if ( 1 /*TOP_ABS(delta) > thres*/ ) {
	    newsamp = (*(pSeq->PntView->AbsPhaseFunc))( pSeq, idx,
	      pSeq->PntView->AbsPhaseCld);
	    adj = newsamp - pMP->i;
	}
	TOPVEC_Get(&pSeq->valPUnwrap,idx,double) = newsamp;
	prevsamp = newsamp;
    } EEM_POLYCPLX_IDX_LOOP_FORW_END();
}
#endif

/**
    Scans the array of freq samples and upwraps the phase.  {init_phase}
    is the initial phase. (In case were doing this in chunks.)
**/
static void
xpCplxSeqPhaseUnwrap( XPCplxSeqSnap *pSeq, double initPhase) {
    XPFlt	thres = 2*M_PI*.75, delta, thesamp;
    XPFlt	newsamp, adj = initPhase, prevsamp = initPhase;

    topVecSetNum( &pSeq->valPUnwrap, EEM_POLYCPLX_Num(&pSeq->valMP));
    EEM_POLYCPLX_IDX_LOOP_FORW_BEGIN( &pSeq->valMP, idx, pMP) {
	thesamp = pMP->i;
	newsamp = thesamp + adj;
	delta = TOP_ABS(prevsamp-newsamp);
	if ( delta > thres ) {
#ifdef notdef
	    if ( pSeq->PntView->AbsPhaseFunc ) {
	        newsamp = (*(pSeq->PntView->AbsPhaseFunc))( pSeq, 
		  ...,
	          pSeq->PntView->AbsPhaseCld);
		adj = TOP_FLOOR(newsamp - prevsamp,2*M_PI);
	    }
#endif
	    adj = TOP_ROUND(prevsamp - thesamp,2*M_PI);
	    newsamp = thesamp + adj;
	}
	TOPVEC_Get(&pSeq->valPUnwrap,idx,double) = newsamp;
	prevsamp = newsamp;
    } EEM_POLYCPLX_IDX_LOOP_FORW_END();
}

void xpCplxSeqCalc( XPCplxSeqSnap *pSeq, XPCplxSeqSnapValMask mask) {
    if ( (mask & (XPCSeqVals_RI|XPCSeqVals_MP|XPCSeqVals_PUnwrap))!=0
      && (pSeq->ValidMask & XPCSeqVals_RI)==0 ) {
	topWarnMsg("xpCplxSeqCalc: created RI part");
	eemPolyCplxSetNum( &pSeq->valRI, XP_CPLXSEQ_NumIndex(pSeq));
	pSeq->ValidMask |= XPCSeqVals_RI;
    }
    if ( (mask & (XPCSeqVals_MP|XPCSeqVals_PUnwrap))!=0
      && (pSeq->ValidMask & XPCSeqVals_MP)==0 ) {
	eemPolyCplxRI2MP( &pSeq->valRI, &pSeq->valMP); 
	pSeq->ValidMask |= XPCSeqVals_MP;
    }
    if ( (mask & XPCSeqVals_PUnwrap)!=0
      && (pSeq->ValidMask & XPCSeqVals_PUnwrap)==0 ) {
#ifdef notdef
	if ( pSeq->PntView->AbsPhaseFunc ) {
	    _xpCplxSeqPU_Abs( pSeq);
	} else {
            xpCplxSeqPhaseUnwrap( pSeq, 0.0);
	}
#else
        xpCplxSeqPhaseUnwrap( pSeq, 0.0);
#endif
	pSeq->ValidMask |= XPCSeqVals_PUnwrap;
    }
}

XPCplxSeqSnapValMask
xpCplxSeqValNameToMask( char *name) {
    if (        strcasecmp(name,"real")==0
      ||        strcasecmp(name,"r")==0 ) {
	return XPCSeqVals_R;
    } else if ( strcasecmp(name,"imag")==0
      ||        strcasecmp(name,"i")==0
      ||        strcasecmp(name,"imaginary")==0 ) {
	return XPCSeqVals_I;
    } else if ( strcasecmp(name,"mag")==0
      ||        strcasecmp(name,"m")==0
      ||        strcasecmp(name,"magnitude")==0 ) {
	return XPCSeqVals_M;
    } else if ( strcasecmp(name,"phase")==0
      ||        strcasecmp(name,"p")==0 ) {
	return XPCSeqVals_PUnwrap;
    } else if ( strcasecmp(name,"phaseUnwrap")==0
      ||        strcasecmp(name,"pu")==0
      ||        strcasecmp(name,"phaseU")==0
      ||        strcasecmp(name,"PUnwrap")==0 ) {
	return XPCSeqVals_PUnwrap;
    } else if ( strcasecmp(name,"wrapphase")==0
      ||        strcasecmp(name,"wp")==0
      ||        strcasecmp(name,"wphase")==0 ) {
	return XPCSeqVals_P;
    } else {
	return XPCSeqVals_Null;
    }
}

char*
xpCplxSeqValMaskToName( XPCplxSeqSnapValMask m) {
    switch ( m ) {
    case XPCSeqVals_R:		return "real";
    case XPCSeqVals_I:		return "imaginary";
    case XPCSeqVals_M:		return "magnitude";
    case XPCSeqVals_P:		return "phase";
    case XPCSeqVals_PUnwrap:	return "phaseUnwrap";
    }
    return NULL;
}

int
xpCplxSeqValMaskToWhich( XPCplxSeqSnapValMask m, XPCplxSeqSnapWhich *pWh) {
    switch ( m ) {
    case XPCSeqVals_Index:
	pWh->VecOfs = TOP_Offset(XPCplxSeqSnap,Index);
	pWh->DblIdx = 0;
	break;
    case XPCSeqVals_R:
	pWh->VecOfs = TOP_Offset(XPCplxSeqSnap,valRI.Coef);
	pWh->DblIdx = 0;
	break;
    case XPCSeqVals_I:
	pWh->VecOfs = TOP_Offset(XPCplxSeqSnap,valRI.Coef);
	pWh->DblIdx = 1;
	break;
    case XPCSeqVals_M:
	pWh->VecOfs = TOP_Offset(XPCplxSeqSnap,valMP.Coef);
	pWh->DblIdx = 0;
	break;
    case XPCSeqVals_P:
	pWh->VecOfs = TOP_Offset(XPCplxSeqSnap,valMP.Coef);
	pWh->DblIdx = 1;
	break;
    case XPCSeqVals_PUnwrap:
	pWh->VecOfs = TOP_Offset(XPCplxSeqSnap,valPUnwrap);
	pWh->DblIdx = 0;
	break;
    default:
	return TOP_FALSE;
    }
    return TOP_TRUE;
}

#ifdef notdef
/**
    Considers the complex n pt sequence c_0, c_1, c_2, ... c_(n-1)
    as a polyomial P(z) = c_0 * z^(n-1) + c_1 * z^(n-2) + ... + c_(n-1)
    and returns the result.
**/
void
xpCplxSeqEvalPoly( XPCplxSeqSnap *pSeq, EEMComplex *pZ, EEMComplex *pP) {
    int			n = pSeq->Vec.Num;

    if ( n == 0 ) {
	EEM_CPLX_SET( pP, 0, 0);
	return;
    }
    *pP = TOPVEC_GET( &pSeq->Vec, 0, XPCplxSeqSnapPt).valRI;
    XPCPLXSEQ_SUBIDX_LOOP_FORW_BEGIN( pSeq, 1, n, ptIdx, pPt) {
	eemCplxMul2( pP, pZ);
	eemCplxAdd2( pP, &pPt->valRI);
    } XPCPLXSEQ_IDX_LOOP_FORW_END();
}
#endif


/*****************************************************************************

		Operators on Complex Sequences Data Modules

*****************************************************************************/

static void
_xpCplxSeqViewConstructor( XPView *pView, char *class) {
    XPCplxSeqView	*pSeqView = (XPCplxSeqView*) pView;
    TOPMask		seqFlags;
    char		*classLead = "CplxSeq", *classSuf;

    if ( class == NULL ) {
	class = classLead;
    } else {
	if ( strncasecmp( class, classLead, strlen(classLead)) != 0 ) {
	    topAbortMsg("Bad %s class: %s", classLead, class);
	    return;
	}
    }
    classSuf = class + strlen(classLead);

    pSeqView->SeqHeap = memPublicHeapCreate( "XPCplxSeqSnap", sizeof(XPCplxSeqSnap));
    seqFlags = 0;
    if ( strcmp(classSuf,"Freq")==0 ) {
        seqFlags = XPCSeqF_Freq;
    }
    if ( pView->XfDomain == XP_XFDOM_Z ) {
	if ( TOP_IsOff(seqFlags,XPCSeqF_Freq) )
	    seqFlags |= XPCSeqF_Discrete;
    }
    pSeqView->Flags = seqFlags;
}

static void
_xpCplxSeqViewDestructor( XPView *pView) {
    XPCplxSeqView	*pSeqView = (XPCplxSeqView*) pView;
    
    memHeapDestroy( pSeqView->SeqHeap);
    pSeqView->SeqHeap = NULL;
}

static int
_xpCplxSeqViewCmdPrs( Tcl_Interp *ip, XPView *pView, char *cmd,
  int aC, char **aV) {
    char	c;
    int		len;
    XPCplxSeqSnap *pSeq = (XPCplxSeqSnap*) xpViewGetSnap( pView, XPGS_AsIs);

    c = cmd[0]; len = strlen(cmd);
    /*IF*/ if ( c=='i' && strncmp(cmd,"index",len)==0 ) {
	if ( aC == 0 ) {
	    topTclRetFmt( ip, "%d %.15g %.15g",
	      XP_CPLXSEQ_NumIndex(pSeq), pSeq->DomOfs, pSeq->DomStep);
	} else {
	    if ( aC != 3 ) {
		Tcl_AppendResult(ip,cmd,": bad args",NULL);
		return TCL_ERROR;
	    }
	    xpCplxSeqSetIndexLin( pSeq, atoi(aV[0]), atof(aV[1]), atof(aV[2]));
	}
    } else {
	return TCL_CONTINUE;
    }
    return TCL_OK;
}

/*global*/ XPViewClass XpCplxSeqViewClass = {
    "CplxSeq", sizeof(XPCplxSeqView), NULL,
    _xpCplxSeqViewConstructor, _xpCplxSeqViewDestructor,
    _xpCplxSeqViewCmdPrs, NULL, NULL, 
    _xpCplxSeqCreateCB, _xpCplxSeqDestroyCB
};


XPCplxSeqSnap*
xpCplxSeqGetSnap( XPAgent *pAg, XPGetSnapHow how) {
    return (XPCplxSeqSnap*) xpAgentGetSnap( pAg, how);
}
