#if !defined(_CPLXSEQ_H_)
#define _CPLXSEQ_H_ 1
/* 
    Author: Kennard White
    RCS: $Header: /vol/cooley/cooley1/kennard/src/xpole/RCS/cplxseq.h,v 1.5 93/08/27 14:28:04 kennard Exp $
    Copyright (c) 1990-93 The Regents of the University of California.
    All rights reserved.  See the file "./copyright.h" for full copyright
    notice, limitation of liability, and disclaimer of warranty provisions.
#include "copyright.h"				COPYRIGHTENDKEY
*/

#if !defined(_XPXFA_H_) && __TOP_AUTOINCLUDE__
#include "xpXfa.h"
#endif
#if !defined(_EEMPOLYCPLX_H_) && __TOP_AUTOINCLUDE__
#include "eemPoly.h"
#endif


/******************************************************************************
    XPCplxSeqSnapPt, XPCplxSeqSnap

    This structure stores sequences of complex values.
    Generally, the sequence will be samples of a time or frequency domain
    signal.  The signal being represented may occur naturally as
    samples (a DT impulse response), or may be samples of a continuous
    signal (a DTFT).  It consists of a vector (dynamic array) of
    points.

    Each point represents a single element, over some index set, in a
    complex-valued sequence.  For time-domain sequences, {Index} is the
    time of the sample, and {valRI} is the value at the time.  For
    freq-domain sequences,  {Index} is the frequency of the sample, and
    {FreqLoc} is a temporary location used to store information for the
    geometric evaluation of the FT or DTFT.

    In all cases, {valMP}, {valPhaseU}, and {valPwr} are all derived 
    directly from {valRI}.
******************************************************************************/

typedef TOPMask XPCplxSeqSnapValMask;

#define XPCSeqVals_Null		0
#define XPCSeqVals_Index	TOP_BIT(0)
#define XPCSeqVals_FreqLoc	TOP_BIT(7)
#define XPCSeqVals_R		TOP_BIT(1)
#define XPCSeqVals_I		TOP_BIT(2)
#define XPCSeqVals_RI		( XPCSeqVals_R | XPCSeqVals_I )
#define XPCSeqVals_M		TOP_BIT(3)
#define XPCSeqVals_P		TOP_BIT(4)
#define XPCSeqVals_MP		( XPCSeqVals_M | XPCSeqVals_P )
#define XPCSeqVals_PUnwrap	TOP_BIT(5)
#define XPCSeqVals_Pwr		TOP_BIT(6)

/**
    Reference one vector of the complex sequence.
    The reference is independent of a particular XPCplxSeqSnap: that is,
    there are only offsets in it, not pointers.
**/
typedef struct _XPCplxSeqSnapWhich XPCplxSeqSnapWhich;
struct _XPCplxSeqSnapWhich {
    int		VecOfs;		/* bytes from pSeq to TOPVector */
    int		DblIdx;		/* index of val within element */
};


#define XPCSeqF_Freq		TOP_BIT(0)
#define XPCSeqF_DomS		TOP_BIT(1)
#define XPCSeqF_Master		TOP_BIT(2)
#define XPCSeqF_Discrete	TOP_BIT(3)	/* linear, for now */

struct _XPCplxSeqSnap {
    XPCplxSeqView*	PntView;	/* MUST be first */
    XPViewPart*		PntVP;		/* MUST be second */
    short		ValidMask;
    short		Flags;
    double		DomOfs;		/* domain ofset, XPCSeqF_Discrete */
    double		DomStep;	/* domain step size, XPCSeqF_Discrete */
    double		SigDomMin;	/* significant domain min & max */
    double		SigDomMax;

    TOPVector		Index;		/* of double */
    EEMPolyCplx		valFreqLoc;
    EEMPolyCplx		valRI;
    EEMPolyCplx		valMP;
    TOPVector		valPUnwrap;	/* of double */
    TOPVector		valPwr;		/* of double */
};

#define XP_CPLXSEQ_GetVec(pSeq,vec_ofs) \
  ( (TOPVector*) (((TOPPtr)(pSeq))+(vec_ofs)) )

#define XP_CPLXSEQ_NumVec(pSeq,vec_ofs)	\
    TOPVEC_Num(XP_CPLXSEQ_GetVec(pSeq,vec_ofs))
#define XP_CPLXSEQ_Num(pSeq,pWh)	XP_CPLXSEQ_NumVec(pSeq,(pWh)->VecOfs)

#define XP_CPLXSEQ_NumIndex(pSeq)	TOPVEC_Num(&(pSeq)->Index)
#define XP_CPLXSEQ_GetIndex(pSeq,i)	TOPVEC_Get(&(pSeq)->Index,(i),double)

#define XP_CPLXSEQ_GetEle(pSeq,vec_ofs,i) \
  ( &TOPVEC_Get( XP_CPLXSEQ_GetVec(pSeq,(vec_ofs)),(i),double) )
#define XP_CPLXSEQ_Get(pSeq,pWh,i) \
  ( XP_CPLXSEQ_GetEle(pSeq,(pWh)->VecOfs,(i))[(pWh)->DblIdx] )

#define XPCPLXSEQ_I_IDX_LOOP_FORW_BEGIN( pSeq, idx, pVal) \
    TOPVEC_IDX_LOOP_FORW_BEGIN( &(pSeq)->Index, idx, double, pVal)
#define XPCPLXSEQ_I_IDX_LOOP_FORW_END() \
    TOPVEC_IDX_LOOP_FORW_END()

#define XPCPLXSEQ_SUBIDX_LOOP_FORW_BEGIN( pSeq, pWh, sI, fI, ptIdx, pVal) { \
    TOPVector	*__xpcplxseq__vec__ = XP_CPLXSEQ_GetVec(pSeq,(pWh)->VecOfs); \
    int		__xpcplxseq__ofs__ = (pWh)->DblIdx;			\
    double	*pVal;							\
    TOPVEC_SUBIDX_SLOOP_FORW_BEGIN( __xpcplxseq__vec__,			\
      (sI), (fI), ptIdx, double, pEle)					\
	pVal = pEle + __xpcplxseq__ofs__;				\
	/* BEGIN USER CODE HERE */
	

#define XPCPLXSEQ_SUBIDX_LOOP_FORW_END() \
    TOPVEC_SUBIDX_SLOOP_FORW_END() }

#define XPCPLXSEQ_IDX_LOOP_FORW_BEGIN( pSeq, pWh, ptIdx, pPt) {		\
    TOPVector	*__xpcplxseq__vec__ = XP_CPLXSEQ_GetVec(pSeq,(pWh)->VecOfs); \
    int		__xpcplxseq__ofs__ = (pWh)->DblIdx;			\
    double	*pVal;							\
    TOPVEC_IDX_SLOOP_FORW_BEGIN( __xpcplxseq__vec__, ptIdx, double, pEle) \
	pVal = pEle + __xpcplxseq__ofs__;				\
	/* BEGIN USER CODE HERE */
    
#define XPCPLXSEQ_IDX_LOOP_FORW_END() \
    TOPVEC_IDX_SLOOP_FORW_END() }

#define XPCPLXSEQ_LOOP_FORW_BEGIN( pSeq, pWh, pPt) \
    XPCPLXSEQ_IDX_LOOP_FORW_BEGIN( (pSeq), (pWh), __xpcplxseq_idx_, pPt)

#define XPCPLXSEQ_LOOP_FORW_END() \
    XPCPLXSEQ_IDX_LOOP_FORW_END()


struct _XPCplxSeqView /* derived(XPView) */ {
    XPView		DM;		/* Must be first */
    int			Flags;
    MEMHeapHandle	SeqHeap;
    double		(*PhaseFunc) ARGS((XPCplxSeqSnap *pSeq,
			  double freq, TOPPtr cld));
    TOPPtr		PhaseCld;
    double		(*AbsPhaseFunc) ARGS((XPCplxSeqSnap *pSeq,
			  int idx, TOPPtr cld));
    TOPPtr		AbsPhaseCld;
};

typedef enum _XPWhich XPWhich;
enum _XPWhich {
    XPWh_Null, XPWh_Before, XPWh_Near, XPWh_After,
};


extern XPViewClass	XpCplxSeqViewClass;

extern XPCplxSeqSnap*	xpCplxSeqCreate( XPCplxSeqView *pSeqDM, int flgs);
extern void		xpCplxSeqDestroy( XPCplxSeqSnap *pSeq);
extern void		xpCplxSeqSetNum( XPCplxSeqSnap *pSeq,
			  XPCplxSeqSnapWhich *pWh, int n);
extern void		xpCplxSeqClear( XPCplxSeqSnap *pSeq, XPCplxSeqSnapWhich *pWh);

extern TOPLogical	xpCplxSeqSetIndexLin( XPCplxSeqSnap *pSeq,
			  int n, double domOfs, double domStep);
extern TOPLogical	xpCplxSeqSetIndexXfa( XPCplxSeqSnap *pSeq, XPXfa *pXfa,
			  int n, int domOfs, int domStep);
extern int		xpCplxSeqFindByIndex( XPCplxSeqSnap *pSeq, double val,
			  XPWhich wh);

extern TOPLogical	xpCplxSeqMinMaxCalc( XPCplxSeqSnap *pSeq,
			  XPCplxSeqSnapWhich *pWWh, XPFlt *pMin, XPFlt *pMax);
extern void		xpCplxSeqCalc( XPCplxSeqSnap *pSeq, XPCplxSeqSnapValMask mask);
#ifdef notdef
extern void		xpCplxSeqEvalPoly( XPCplxSeqSnap *pSeq, EEMComplex *pZ,
			  EEMComplex *pP);
#endif

extern XPCplxSeqSnapValMask	xpCplxSeqValNameToMask( char *name);
extern char*		xpCplxSeqValMaskToName( XPCplxSeqSnapValMask m);
extern int		xpCplxSeqValMaskToWhich( XPCplxSeqSnapValMask m,
			  XPCplxSeqSnapWhich *pWh);

extern XPView*		xpCplxSeqDmCreate( XPFilter *pFltr, 
			  char *name, char *class);
extern void		xpCplxSeqDmDestroy( XPView *pView);
extern XPCplxSeqSnap*	xpCplxSeqGetSnap(XPAgent *pAgent, XPGetSnapHow how);


#if defined(XP_HAVE_X11)
extern XPLabel*	xpCreateSeqBdg( Widget pntWdg, char *dataSpec,
			  char *class);
#endif

/*
 * seqDh.c
 */
extern int		xpCplxSeqDhRegisterCmds ARGS_TCL((Tcl_Interp *ip));
#endif /* _CPLXSEQ_H_ */
