/* 
    seqDh.c :: SequenceData Handler :: XPole

    Author: Kennard White

    Copyright (c) 1990-93 The Regents of the University of California.
    All rights reserved.  See the file "./copyright.h" for full copyright
    notice, limitation of liability, and disclaimer of warranty provisions.
#include "copyright.h"				COPYRIGHTENDKEY
*/
#if !defined(lint) && !defined(SABER)
static char rcsid[] = "$Header: /vol/cooley/cooley1/kennard/src/xpole/RCS/cseqDh.c,v 1.5 93/08/27 14:28:21 kennard Exp $";
#endif

#include "topFixup.h"
#include <stdio.h>
#include <X11/Xlib.h>
#include <tcl.h>

#include "topStd.h"
#include "topMem.h"
#include "topVec.h"
#include "eemSilar.h"

#include "xpXfa.h"
#include "xpSeq.h"

#include "pole.h"
#include "filter.h"
#include "cplxseq.h"

/**
    XPCplxSeqDHA :: Agent for CplxSeqDH

    All components of the same CplxSeq View will share a common agent,
    contained in a {XPCplxSeqDHA}.  This XPCplxSeqDHA is responsible
    for doing the common (x-axis) config for all graph sequences
    (y-axis mapping is done independent by each CplxSeqDH).

    The {domainIdxLo} and {domainIdxHi} are indexes (0-based) into
    {wPtsVec}.  They are the first and last points that that will
    be mapped and given to the core for drawing.  In general, these
    two points will be the points just *outside* the current
    viewport.  This is so that in connect line mode, there will be
    lines drawn to the edge.  It also provides a line when when
    none of the points are on the viewport.
**/

static DLPtrListHandle _XpCplxSeqDhas = NULL;

typedef struct _XPCplxSeqDHA XPCplxSeqDHA;
struct _XPCplxSeqDHA {
    XPLabel		Lbl;
    XPConsumer		CM;
    /* TOPMask		valMask; */
    int			domainIdxLo;
    int			domainIdxHi;
    DLPtrListHandle	DHs;
    int			serialX;
};

typedef struct _XPCplxSeqDH XPCplxSeqDH;
struct _XPCplxSeqDH {
    XPCplxSeqDHA*	DHA;
    TOPMask		valMask;
    XPCplxSeqSnapWhich	Wh;
    XPSeqDataExt*	ext;
};

static XPNotifyWhen
_xpCplxSeqDhNotifyCB( XPAgent *pAgent, XPViewPart *pVP, XPViewEvent ev) {
    XPCplxSeqDHA	*pDHA = (XPCplxSeqDHA*) pAgent->Lbl->Me;

    DLPTRLIST_LOOP_FORW_BEGIN(pDHA->DHs, XPCplxSeqDH*, pDH) {
    	/* XXX: this will do the update immediately */
    	xpSeqDataUpdate( pDH->ext);
    } DLPTRLIST_LOOP_FORW_END();
    return XPNW_Null;
}

static XPCplxSeqDHA*
_xpCplxSeqGetDHA( XPView *pView, char *name) {
    static int		SN;
    XPCplxSeqDHA	*pDHA;

    if ( _XpCplxSeqDhas == NULL )
	_XpCplxSeqDhas = dlPtrListCreate("XPCplxSeqDHA");

    DLPTRLIST_LOOP_FORW_BEGIN(_XpCplxSeqDhas, XPCplxSeqDHA*, pPreDHA) {
	if ( pPreDHA->CM.CurView == pView && strcmp(pPreDHA->Lbl.Name,name)==0 )
	    return pPreDHA;
    } DLPTRLIST_LOOP_FORW_END();

    pDHA = MEM_ALLOC(XPCplxSeqDHA);
    dlPtrListInsertAtTail( _XpCplxSeqDhas, pDHA);
    xpLabelConstructor( &pDHA->Lbl, (TOPPtr)pDHA, &SN, name, "CplxSeqDHA");
    pDHA->DHs = dlPtrListCreate("XPCplxSeqDH");
    xpAgentConstructor( &pDHA->CM, &pDHA->Lbl, NULL);
    xpViewAddAgent( pView, &pDHA->CM, name);
    pDHA->CM.NotifyFunc = _xpCplxSeqDhNotifyCB;
    pDHA->serialX = 0;
    return pDHA;
}

/**
    Main task is to parse the {dataSpec} and decide if its valid
    or not.
**/
static TOPLogical
_xpCplxSeqDhConstructor( XPSeqDataExt *pExt) {
    XPCplxSeqDH		*pDH;
    XPCplxSeqDHA	*pDHA;
    XPView		*pView;
    char		*dm, *dc;	/* data mod and data component */
    TOPMask		valmask;	/* which component */

    /*
     * Get the View
     */
    dm = memStrSave(pExt->dataSpec);
    if ( (dc=strrchr( dm, ',')) != NULL ) {
	*dc = '\0';
	if ( (valmask = xpCplxSeqValNameToMask( dc+1)) == XPCSeqVals_Null ) {
	    MEM_FREE( dm);
	    return TOP_FALSE;
	}
    } else {
	valmask = XPCSeqVals_R;
    }
    if ( (pView = xpViewGetBySpec( dm, "CplxSeq", NULL, TOP_TRUE)) == NULL ) {
	MEM_FREE( dm);
	return TOP_FALSE;
    }
    MEM_FREE( dm);

    /*
     * Set up DH and extract component
     */
    pDH = MEM_ALLOC(XPCplxSeqDH);
    pDH->valMask = valmask;
    xpCplxSeqValMaskToWhich( valmask, &pDH->Wh);

    /*
     * Get DHA, crosslink DH & DHA
     */
    pDHA = _xpCplxSeqGetDHA( pView, pExt->graphTag);
    pDH->DHA = pDHA;
    dlPtrListInsertAtTail( pDHA->DHs, pDH);

    /* cross-link the GraphExt and DH */
    pDH->ext = pExt;
    pExt->userData = (TOPPtr) pDH;
    return TOP_TRUE;
}

static TOPLogical
_xpCplxSeqDhDestructor( XPSeqDataExt *pExt) {
    XPCplxSeqDH		*pDH = (XPCplxSeqDH*) pExt->userData;
    XPCplxSeqDHA	*pDHA = pDH->DHA;

    /* unlink DH and GraphExt */
    pDH->ext = NULL;
    pExt->userData = NULL;

    /* unlink DH and DHA */
    dlPtrListElemRemove( pDHA->DHs, pDH);
    pDH->DHA = NULL;

    if ( dlPtrListEmptyB(pDHA->DHs) ) {
	/* kill the DHA */
        dlPtrListElemRemove( _XpCplxSeqDhas, pDHA);
    	xpAgentDestructor( &pDHA->CM);
    	xpLabelDestructor( &pDHA->Lbl);
	MEM_FREE(pDHA);
    }

    MEM_FREE(pDH);
    return TOP_TRUE;
}

static TOPLogical
_xpCplxSeqDhConfig( XPSeqDataExt *pExt) {
    XPCplxSeqDH		*pDH = (XPCplxSeqDH*) pExt->userData;
    XPCplxSeqDHA	*pDHA = pDH->DHA;
    XPViewPart		*pVP;
    XPCplxSeqSnap	*pSeq;
    int			ret;
    int			resol = pExt->vResol;
    
    if ( pDH == NULL || pExt->xfaX == NULL ) {
	/* MUST have domain axis and cooresponding xfa */
	return TOP_FALSE;
    }
    pVP = xpAgentGetCurVP( &pDHA->CM);
    if ( (pSeq = (XPCplxSeqSnap*) xpViewPartGetSnap( pVP, XPGS_AsIs)) == NULL )
	return TOP_FALSE;
    if ( pDHA->serialX == pExt->xfaX->serial )
	return TOP_TRUE;

    if ( pVP->RangeOwner != &pDHA->CM ) {
	int n = XP_CPLXSEQ_NumIndex(pSeq);
        pDHA->domainIdxLo = xpCplxSeqFindByIndex( pSeq, pExt->wDomainLo,
	  XPWh_Before);
        if ( pDHA->domainIdxLo < 0 ) pDHA->domainIdxLo = 0;
        pDHA->domainIdxHi = xpCplxSeqFindByIndex( pSeq, pExt->wDomainHi,
	  XPWh_After);
        if ( pDHA->domainIdxHi > n-1 ) pDHA->domainIdxHi = n-1;
    } else {
	/* calculate the index range based on the domain axis */
	double	wDelta = pSeq->DomStep; 
	if ( TOP_IsOff(pSeq->Flags,XPCSeqF_Discrete) || wDelta <= 0.0 ) {
	    /* continuous: put a draw point at every vResol pixels */
	    int numPts = (pExt->vWidth+resol-1)/resol+1;
	    ret = xpCplxSeqSetIndexXfa( pSeq, pExt->xfaX, numPts, 0, resol);
	} else {
	    /* discrete: put a draw point at the DomStep increments */
	    double	wStart = xpXfaToWin( pExt->xfaX, 0);
	    double	wEnd =   xpXfaToWin( pExt->xfaX, pExt->vWidth);
	    double	iStart, iEnd;
	    int		numPts;

	    iStart = floor(wStart/wDelta);
	    iEnd   = ceil(wEnd/wDelta);
	    numPts = nint(iEnd-iStart)+1;
	    ret = xpCplxSeqSetIndexLin( pSeq, numPts, iStart * wDelta, wDelta);
	}
	pDHA->domainIdxLo = 0;
	pDHA->domainIdxHi = XP_CPLXSEQ_NumIndex(pSeq)-1;
	if ( ret )
	    pVP->Flags &= ~XPViewF_Valid;
    }
    pDHA->serialX = pExt->xfaX->serial;
    return TOP_TRUE;
}

static TOPLogical
_xpCplxSeqDhMap( XPSeqDataExt *pExt) {
    XPCplxSeqDH		*pDH = (XPCplxSeqDH*) pExt->userData;
    XPCplxSeqDHA	*pDHA = pDH->DHA;
    XPCplxSeqSnap	*pSeq;
    TOPVector		*vPts = &pExt->vPtsVec;
    XPoint		*vPt;
    int			lidx, hidx, vPtNum;
    double		xval, yval, ymin = 0.0, ymax = 0.0;

    if ( (pSeq = xpCplxSeqGetSnap( &pDHA->CM, XPGS_Upd)) == NULL )
	return TOP_FALSE;
    xpCplxSeqCalc( pSeq, pDH->valMask);
    lidx = pDHA->domainIdxLo;
    hidx = pDHA->domainIdxHi;
    if ( lidx < 0 || hidx<lidx || hidx >= XP_CPLXSEQ_NumIndex(pSeq) 
      || hidx >= XP_CPLXSEQ_Num(pSeq, &pDH->Wh) ) {
        topVecClear( vPts);
    } else {
	topVecSetNum( vPts, hidx-lidx+1);
	XPCPLXSEQ_SUBIDX_LOOP_FORW_BEGIN( pSeq, &pDH->Wh, lidx, hidx+1,
	  ptNum, pVal) {
	    vPtNum = ptNum - lidx;
	    vPt = &TOPVEC_Get(vPts, vPtNum, XPoint);
	    xval = XP_CPLXSEQ_GetIndex(pSeq,ptNum);
	    vPt->x = xpXfaToView( pExt->xfaX, xval);
	    yval = *pVal;
	    vPt->y = xpXfaToView( pExt->xfaY, yval);
	    /* printf( "seq: %g %d %d\n", xval, vPt->x, vPt->y); */
	    if ( yval < ymin || vPtNum==0 )  ymin = yval;
	    if ( yval > ymax || vPtNum==0 )  ymax = yval;
	} XPCPLXSEQ_SUBIDX_LOOP_FORW_END();
    }
    pExt->vZero = xpXfaToView( pExt->xfaY, 0.0);
    pExt->minVal = ymin;
    pExt->maxVal = ymax;
    if ( pSeq->PntVP->RangeOwner == &pDHA->CM ) {
	pExt->minDom = pSeq->SigDomMin;
	pExt->maxDom = pSeq->SigDomMax;
    } else {
	pExt->minDom = XP_CPLXSEQ_GetIndex(pSeq,0);
	pExt->maxDom = XP_CPLXSEQ_GetIndex(pSeq,XP_CPLXSEQ_NumIndex(pSeq)-1);
    }
    return TOP_TRUE;
}

static TOPLogical
_xpCplxSeqDhInsertPts( XPSeqDataExt *pExt, TOPDblPoint2 *pts, int num) {
    return TOP_FALSE;
}

static TOPLogical
_xpCplxSeqDhDeletePts( XPSeqDataExt *pExt, double wLo, double wHi) {
    return TOP_FALSE;
}

static int
_xpCplxSeqDhGetIndex( XPSeqDataExt *pExt, double val) {
    XPCplxSeqDH		*pDH = (XPCplxSeqDH*) pExt->userData;
    XPCplxSeqDHA	*pDHA = pDH->DHA;
    XPCplxSeqSnap	*pSeq;

    if ( (pSeq = xpCplxSeqGetSnap( &pDHA->CM, XPGS_AsIs)) == NULL )
	return -1;
    return xpCplxSeqFindByIndex( pSeq, val, XPWh_Near);
}

static TOPLogical
_xpCplxSeqDhGetPts( XPSeqDataExt *pExt, int lidx, TOPDblPoint2 *pts, int num) {
    XPCplxSeqDH		*pDH = (XPCplxSeqDH*) pExt->userData;
    XPCplxSeqDHA	*pDHA = pDH->DHA;
    XPCplxSeqSnap	*pSeq;
    int			nidx = lidx+num;

    if ( (pSeq = xpCplxSeqGetSnap( &pDHA->CM, XPGS_AsIs)) == NULL )
	return TOP_FALSE;
    if ( lidx < 0 || nidx<=lidx || nidx > XP_CPLXSEQ_NumIndex(pSeq) )
	return TOP_FALSE;
    XPCPLXSEQ_SUBIDX_LOOP_FORW_BEGIN( pSeq, &pDH->Wh, lidx, nidx, ptNum, pVal) {
	pts[ptNum-lidx].x = XP_CPLXSEQ_GetIndex(pSeq, ptNum);
	pts[ptNum-lidx].y = *pVal;
    } XPCPLXSEQ_SUBIDX_LOOP_FORW_END();
    return TOP_TRUE;
}

static TOPLogical
_xpCplxSeqDhSetPts( XPSeqDataExt *pExt, int lidx, TOPDblPoint2 *pts, int num) {
    XPCplxSeqDH		*pDH = (XPCplxSeqDH*) pExt->userData;
    XPCplxSeqDHA	*pDHA = pDH->DHA;
    XPCplxSeqSnap	*pSeq;
    int			nidx = lidx+num;
    XPViewPart		*pVP;

    pVP = xpAgentGetCurVP( &pDHA->CM);
    if ( (pSeq = (XPCplxSeqSnap*) xpViewPartGetSnap( pVP, XPGS_AsIs)) == NULL )
	return TOP_FALSE;
    if ( lidx < 0 || nidx<=lidx || nidx > XP_CPLXSEQ_NumIndex(pSeq) )
	return TOP_FALSE;
    XPCPLXSEQ_SUBIDX_LOOP_FORW_BEGIN( pSeq, &pDH->Wh, lidx, nidx, ptNum, pVal) {
	/* XXX: what about .x values (seq index) */
	*pVal = pts[ptNum-lidx].y;
    } XPCPLXSEQ_SUBIDX_LOOP_FORW_END();
    xpViewPartNotify( pSeq->PntVP, XPNW_Idle, XPVE_Modify, &pDHA->CM);
    return TOP_TRUE;
}

static XPSeqDataHandler _XpCplxSeqDataHandler = {
    _xpCplxSeqDhConstructor,
    _xpCplxSeqDhDestructor,
    _xpCplxSeqDhConfig,
    _xpCplxSeqDhMap,
    _xpCplxSeqDhInsertPts,
    _xpCplxSeqDhDeletePts,
    _xpCplxSeqDhGetIndex,
    _xpCplxSeqDhGetPts,
    _xpCplxSeqDhSetPts,
};

int
xpCplxSeqDhRegisterCmds( Tcl_Interp *ip) {
    xpRegisterDataHandler( ip, "xpole", "XPSeqDataHandler",
      &_XpCplxSeqDataHandler);
    return TCL_OK;
}

