/* 
    filter.c

    This implements the overall structure for relating various
    aspects of a single filter together.  For now, this means
    mostly just handling the user interface.

    90/9/30 KDW: Created as zplane.c
    91/01/09 KDW: Extracted graphics stuff into plane.c
    91/Mar/22 KDW: Extracted menu stuff into filter.c

    Author: Kennard White

    Copyright (c) 1990-93 The Regents of the University of California.
    All rights reserved.  See the file "./copyright.h" for full copyright
    notice, limitation of liability, and disclaimer of warranty provisions.
#include "copyright.h"				COPYRIGHTENDKEY
*/
#if !defined(lint) && !defined(SABER)
static char rcsid[] = "$Header: /vol/cooley/cooley1/kennard/src/xpole/RCS/fltr.c,v 1.5 93/08/27 14:28:21 kennard Exp $";
#endif

#include "topFixup.h"
#include <tcl.h>

#include "topStd.h"
#include "topCB.h"

#include "pole.h"
#include "filter.h"
/* #include "cplxseq.h" */
/* #include "silar.h" */

static char _XpFltrSubPackage[] = "xp.fltr";
#define SPKG _XpFltrSubPackage

/************************************************************************


************************************************************************/

void xpDestroy() {
    exit( 0 );
}

XPXformDomain
xpDomainNameToType(char name) {
    switch ( TOP_ToLower(name) ) {
    case 's':		return XP_XFDOM_S;
    case 'z':		return XP_XFDOM_Z;
    }
    return XP_XFDOM_UNKNOWN;
}

char
xpDomainTypeToName(XPXformDomain t) {
    switch ( t ) {
    case XP_XFDOM_S:		return 's';
    case XP_XFDOM_Z:		return 'z';
    }
    return '?';
}

TOPLogical
xpFltrParseName( char *name, XPXformDomain defaultDomain,
  char **nameEnd, char *base, XPXformDomain *d) {
    char		*s, *b;

    s = name;
    while (isspace(*s))
        s++;
    if (nameEnd != NULL)
	*nameEnd = s;
    for (b=base; isalnum(*s) || *s == '_' || *s == '-'; )
	*b++ = *s++;
    *b++ = '\0';
    *d = XP_XFDOM_UNKNOWN;
    if ( *s == '(' ) {
	s++;
    	while (isspace(*s))
	    s++;
	if ( *s == ')' ) {
	    *d = defaultDomain;
	} else {
	    if ( ((*d)=xpDomainNameToType(*s)) == XP_XFDOM_UNKNOWN )
	    	return TOP_FALSE;
	    s++;
    	    while (isspace(*s))
	        s++;
	    if (*s != ')' ) {
	        return TOP_FALSE;
	    }
	}
	s++;
    } else {
	*d = defaultDomain;
    }
    while (isspace(*s))
	s++;
    if (nameEnd != NULL)
	*nameEnd = s;
    return TOP_TRUE;
}

XPFilter*
xpFltrFindByName( char *name, XPXformDomain defaultDomain) {
    char		baseName[XP_FLTR_MAX_NAME_LEN], *nameEnd;
    XPXformDomain	dom;

    if ( xpFltrParseName( name, defaultDomain, &nameEnd, baseName, &dom) 
      == TOP_FALSE
      || *nameEnd != '\0' ) {
	return NULL;
    }
    DLLIST_LOOP_FORW_BEGIN(XPTheMachine.FltrList, XPFilter*, pFltr) {
	if ( pFltr->Name && strcasecmp( pFltr->Name, baseName)==0 
	  && (dom == XP_XFDOM_UNKNOWN || pFltr->XfDomain == dom) ) {
	    return pFltr;
	}
    } DLLIST_LOOP_FORW_END();
    return NULL;
}


/**
    A fltrSpec has two forms:
	(1) fltrName
	(2) ""
    In the later case, {*ppFltr} must be non-NULL.

    In the former case, there might be a conflict with {*ppFltr}.
    If conflict occurs, {hintB} controls what happens:
      if true, then {fltrSpec} will dominate.
      if false, then an error is signaled.

    True is returned if a valid filter is specified, and {*ppFltr}
    indicates that filter.
**/
TOPLogical
xpFltrFindBySpec( char *spec, XPFilter **ppFltr, TOPLogical hintB) {
    char	*fltrName;
    XPFilter	*pFltr;

    if ( spec==NULL || *spec=='\0' ) {
	/* form (2) */
	if ( *ppFltr == NULL )
	    goto bad;		/* mal-formed */
	return TOP_TRUE;
    }
    fltrName = spec;
    if ( (pFltr=xpFltrFindByName( fltrName, XP_XFDOM_UNKNOWN)) == NULL )
	goto bad;	/* the fltr spec is ok, no such filter */

    if ( !hintB && *ppFltr != NULL && pFltr != *ppFltr )
	goto bad;	/* filter doesn't agree */

    *ppFltr = pFltr;
    return TOP_TRUE;
bad:
    *ppFltr = NULL;
    return TOP_FALSE;
}

void
xpFltrAddView( XPFilter *pFltr, XPView *pView) {
    pView->Idx = topVecAppendIdx( &pFltr->Views);
    TOPVEC_GET( &pFltr->Views, pView->Idx, XPView*) = pView;
    xpFltrListChange();
}

char*
xpGenFilterName() {
    static int XPFilterCounter = 0;
    static char buf[20];
    for(;;) {
	sprintf( buf, "H%d", ++XPFilterCounter);
	if ( xpFltrFindByName( buf, XP_XFDOM_UNKNOWN) == NULL )
	    break;
    }
    return memStrSave(buf);
}

void
xpFilterUpdate( XPFilter *pFltr) {
#if defined(XPOLE_SEP_OLD)
    if ( pFltr->ChangeMask & XP_CHNG_SEP ) {
	xpFilterUpdateFromSep( pFltr);
    }
#endif
    pFltr->ChangeMask = 0;
}


static int
_xpFltrConstructor( XPFilter *pFltr, Tcl_Interp *ip, 
  char *name, XPXformDomain xd) {
    char	buf[XP_FLTR_MAX_NAME_LEN];
    static int	_XpFltrSN = 0;

    MEM_CLEAR(pFltr);
    xpLabelConstructor( &pFltr->Lbl, (TOPPtr)pFltr, &_XpFltrSN, name,
      "XPFilter");
    pFltr->XfDomain = xd;
    pFltr->ChangeMask = 0;
    pFltr->Flags = 0;
    pFltr->Desc = NULL;
    if ( name == NULL || name[0] == '\0' ) {
	pFltr->Name = xpGenFilterName();
    } else {
	if ( xpFltrFindByName( name, xd) != NULL ) {
	    topTclAppendResultFmt(ip, "Filter with name ``%s'' already exists",
	      name);
	    return TCL_ERROR;
	}
	pFltr->Name = memStrSave( name);
    }
    sprintf( buf, "%s(%c)", pFltr->Name, xd == XP_XFDOM_S ? 's' : 'z');
    pFltr->XfName = memStrSave( buf);

    topPtrVecConstructor( &pFltr->Views);

    Tcl_CreateCommand( ip, pFltr->Name, _xpFltrCmds, (ClientData)pFltr, NULL);
    Tcl_CreateCommand( ip, pFltr->XfName, _xpFltrCmds, (ClientData)pFltr, NULL);

    return TCL_OK;
}

static void
_xpFltrDestructor( XPFilter *pFltr, Tcl_Interp *ip) {
    if ( pFltr->Name ) {
        Tcl_DeleteCommand( ip, pFltr->Name);
    }
    if ( pFltr->XfName ) {
        Tcl_DeleteCommand( ip, pFltr->XfName);
    }
#ifdef notdef
    DLLIST_LOOP_FORW_BEGIN( XPTheMachine.FltrBdgList, XPFilterBdg*, pBdg) {
	if ( pBdg->Filter == pFltr )
	    xpFltrBdgDestroy( pBdg);
    } DLLIST_LOOP_FORW_END();
#endif
    XPTheMachine.ChangeMask |= XP_CHNG_FLTR_EXIST;
}

static TOPStatus
_xpFltrListChangeCB( TOPPtr cbClosure, TOPPtr userData) {
    Tcl_Interp	*ip = XPTheMachine.ip;

    if ( TOP_IsOn(XPTheMachine.ChangeMask,XP_CHNG_FLTR_EXIST) ) {
    	Tcl_SetVar2( ip, "xpole", "filters", "changed", TCL_GLOBAL_ONLY);
	XPTheMachine.ChangeMask &= ~XP_CHNG_FLTR_EXIST;
    }
    return TOP_CB_REMOVE;
}

void
xpFltrListChange() {
    XPTheMachine.ChangeMask |= XP_CHNG_FLTR_EXIST;
    topIdleCallbacksAdd( _xpFltrListChangeCB, NULL);
}

XPFilter*
xpFltrCreate( Tcl_Interp *ip, char *name, XPXformDomain xd) {
    XPFilter	*pFltr;

    pFltr = (XPFilter*) dlListCreateAtTail( XPTheMachine.FltrList);
    if ( _xpFltrConstructor( pFltr, ip, name, xd) != TCL_OK ) {
	xpFltrDestroy( ip, pFltr);
	return NULL;
    }
    TOP_PDBG((SPKG,"filter %d: create", pFltr->Lbl.SN));
    xpFltrListChange();
    return pFltr;
}
    
void
xpFltrDestroy( Tcl_Interp *ip, XPFilter *pFltr) {
    _xpFltrDestructor( pFltr, ip);
    dlListElemDestroy( XPTheMachine.FltrList, pFltr);
    xpFltrListChange();
}
