/* 
    TCL interface to filter.c

    Author: Kennard White
    91/Sep/4 KDW: Created as filter_t.c

    Copyright (c) 1990-93 The Regents of the University of California.
    All rights reserved.  See the file "./copyright.h" for full copyright
    notice, limitation of liability, and disclaimer of warranty provisions.
#include "copyright.h"				COPYRIGHTENDKEY
*/
#if !defined(lint) && !defined(SABER)
static char rcsid[] = "$Header: /vol/cooley/cooley1/kennard/src/xpole/RCS/fltrTcl.c,v 1.5 93/08/27 14:28:19 kennard Exp $";
#endif

#include "topFixup.h"
#include <tcl.h>

#include "topStd.h"
#include "topTcl.h"

#include "pole.h"
#include "filter.h"

/**
    Format info about a single, pre-existing filter.
    info = name xfd xfname exist
**/
static void
_xpFltrInfoFmtTcl( Tcl_Interp *ip, XPFilter *pFltr, TOPLogical nestB) {
    char		buf[100];

    if ( nestB )		Tcl_AppendResult(ip," {", NULL);
    Tcl_AppendElement(ip,pFltr->Name,TOP_FALSE);
    buf[0] = xpDomainTypeToName(pFltr->XfDomain);
    buf[1] = '\0';
    Tcl_AppendElement(ip,buf,TOP_FALSE);
    Tcl_AppendElement(ip,pFltr->XfName,TOP_FALSE);
    Tcl_AppendElement(ip,"exist",TOP_FALSE);
    if ( nestB )		Tcl_AppendResult(ip,"} ", NULL);
}

/**
    Format a list of all the views in {pFltr}.
**/
static void
_xpFltrViewsFmtTcl( Tcl_Interp *ip, XPFilter *pFltr, int nestB) {
    if ( nestB )		Tcl_AppendResult(ip," {",NULL);
    XPVIEW_LOOP_FORW_BEGIN(pFltr, pView) {
	if ( pView == NULL ) continue;
	_xpViewInfoFmtTcl( ip, pView, TOP_TRUE);
    } XPVIEW_LOOP_FORW_END();
    Tcl_AppendResult(ip," }",NULL);
    if ( nestB )		Tcl_AppendResult(ip," }",NULL);
}

/**
    Format info about a list of filters that may or may not exist.
**/
static int
_xpFltrInfoListFmtTcl( Tcl_Interp *ip, char *namespec, TOPLogical nestB, 
  TOPLogical getB) {
    char		name[XP_FLTR_MAX_NAME_LEN], *namePtr, *nameEnd;
    XPXformDomain	domain;
    char		domchar;
    XPFilter		*pFltr;

    if ( nestB )		Tcl_AppendResult(ip," {", NULL);

    for ( namePtr=namespec; *namePtr != '\0'; namePtr=nameEnd) {
	if ( !xpFltrParseName( namePtr, XP_XFDOM_UNKNOWN, &nameEnd, 
	  name, &domain) || namePtr == nameEnd ) {
	    Tcl_ResetResult(ip);
	    Tcl_AppendResult(ip,"invalid filter spec: ``",namePtr,"''",NULL);
	    return TCL_ERROR;
	}
	if ( (pFltr=xpFltrFindByName( name, XP_XFDOM_UNKNOWN)) == NULL ) {
	    if ( getB ) {
		if ( (pFltr = xpFltrCreate( ip, name, domain)) == NULL ) {
		    Tcl_ResetResult(ip);
		    Tcl_AppendResult(ip,"creation of filter ``",pFltr,
		      "'' failed.", NULL);
		    return TCL_ERROR;
		}
	    }
	}
	if ( pFltr == NULL ) {
	    domchar = xpDomainTypeToName(domain);
	    topTclAppendResultFmt( ip, "{ %s %c \"%s(%c)\" notexist } ",
	      name, domchar, name, domchar);
	} else {
	    _xpFltrInfoFmtTcl( ip, pFltr, TOP_TRUE);
	}
    }
    if ( nestB )		Tcl_AppendResult(ip,"}", NULL);
    return TCL_OK;
}

/**
    This implements commands bound to a particular (pre-existing) filter.
    This function is daisy-chained from the "filter" command; to allow
    for this, the command keyword {cmd} is passed explictly, and {aC,aV}
    form the arguments to the {cmd}.  Also, TCL_CONTINUE is returned
    if the command is not recognized.
**/
int
_xpFltrCmdsPrs( XPFilter *pFltr, Tcl_Interp *ip, char *cmd, int aC, char **aV) {
    char		c;
    int			len;

    if ( cmd==NULL || pFltr==NULL ) {
	Tcl_AppendResult(ip,"fltr_cmds: canonical|description|destroy|trajectory|xfname",NULL);
	return TCL_ERROR;
    }
    c = cmd[0]; len = strlen(cmd);

    /*IF*/ if (c=='c' && strncmp(cmd, "canonical",len)==0) {
	if ( aC!=0 )	goto extra_args;
	Tcl_SetResult( ip, pFltr->Name, TCL_STATIC);
    } else if ( c=='d' && strncmp(cmd,"description",len)==0 ) {
	if ( aC==0 ) {
	    Tcl_SetResult(ip,pFltr->Desc?pFltr->Desc:"",TCL_STATIC);
	} else {
	    MEM_FREE(pFltr->Desc);
	    pFltr->Desc = memStrSave(aV[0]);
	}
    } else if ( c=='d' && strncmp(cmd,"destroy",len)==0 ) {
	xpFltrDestroy( ip, pFltr);
    } else if ( c=='i' && strncmp(cmd,"info",len)==0 ) {
	_xpFltrInfoFmtTcl( ip, pFltr, TOP_FALSE);
    } else if ( c=='x' && strncmp(cmd,"xfname",len)==0 ) {
	Tcl_SetResult( ip, pFltr->XfName, TCL_STATIC);
#ifdef notdef
    } else if ( strcasecmp( aV[1], "save")==0 ) {
	sprintf( buf, "%s.tcl", pFltr->Name);
	if ( (f = fopen( buf, "w")) == NULL ) {
	    return topTclErrFmt( ip, "%s: cant open file ``%s''",
	      aV[1], buf);
	}
	_xpSaveSilarDBTcl( pDB, f);
	fclose( f);
    } else if ( strcasecmp( aV[1], "load")==0 ) {
	sprintf( buf, "%s.tcl", pFltr->Name);
	if ( (f = fopen( buf, "r")) == NULL ) {
	    return topTclErrFmt( ip, "%s: cant open file ``%s''",
	      aV[1], buf);
	}
	fclose( f);
	xpTclCmdFmt("source %s; ReCreateSilarDB %s", buf, pFltr->Name);
    } else if ( strcasecmp( aV[1], "export")==0 ) {
	if ( aC < 3 )
	    return topTclErrFmt( ip, "usage: %s %s ipcName", aV[0], aV[1]);
	xpCreateExport( pFltr, aV[2]);
#endif
    } else if ( c=='v' && strncmp(cmd,"views",len)==0 ) {
	_xpFltrViewsFmtTcl( ip, pFltr, TOP_FALSE);
    } else {
	XPView		*pView;
	int		r = TCL_ERROR;
	if ( aC == 0 ) {
	    Tcl_AppendResult(ip,"fltr_cmd ",cmd,": Expected view arg.",
	      NULL);
	    return TCL_ERROR;
	}
	/*
	 * Try chaining onto other commands.
	 */
	if ( (pView=xpViewFindBySpec( aV[0], NULL, pFltr, TOP_FALSE)) != NULL ){
	    /* cmd=viewcmd; aV=viewname viewcmdargs ... */
	    if ( (r = _xpViewCmdsPrs( pView, ip, cmd, aC-1, aV+1))==TCL_OK)
		return TCL_OK;
	    if ( r!=TCL_CONTINUE ) {
		Tcl_AppendResult(ip,"\nfltr_cmd ",cmd,": view ``",aV[0],
		  "'' command failed.",NULL);
		return TCL_ERROR;
	    }
	}
	if ( r == TCL_CONTINUE )
	    return TCL_CONTINUE;
	Tcl_AppendResult(ip,"fltr_cmd ",cmd,": Not view: ``",
	  aV[0],"''",NULL);
	return TCL_ERROR;
    }
    return TCL_OK;
extra_args:
    Tcl_AppendResult(ip,"fltr_cmd ",cmd,": extra arguments",NULL);
    return TCL_ERROR;
}

int
_xpFltrCmds( ClientData cld, Tcl_Interp *ip, int aC, char **aV) {
    XPFilter		*pFltr = (XPFilter*) cld;
    int			r;

    if ( (r = _xpFltrCmdsPrs( pFltr, ip, aV[1], aC-2, aV+2)) == TCL_OK )
	return TCL_OK;
    if ( r==TCL_CONTINUE ) {
	Tcl_AppendResult(ip,aV[0],": invalid command ``",aV[1],"''",NULL);
    } else {
	Tcl_AppendResult(ip,"\n",aV[0]," ",aV[1],": command failed",NULL);
    }
    return TCL_ERROR;
}

int
_xpFltrDotCmd( ClientData cld, Tcl_Interp *ip, int aC, char **aV) {
#ifdef notdef
    XPFilter		*pFltr;
    pFltr = xpFltrFindCur();
    if ( pFltr == NULL ) 
	return topTclErrFmt( ip, "%s: No filters defined.", aV[0]);

    return _xpFltrCmds( (ClientData)pFltr, ip, aC, aV);
#endif
    return TCL_ERROR;
}


int
_xpFilterCmd( ClientData cld, Tcl_Interp *ip, int aC, char **aV) {
    char		name[XP_FLTR_MAX_NAME_LEN], *nameEnd, *names;
    XPXformDomain	domain;
    XPFilter		*pFltr;
    int			i, len, r;
    char		*cmd, c;

    if (aC < 2) {
	Tcl_AppendResult(ip,"usage: ",aV[0]," create|info", NULL);
	return TCL_ERROR;
    }
    cmd = aV[1]; c = *cmd; len = strlen(cmd);
    /*IF*/ if (c=='c' && strncmp(cmd, "create",len)==0) {
	if (aC != 3 && aC != 4) {
	    Tcl_AppendResult(ip,"usage: ",aV[0]," ",aV[1],
	      " name ?domain?", NULL);
	    return TCL_ERROR;
	}
	domain = aC >= 4 ? xpDomainNameToType( *aV[3]) : xpGetDefXfDomain();
	if ( xpFltrParseName( aV[2], domain, 
	  &nameEnd, name, &domain) == TOP_FALSE
	  || *nameEnd != '\0') {
	    Tcl_AppendResult(ip,aV[0]," ",aV[1],": mal-formed filter name ``",
	      aV[2],"''", NULL);
	    return TCL_ERROR;
	}
	if ( domain==XP_XFDOM_UNKNOWN ) {
	    Tcl_AppendResult(ip,aV[0]," ",aV[1],
	      ": Invalid or missing domain for ``", aV[2],"''",NULL);
	    return TCL_ERROR;
	}
	if ( (pFltr=xpFltrCreate( ip, name, domain)) == NULL )
	    return TCL_ERROR;
	Tcl_SetResult( ip, pFltr->Name, TCL_STATIC);
    } else if (c=='d' && strncmp(cmd, "destroy",len)==0) {
	if (aC != 3) {
	    topTclErrFmt( ip, "usage: %s %s names", aV[0], aV[1]);
	    return TCL_ERROR;
	}
	for (names=aV[2]; *names; names=nameEnd) {
	    if ( xpFltrParseName( names, XP_XFDOM_UNKNOWN,
	      &nameEnd, name, &domain) == TOP_FALSE) {
		return topTclErrFmt( ip, "%s %s: mal-formed filter name ``%s''",
		  aV[0], aV[1], aV[2]);
	    }
	    if ( (pFltr=xpFltrFindByName( name, XP_XFDOM_UNKNOWN)) == NULL ) {
		return topTclErrFmt( ip, "%s %s: no such filter``%s''",
		  aV[0], aV[1], aV[2]);
	    }
	    xpFltrDestroy( ip, pFltr);
	}
    } else if (c=='f' && strncmp(cmd, "fltrlist",len)==0) {
	if ( aC == 2 ) {
	    DLLIST_LOOP_FORW_BEGIN( XPTheMachine.FltrList, XPFilter*, pFltr) {
		_xpFltrInfoFmtTcl( ip, pFltr, TOP_TRUE) ;
	    } DLLIST_LOOP_FORW_END();
        } else {
	    for (i=2; i < aC; i++) {
		if ( _xpFltrInfoListFmtTcl( ip, aV[i], TOP_FALSE, TOP_FALSE)
		  != TCL_OK ) {
		    Tcl_AppendResult(ip,"\n",aV[0]," ",aV[1],": bad spec ``",
		      aV[i],"''",NULL);
		    return TCL_ERROR;
		}
	    }
	}
    } else if (c=='g' && strncmp(cmd, "get",len)==0) {
	for (i=2; i < aC; i++) {
	    if ( _xpFltrInfoListFmtTcl( ip, aV[i], TOP_FALSE, TOP_TRUE) != TCL_OK ){
		Tcl_AppendResult(ip,"\n",aV[0]," ",aV[1],": bad spec ``",
		  aV[i],"''",NULL);
		return TCL_ERROR;
	    }
	}
    } else if (c=='a' && strncmp(cmd,"autoname",len)==0) {
        Tcl_SetResult( ip, xpGenFilterName(), TCL_DYNAMIC);
    } else {
	/*
	 * NOW do convienence chaining onto other commands...
	 */
	int		skip;
	if ( c=='c' && strncmp(cmd,"cmdfltr",len)==0 ) {
	    /* filter cmdfltr subcmd fltrname subcmdargs ... */
	    cmd = aV[2]; skip = 1;
	} else {
	    /* filter subcmd fltrname subcmdargs ... */
	    skip = 0;
	}

	if ( aC < 3+skip ) {
	    Tcl_AppendResult(ip,aV[0]," ",aV[1],": Expected fltr arg.",NULL);
	    return TCL_ERROR;
	}
	if ( (pFltr=xpFltrFindByName( aV[2+skip], XP_XFDOM_UNKNOWN))==NULL ) {
	    Tcl_AppendResult(ip,aV[0]," ",aV[1],": bad filter spec ``",
	      aV[2+skip],"''",NULL);
	    return TCL_ERROR;
	}
	if ( (r=_xpFltrCmdsPrs( pFltr, ip, cmd, aC-3-skip, aV+3+skip))==TCL_OK)
	    return TCL_OK;
	if ( r == TCL_CONTINUE ) {
	    Tcl_AppendResult(ip,aV[0]," invalid command: ``",cmd,"''",NULL);
	    return TCL_ERROR;
	}
	Tcl_AppendResult(ip,"\n",aV[0]," ",cmd,": cmd failed.",NULL);
	return TCL_ERROR;
    }
    return TCL_OK;
}
