#
# User Interface to XPole agents.
#


proc xpAD.MakeInfoArea { win } {
    global XpLabelFont
    set dd $win.head1; pack append $win [frame $dd] {top fillx}
    pack append $dd [label $dd.namelbl -text "Name:" -font $XpLabelFont] {left}
    pack append $dd [label $dd.name -width 30 -anc w] {left}
    pack append $dd [label $dd.clslbl -text "Class:" -font $XpLabelFont] {left}
    pack append $dd [label $dd.class -width 20 -anc w] {left}

    set dd $win.head2; pack append $win [frame $dd] {top fillx}
    pack append $dd [label $dd.desclbl -text "Desc:" -font $XpLabelFont] {left}
    pack append $dd [label $dd.desc -anc w] {left}

    set dd $win.head3; pack append $win [frame $dd] {top fillx}
    pack append $dd [label $dd.viewlbl -text "View:" -font $XpLabelFont] {left}
    pack append $dd [label $dd.view -anc w] {left}
}

proc xpAD.SetVar { ent n1 n2 } {
    upvar #0 $n1 vars
    set vars($n2) [$ent get]
    topEntry.return $ent
}

proc xpAD.MakeVarEntry { lent label n1 n2 } {
    upvar #0 $n1 vars
    if ![info exist vars($n2)] {
	set vars($n2) ""
    }
#    trace var vars($n2) w
    topWin.LbldEntry $lent [list -text $label] {-relief sunken} {}
    bind $lent.entry <Return> "xpAD.SetVar %W $n1 $n2"
    topEntry.set $lent.entry $vars($n2)
    return $lent
}

proc xpAD.SetAgInfo { win ag } {
    set pipeB 0
    if [catch {agent info $ag} aginfo] {
	set pipeB 1
        if [catch {xppipe info $ag} aginfo] {
	    set aginfo "$ag Unknown Unknown"
	}
    }
    set name [lindex $aginfo 0]
    set class [lindex $aginfo 1]
    $win.head1.name conf -text $name
    $win.head1.class conf -text $class
    set desc $win.head2.desc
    if { $pipeB } {
	$win.head3.view conf -text N/A
    } else {
	$win.head3.view conf -text [lindex $aginfo 4]
    }

    set dd $win.param; catch {destroy $dd}
    pack append $win [frame $dd] {top expand}
    case $class {
      SilarDH {
	$desc conf -text "Data handler used by Pole-Zero Window."
      }
      CplxSeqDHA {
	$desc conf -text "Data handler used by CplxSeq Window."
      }
      PipeAg {
        set pipename [lindex $aginfo 5]
	$desc conf -text "Agent for pipe $pipename."
      }
      SilarFourier {
	$desc conf -text "Calculate fourier from pole-zero via direct eval."
      }
      ImpulseFourier {
	$desc conf -text "Calculate fourier from impulse via DTFT."
	pack append $dd [xpAD.MakeVarEntry $dd.winsize "window size" $name windowsize] {top}
	pack append $dd [xpAD.MakeVarEntry $dd.wintype "window type" $name windowtype] {top}
      }
      {PolezeroMap SilarMap} {
	$desc conf -text "Map polezeros into another set of polezeros."
	pack append $dd [xpAD.MakeVarEntry $dd.maptype "maptype" $name maptype] {top}
	pack append $dd [xpAD.MakeVarEntry $dd.numzeros "numzeros" $name numzeros] {top}
      }
      SpectralXform {
	$desc conf -text "Map polezeros into another set of polezeros."
	pack append $dd [xpAD.MakeVarEntry $dd.maptype "maptype" $name Type] {top}
	pack append $dd [xpAD.MakeVarEntry $dd.oldlo "oldLo" $name OldLo] {top}
	pack append $dd [xpAD.MakeVarEntry $dd.oldhi "oldHi" $name OldHi] {top}
	pack append $dd [xpAD.MakeVarEntry $dd.newlo "newLo" $name NewLo] {top}
	pack append $dd [xpAD.MakeVarEntry $dd.newhi "newHi" $name NewHi] {top}
      }
      default {
	$desc conf -text "Unknown Agent Class"
      }
    }
}

proc xpAD.Box_B1 { dlg y } {
    set lb $dlg.sel.box
    $lb select from [$lb nearest $y]

    set ag [$lb get [$lb cursel]]
    xpAD.SetAgInfo $dlg.info $ag
}

proc xpAD.FillBox { dlg } {
    set lb $dlg.sel.box
    set curidx [$lb cursel]
    set cursel [expr { "$curidx"=="" ? "" : [$lb get $curidx] }]
    $lb delete 0 end
    foreach ag "[xppipe info] [agent info]" {
	lappend agnames [lindex $ag 0]
    }
    foreach agname [lsort $agnames] {
	$lb insert end $agname
	if { "$agname"=="$cursel" } {
	    $lb select from end
	}
    }
}

proc xpAD.TraceFilters { dlg n1 n2 op } {
    if [catch {xpAD.FillBox $dlg} error] {
	global errorInfo
	xpStd.DlgErr "" AgentDlg $error $errorInfo
    }
}

proc xpAgentDlg { } {
    global xpole
    set dlg [xpStdDlgPopup .agent "" "Agent Info" "(xpole)AgentDlg"]

    set dd $dlg.sel; pack append $dlg [frame $dd] {left fill}
    pack append $dd [listbox $dd.box -relief sunken -export no -geom 25x10] \
      {left fill}
    tk_listboxSingleSelect $dd.box
    foreach b "<Button-1> <B1-Motion> <Shift-1> <Shift-B1-Motion>" {
        bind $dd.box $b "xpAD.Box_B1 $dlg %y"
    }
    pack append $dd [scrollbar $dd.sb -com "$dd.box yview"] {left fill}
    $dd.box conf -yscroll "$dd.sb set"
    xpAD.FillBox $dlg
    trace var xpole(filters) w "xpAD.TraceFilters $dlg"

    set dd $dlg.info; pack append $dlg [frame $dd] {left fill}
    xpAD.MakeInfoArea $dd
}
