#
# cseqgraph.tcl :: Complex Sequence Graph :: XPole
#
# This is a Tk script to implement the windows and user interface
# for XPole's complex sequence windows.  This window allows input/output
# of the complex sequences within XPole including fourier (frequency) response
# and impulse (time) response.
#
# The core of this window is the "xpseqwidget" widget implemented by "seqTk.c".
# That widget does the real work...this is just the fluff.
#
proc xpCSG.Reset { w } {
}

proc xpCSG.ConfigView { graph viewspec } {
    upvar #0 $graph.$viewspec vdvars
    set glrow $vdvars(glrow)
    set dlg [xpStdDlgPopup $glrow "" "$viewspec Config" "(xpole)CSG ConfigView"]
    pack append $dlg [topWin.LbldEntry $dlg.num {-text NumSamps -width 10} \
      {-rel sunken -width 10} {-rel raised -bd 2}] {top}
    pack append $dlg [topWin.LbldEntry $dlg.step {-text Step -width 10} \
      {-rel sunken -width 10} {-rel raised -bd 2}] {top}

    set dd $dlg.buts; pack append $dlg [frame $dd] {top fill}
    pack append $dd [button $dd.ok -text Ok \
      -com "xpCSG.CV.Ok $graph $viewspec $dlg"] {left fill}
    pack append $dd [button $dd.apply -text Apply \
      -com "xpCSG.CV.Apply $graph $viewspec $dlg"] {left fill}
    pack append $dd [button $dd.cancel -text Cancel \
      -com "xpCSG.CV.Popdown $graph $viewspec $dlg"] {left fill}
}

proc xpCSG.CV.Popdown { graph viewspec dlg } {
    destroy $dlg
}

proc xpCSG.CV.Ok { graph viewspec dlg } {
    xpCSG.CV.Apply $graph $viewspec $dlg
    xpCSG.CV.Popdown $graph $viewspec $dlg
}

proc xpCSG.CV.Apply { graph viewspec dlg } {
    upvar #0 $graph.$viewspec vdvars
    set glrow $vdvars(glrow)

    set numsamp [$dlg.num.entry get]
    view index $viewspec $numsamp 0.0 1.0
    view event $viewspec
}

proc _xpCSG.ParseSeqSpec { graph viewspec_var comps_var defcomps } {
    upvar $viewspec_var viewspec $comps_var comps
    upvar #0 $graph.draw svars

    set sds [split $viewspec ","]
    if {[llength $sds]>1} then {
	set viewspec [lindex $sds 0]
	set comps [lrange $sds 1 end]
    } 

    # Cononicalize the viewspec
    set viewinfo [xpView.GetByClass $viewspec CplxSeq$svars(subClass)]
    set viewspec [lindex $viewinfo 0].[lindex $viewinfo 1]

    # Default empty comps
    set nc [llength $comps]
    if {$nc==0} {
	if { "$defcomps"!="" } {
	    set comps $defcomps
	} else {
	    case $svars(subClass) {
	      Freq	{ set comps "m p" }
	      default	{ set comps "r" }
	    }
	}
    }

    # Cononicalize the comps
    set realcomps ""
    foreach comp $comps {
	case $comp in {
	  {r*}	{ lappend realcomps r }
	  {i*}	{ lappend realcomps i }
	  {m*}	{ lappend realcomps m }
	  {p*}	{ lappend realcomps p }
	  {w*}	{ lappend realcomps wp }
	  default	{
	    error "Invalid component ``$comp'' in ``$viewspec''"
	  }
	}
    }
    set comps $realcomps
}

#
# If {viewspec} doenst specify any comps and, {comps} is empty,
# then the entire view will be removed.
#
proc xpCSG.DelSeq { graph viewspec {comps ""} } {
    set origcomps $comps
    _xpCSG.ParseSeqSpec $graph viewspec comps "r i m p wp"
    upvar #0 $graph.$viewspec vdvars

#puts stdout "xpCSG.DelSeq $viewspec $comps"
    foreach comp $comps {
	if [xpSSG.SeqExistB $graph $viewspec,$comp] {
	    # Setting vdvars() will trigger a trace that will recursively
	    # invoke us.  This is further complicated b/c the actual
	    # line delete occurs in idle time.  We depend on SeqExistB
	    # knowing if there is an idle delete pending.
            xpSSG.DelLine $graph $viewspec,$comp
	    set vdvars($comp) 0
	}
    }

    if { "$origcomps"=="" } {
	set glrow $vdvars(glrow)
	# The unset prob. wont do much good, since stuff in the glrow
	# is still tracing it.  Sigh.
#puts stdout "DelSeq: killing entire view"
	unset vdvars
        # Delay destroy so unpost by button-release wont fail.
	after 1 topWin.SafeDestroy $glrow
    }
}

#
# Check to see if {viewspec} with graph legend row glrow is empty:
# e.g., if no components of it are displayed.
# If so, remove it completely from the graph.
#
proc xpCSG.EmptyCheck { graph viewspec glrow } {
    upvar #0 $graph.$viewspec vdvars
    if { ![topWin.ExistB $glrow] } return
    if { [llength [winfo children $glrow]] <= 1 } {
#puts stdout "EmptyCheck: Cleaning $viewspec"
	unset vdvars
	destroy $glrow
    }
}


proc _xpCSG.InsView { graph viewspec } {
    upvar #0 $graph.draw svars $graph.$viewspec vdvars ${graph}-views gvs
    lappend gvs $viewspec

    set viewinfo [view info $viewspec]
    set fltrinfo [filter info [lindex $viewinfo 0]]
    set viewspecxf [lindex $fltrinfo 2].[lindex $viewinfo 1]
    set fltrDomain [lindex $fltrinfo 1]
    set viewClass [lindex $viewinfo 2]
    if { "$viewClass"=="CplxSeqTime" && "$fltrDomain"=="z" } {
	set vdvars(styleargs) "-linestyle fence -markstyle dot -markwidth 2"
    } else {
	set vdvars(styleargs) "-linestyle solid -markstyle none -markwidth 2"
    }

    set safeviewspec [string tolower $viewspec]
    regsub -all {\.} $safeviewspec - safeviewspec

    set gl $graph.legend
    set glrow $gl.$safeviewspec
#puts stdout "InsView: $graph $viewspec $glrow"

    set vdvars(glrow) $glrow

    pack append $gl [frame $glrow] { top fillx }
    pack append $glrow [frame $glrow.l] {left frame w}
    pack append $glrow [menubutton $glrow.name -text $viewspecxf] {left expand}
    pack append $glrow [frame $glrow.r] {left frame e}
    set mm $glrow.name.m
    $glrow.name conf -menu $mm
    menu $mm
    $mm add command -lab "Rect" -acc R \
      -com "xpCSG.SetViewComps $graph $viewspec {r i}"
    bind $glrow.name R "$mm invoke Rect"
    $mm add command -lab "Polar"	-acc P \
      -com "xpCSG.SetViewComps $graph $viewspec {m p}"
    bind $glrow.name P "$mm invoke Polar"
    $mm add command -lab "Zap" -acc "^z"	\
      -com "xpCSG.DelSeq $graph $viewspec"
    $mm add command -lab "Config..." \
      -com "xpCSG.ConfigView $graph $viewspec"
    bind $glrow.name <Control-z> "$mm invoke Zap"
    $mm add sep
    foreach cs { "r Real" "i Imag" "m Mag" "p Phase" "wp WrapPhase" } {
	set cscomp [lindex $cs 0]
	set cskey [string index $cscomp 0]
	set cslbl [lindex $cs 1]
	$mm add check -lab $cslbl -acc $cskey \
	  -var ${graph}.${viewspec}($cscomp)
	bind $glrow.name $cskey "$mm invoke $cslbl"
	trace var vdvars($cscomp) w "xpCSG.SeqCompTrace $graph $viewspec"
#trace var vdvars u "topEcho unset vdvars: "
    }

    case $svars(subClass) {
      {Time Impulse} {
	$mm add command -lab "Display Fourier" -com "xpCSG.DpyFourier $viewspec"
      }
    }
}

proc xpCSG.DpyFourier { impulseView } {
     xpView.DisplayByClass $impulseView CplxSeqFourier
#    set viewinfo [view get $impulseView-fourier=ImpulseFourier/$impulseView]
#    xpView.Display [lindex $viewinfo 0].[lindex $viewinfo 1]
}

proc xpCSG.SetViewComps { graph viewspec {comps ""} } {
    upvar #0 $graph.draw svars
    _xpCSG.ParseSeqSpec $graph viewspec comps ""
    upvar #0 $graph.$viewspec vdvars
#    puts stdout "SetViewComps $comps"
    foreach comp "r i m p wp" {
	set vdvars($comp) [expr { !([lsearch $comps $comp]==-1) }]
    }
}

proc xpCSG.InsSeq { graph viewspec {comps ""} {insureOnlyB 0} } {
    upvar #0 $graph.draw svars
    _xpCSG.ParseSeqSpec $graph viewspec comps ""

    upvar #0 $graph.$viewspec vdvars

    if { [info exist vdvars(glrow)] } {
	if $insureOnlyB return
    } else {
	_xpCSG.InsView $graph $viewspec
    }
    set glrow $vdvars(glrow)

    foreach comp $comps {
	set seqname $viewspec,$comp
	if { [xpSSG.SeqExistB $graph $seqname] } continue

	set compinfo $svars(compinfo-$comp)
#puts stdout "$comp $compinfo"
	set complbl [lindex $compinfo 1]
	set compaxis [lindex $compinfo 2]
	set compcolor [lindex $compinfo 3]

	eval {$graph.draw} seqcreate {$seqname} \
	  -fg {$compcolor} -legendlabel {$complbl} -axis {$compaxis} \
	  -datasrc xpole -dataspec {$seqname} $vdvars(styleargs)
# Line below is old-style legend geom
#	pack append $glrow [menubutton $glrow.$comp] { left fill expand }
	set glc $glrow.$compaxis.$comp
	pack append $glrow.$compaxis [menubutton $glc] { left fill expand }
	xpSSG.BindLineLegend $graph $seqname $glc
	$glc.m entryconf Zap -com \
	  "xpCSG.SeqVarSet $graph $viewspec $comp 0"
	set vdvars($comp) 1
	xpSSG.CheckAxes $graph
#	xpSSG.zoomFull $graph.draw
    }
}

proc xpCSG.SeqVarSet { graph viewspec var val } {
    upvar #0 $graph.$viewspec vdvars
    set vdvars($var) $val
}

proc xpCSG.SeqCompTrace { graph viewspec n1 n2 op } {
    upvar #0 $graph.$viewspec vdvars
    set comp $n2
    set existB [xpSSG.SeqExistB $graph $viewspec,$comp]
    if { $vdvars($comp) } {
	if { ! $existB } {
#puts stdout "TraceIns: $comp"
	    if [catch {xpCSG.InsSeq $graph $viewspec $comp} error] {
		tkerror "xpCSG.DelSeq failed"
	    }
	}
    } else {
	if { $existB } {
#puts stdout "TraceDel: $comp"
	    if [catch {xpCSG.DelSeq $graph $viewspec $comp} error] {
		tkerror "xpCSG.DelSeq failed"
	    }
	}
    }
}

proc xpCSG.InsSeqDlgDone { graph viewspec } {
    xpCSG.InsSeq $graph $viewspec
}

proc xpCSG.InsSeqDlg { graph } {
    upvar #0 $graph.draw svars

    xpViewSelDlg "Add $svars(subClass) Sequence" "xpCSG.InsSeqDlgDone $graph"
}

#
# We trace of $graph, and when the graph is destroyed, it unsets
# $graph.  We need to clean up our other global vars.  These are
# listed in ${graph}-views.  Note that we cant put then in
# ${graph}(views), since that gets unset prior to us being invoked.
# Note: Don't trigger if just a single element is being unset.
#
proc _xpCSG.DestroyTraceCB { graph n1 n2 f } {
    if { "$n2" != "" } return
    upvar #0 ${graph}-views gvs
    if [info exist gvs] {
	foreach viewspec $gvs {
#puts stdout "Cleanup $viewspec"
            upvar #0 $graph.$viewspec gdvars
	    catch {unset gdvars}
	}
	unset gvs
    }
}


proc xpCplxSeqGraph { w subClass } {
    upvar #0 $w.draw svars
    global XpFixedFont
    option add *XPSeqGraph.loc*font	$XpFixedFont		30

    xpSSG.InitVars $w
    set svars(subClass) $subClass
    case $subClass {
      Freq { set title "Frequency Domain" }
      Time { set title "Time Domain" }
      default { set title $subClass }
    }
    xpStdTitleBar $w.topbar $title "(xpole)$subClass Win"
    pack append $w $w.topbar { top fillx }

    # Main Menu
    set mm $w.topbar.title.m
    menu $mm
    $mm add command -lab "Add Sequence" -com "xpCSG.InsSeqDlg $w"
    xpSSG.addmenu $w $w.draw $mm

    frame $w.legend
    pack append $w $w.legend {top fill}

    # Measurement cursors
    set dd $w.loc; pack append $w [frame $dd] {top fill}
    pack append $dd [label $dd.l -text "0.0"] {left frame w}
    pack append $dd [frame $dd.name] {left expand}
    pack append $dd.name [label $dd.name.lbl -text "Cursor:"] {left}
    pack append $dd.name [label $dd.name.type -text "Ptr"] {left}
    pack append $dd.name [label $dd.name.xloc -text "0.0"] {left}
    pack append $dd [label $dd.r -text "0.0"] {left frame e}

    set svars(locWins) [list \
      [list $w.bAxis x $w.loc.name.xloc] \
      [list $w.lAxis y $w.loc.l] \
      [list $w.rAxis y $w.loc.r] ]
    xpSSG.leave $w.draw

    # The axis and drawing core
    xpAxisCreate $w.lAxis -side right -flip true -clip 1e-6
    set svars(packcmd-$w.lAxis) [list after $w.loc $w.lAxis {left filly}]

    xpAxisCreate $w.rAxis -side left -flip true
    set svars(packcmd-$w.rAxis) [list after $w.loc $w.rAxis {right filly}]

    xpAxisCreate $w.bAxis -side top
    pack append $w $w.bAxis {bottom fillx}


    xpseqwidget $w.draw
    $w.draw axisconfigure 0 -name b -path $w.bAxis
    $w.draw axisconfigure 1 -name l -path $w.lAxis
    $w.draw axisconfigure 2 -name r -path $w.rAxis
    pack append $w $w.draw {top expand fill}

    $w.lAxis conf -datawindow $w.draw -command "$w.draw axisxf l"
    $w.rAxis conf -datawindow $w.draw -command "$w.draw axisxf r"
    $w.bAxis conf -datawindow $w.draw -command "$w.draw axisxf b"

    rubber $w.draw.rubber $w.draw -command "xpSSG.zoomBox $w.draw"

    # format is "key legend_name axis_name color"
    foreach cs { "r real l red" "i imag l green" \
      "m mag l magenta" "p phase r yellow" "wp wphase r yellow" } {
	set color [lindex $cs 3]
    	set svars(compinfo-[lindex $cs 0]) [lreplace $cs 3 3 [xpDefVal.Color $color White]]
    }
    xpSSG.addbind $w $w.draw $mm
    trace variable svars(legends) w "xpSSG.LegendRefresh $w"
    trace variable svars u "_xpCSG.DestroyTraceCB $w"
    after 100 xpSSG.zoomFull $w.draw
}
