#
#	export.tcl :: UI to export filters to other packages
#
#	Currently only implmented for Ptolemy.
#

proc export.tcl { } { }

proc xpEFD.WriteFileHeader { dlg } {
    upvar #0 $dlg fvars

    puts $fvars(fileid) "# XPole generated filter description"
    puts $fvars(fileid) "# Filter: $fvars(viewspec)"
    puts $fvars(fileid) "# Representation: Ptolemy $fvars(star) $fvars(stardom)"
    puts $fvars(fileid) "#"
}

proc xpEFD.WriteFile { dlg fn args } {
    global errorInfo

    if [catch {set id [open $fn "w"]} error] {
        xpStd.DlgErr $dlg "Filter Realization" $error $errorInfo
    }
    foreach line $args {
        puts $id $line
    }
    close $id
}

proc xpEFD.DispWarning { dlg warning } {
    upvar #0 $dlg fvars
    if { "$warning"=="" } return
    set fvars(warningB) 1
    set firstMsg [lindex [split $warning "\n"] 0]
    xpStd.DlgErr $dlg "Filter Realization" $firstMsg $warning "Warning"
}

#
# common config for all (or most) cascade structs
#
proc xpEFD.CascadeConfig { dlg } {
    upvar #0 $dlg fvars

    set fvars(header_infos) {
      { gainlbl 20 "Global: Gain" }
      { gainval 33 "" }
      { shiftlbl 50 "Shift" }
      { shiftval 57 "" }
      { pad0 0 " " }
      { pad1 70 " " }
    }

    set fvars(tap_infos) {
      { tapsetnum 3 "" } 
      { t0  7 "" } 
      { t1 25 "" } 
      { t2 43 "" } 
      { t3 61 "" } 
    }
}

#
# common config for all (or most) cascade structs
#
proc xpEFD.CascadeUpdate { dlg struct info } {
    upvar #0 $dlg fvars

    case $fvars(how) {
      file {
    	puts $fvars(fileid) "# Structure: $struct/[lindex $info 0]"
    	puts $fvars(fileid) "# GlobalGain: [lindex $info 1]"
    	puts $fvars(fileid) "# GlobalShift: [lindex $info 2]"
      }
      page {
	set tw $fvars(pageid).t
	xpPage.Update $tw $fvars(header_infos) [lrange $info 1 2] header
      }
    }
}

proc xpEFD.PageTaps { page taps infos key } {
    set len [llength $taps]
    for {set i 0} {$i < $len} {incr i 4} {
	set tapset [lrange $taps $i [expr {$i+3}]]
	xpPage.Update $page $infos [linsert $tapset 0 $i] ${key}_${i}
    }
}

proc xpEFD.ConfigFIR { dlg } {
    upvar #0 $dlg fvars
    xpEFD.CascadeConfig $dlg
    append fvars(options) " -real -fir"


    set fvars(firlead_infos) {
      { firleadlbl 0 "FIR#" }
      { firleadnum 4 "" }
      { leadgainlbl 15 "Gain" }
      { leadgainval 20 "" }
      { leadshiftlbl 50 "Shift" }
      { leadshiftval 57 "" }
      { tapsetlbl 2 "Tap#" }
      { t0lbl 13 "i+0" } 
      { t1lbl 31 "i+1" } 
      { t2lbl 49 "i+2" } 
      { t3lbl 67 "i+3" } 
    }
}

proc xpEFD.UpdateFIR { dlg } {
    upvar #0 $dlg fvars

    set firinfo [eval cascade $fvars(options) {$fvars(viewspec)}]
    # firinfo = { structname gain shift firs ... }
    # fir = { gain shift taps ... }
    xpEFD.DispWarning $dlg [lindex $firinfo 0]
    set firinfo [lindex $firinfo 1]
    xpEFD.CascadeUpdate $dlg FIR [lrange $firinfo 0 2]
    case $fvars(how) {
      file {
    	puts $fvars(fileid) "#\n# FIRs are below (format: gain shift taps...)"
	set suf [llength $firinfo]
	set suf [expr { $suf == 4 ? "" : "0" } ]
	foreach fir [lrange $firinfo 3 end] {
	    puts $fvars(fileid) "#   FIR: $fir"
	    xpEFD.WriteFile $dlg "$fvars(filename).taps$suf" [lindex $fir 2]
	    if { "$suf" != "" } { incr suf }
	}
    	puts $fvars(fileid) "#"
    	puts $fvars(fileid) $firinfo
      }
      ipc {
	tcpConnCmd $fvars(ipcid) $firinfo
      }
      page {
	set tw $fvars(pageid).t
	set n 0
	foreach fir [lrange $firinfo 3 end] {
	    # fir = { gain shift { taps } }
	    xpPage.Update $tw $fvars(firlead_infos) "$n [lrange $fir 0 1]" lead${n}
	    xpEFD.PageTaps $tw [lindex $fir 2] $fvars(tap_infos) tset${n}
	    incr n
	}
	xpPage.UpdateEnd $tw
      }
    }
}

proc xpEFD.ConfigIIR { dlg } {
    upvar #0 $dlg fvars
    xpEFD.CascadeConfig $dlg
    append fvars(options) " -real"

    set fvars(iirlead_infos) {
      { iirleadlbl 0 "IIR#" }
      { iirleadnum 4 "" }
      { leadgainlbl 15 "Gain" }
      { leadgainval 20 "" }
      { leadshiftlbl 50 "Shift" }
      { leadshiftval 57 "" }
      { tapsetlbl 2 "N#" }
      { nt0lbl 13 "i+0" } 
      { nt1lbl 31 "i+1" } 
      { nt2lbl 49 "i+2" } 
      { nt3lbl 67 "i+3" } 
    }
    set fvars(iirdenom_infos) {
      { tapsetlbl 2 "D#" }
      { dt0lbl 13 "i+0" } 
      { dt1lbl 31 "i+1" } 
      { dt2lbl 49 "i+2" } 
      { dt3lbl 67 "i+3" } 
    }
}


proc xpEFD.UpdateIIR { dlg } {
    upvar #0 $dlg fvars

    set iirinfo [eval cascade $fvars(options) {$fvars(viewspec)}]
    xpEFD.DispWarning $dlg [lindex $iirinfo 0]
    set iirinfo [lindex $iirinfo 1]
    # iirinfo = { structname gain shift iirs ... }
    # iir = { gain shift {numer_taps} {denom_taps} }
    xpEFD.CascadeUpdate $dlg IIR [lrange $iirinfo 0 2]
    case $fvars(how) {
      file {
    	puts $fvars(fileid) "#\n# IIRs are below (format: gain shift numer_taps denom_taps)"
	set suf [llength $iirinfo]
	set suf [expr { $suf == 4 ? "" : "0" } ]
	foreach iir [lrange $iirinfo 3 end] {
	    puts $fvars(fileid) "#   IIR: [lrange $iir 0 1]"
	    puts $fvars(fileid) "#\tNumer: [lindex $iir 2]"
	    puts $fvars(fileid) "#\tDenom: [lindex $iir 3]"
	    xpEFD.WriteFile $dlg "$fvars(filename).gain$suf" [lindex $iir 0]
	    xpEFD.WriteFile $dlg "$fvars(filename).numer$suf" [lindex $iir 2]
	    xpEFD.WriteFile $dlg "$fvars(filename).denom$suf" [lindex $iir 3]
	    if { "$suf" != "" } { incr suf }
	}
    	puts $fvars(fileid) "#"
    	puts $fvars(fileid) $iirinfo
      }
      ipc {
	tcpConnCmd $fvars(ipcid) $iirinfo
      }
      page {
	set tw $fvars(pageid).t
	set n 0
	foreach iir [lrange $iirinfo 3 end] {
	    xpPage.Update $tw $fvars(iirlead_infos) "$n [lrange $iir 0 1]" nlead${n}
	    xpEFD.PageTaps $tw [lindex $iir 2] $fvars(tap_infos) ntset${n}
	    xpPage.Update $tw $fvars(iirdenom_infos) "" dlead${n}
	    xpEFD.PageTaps $tw [lindex $iir 3] $fvars(tap_infos) dtset${n}
	    incr n
	}
	xpPage.UpdateEnd $tw
      }
    }
}

proc xpEFD.ConfigBiquad { dlg } {
    upvar #0 $dlg fvars
    append fvars(options) " -real -numtaps 3"

    xpEFD.CascadeConfig $dlg
    set fvars(biquadlbl_infos) {
      { quadnumlbl 0 "Quad#" }
      { g0lbl 13 "gain" } 
      { nd0lbl 42 "n0/d0" } 
      { nd1lbl 50 "n1/d1" } 
      { nd2lbl 58 "n2/d2" } 
    }
    set fvars(biquad_infos) {
      { quadnum 0 "" } 
      { g0  7 "" } 
      { n0 25 "" } 
      { n1 43 "" } 
      { n2 61 "" } 
      { d0 25 "" } 
      { d1 43 "" } 
      { d2 61 "" } 
    }
}

proc xpEFD.UpdateBiquad { dlg } {
    upvar #0 $dlg fvars

    set quadinfo [eval cascade $fvars(options) {$fvars(viewspec)}]
    xpEFD.DispWarning $dlg [lindex $quadinfo 0]
    set quads [lindex $quadinfo 1]
    # quads = { structure gain shift quad... }
    # quad = { ... }
    xpEFD.CascadeUpdate $dlg Biquad [lrange $quads 0 2]
    case $fvars(how) {
      file {
    	puts $fvars(fileid) "#\n# Biquads are below format: gain shift {n0 n1 n2} {d0 d1 d2}"
	foreach quad [lrange $quads 3 end] {
	    puts $fvars(fileid) "#   Quad: $quad"
	}
    	puts $fvars(fileid) $quads
      }
      ipc {
	tcpConnCmd $fvars(ipcid) $quads
      }
      page {
	set tw $fvars(pageid).t
	xpPage.Update $tw $fvars(biquadlbl_infos) "" biquadlbl
	set i 0
	foreach quad [lrange $quads 3 end] {
	    # quad = { gain shift zeroTaps poleTaps }
	    set flatquad "$i [lrange $quad 0 0] [lindex $quad 2] [lindex $quad 3]"
	    xpPage.Update $tw $fvars(biquad_infos) $flatquad quad$i
	    incr i
	}
	xpPage.UpdateEnd $tw
      }
    }
}


proc xpEFD.UpdateMedia { dlg } {
    upvar #0 $dlg fvars

    if { "$fvars(how)"=="file" } {
	set fvars(fileid) [open $fvars(filename) w]
	xpEFD.WriteFileHeader $dlg
    }
    set fvars(warningB) 0
    xpEFD.Update$fvars(star) $dlg
    if { "$fvars(how)"=="file" } {
	close $fvars(fileid)
	set fvars(fileid) ""
    }
    if { ! $fvars(warningB) } {
	set fvars(needUpdateB) 0
	xpStd.DlgErrSetStatus $dlg "(valid)"
	xpStd.DlgErrClear $dlg
    }
}

proc xpEFD.SafeUpdateMedia { dlg } {
    global errorInfo
    upvar #0 $dlg fvars

    if [catch {xpEFD.UpdateMedia $dlg} error] {
	xpStd.DlgErr $dlg "Export Filter (xpEFD.UpdateMedia)" $error $errorInfo
	return 0
    }
    return 1
}

proc xpEFD.SilarUpdateCB { dlg tag idxList netPower } {
    upvar #0 $dlg fvars
    set fvars(needUpdateB) 1
#puts stdout "Got update for $dlg : $idxList"
    if { $fvars(autoUpdateB) } {
        xpEFD.SafeUpdateMedia $dlg
    } else {
	xpStd.DlgErrSetStatus $dlg "(update needed)"
    }
}

#
# called by trace on fvars(autoUpdateB)
#
proc xpEFD.AutoUpdateCB { dlg n1 n2 op } {
    upvar #0 $dlg fvars
#puts stdout "AutoUpdateCB: $dlg  $fvars(autoUpdateB) $fvars(needUpdateB)"
    if { $fvars(autoUpdateB) && $fvars(needUpdateB) } {
        xpEFD.SafeUpdateMedia $dlg
    }
}

proc xpEFD.Open { dlg } {
    upvar #0 $dlg fvars

    case $fvars(how) {
      file {
	  # Do open in media update.
      }
      page {
	# Dont need to do much, since the export control window
	# will also serve as the paged display window
	set w $fvars(pageid)
        pack before $w.buts [xpPage.Create $w.t] { top fill expand }
      }
      ipc {
	set portinfo [split $fvars(portn) "@"]
	set fvars(ipcid) \
	  [tcpConn XPole [lindex $portinfo 0] [lindex $portinfo 1]]
      }
    }

    set fvars(orig_opts) $fvars(options)
    xpEFD.Config$fvars(star) $dlg
}

proc xpEFD.MakeCtrlWin { dlg } {
    upvar #0 $dlg fvars

    set w $dlg
    topWin.DestroyChildren $w
    pack append $w [xpStdTitleBar $w.topbar "Export Control" \
      "(xpole)Export Fltr Win"] {top fillx}
    set fvars(pageid) $w
    set m $w.topbar.title.m; menu $m
    $m add check -lab "Auto Update" -var ${dlg}(autoUpdateB)
    $m add command -lab "Zap Window" -acc "^z" \
      -com "xpEFD.Close $dlg"

    set fullstar "$fvars(stardom) $fvars(star)"
    pack append $w [label $w.title \
      -text "$fvars(viewspecfull) via $fvars(howdesc) as $fullstar"] { top fillx }
    pack append $w [xpStd.DlgErrCreate $dlg $w.status "Ready"] { top fillx }
    pack append $w [topWin.LbldEntry $w.options \
      [list -text "Options:"] \
      [list -textvar ${dlg}(options) -rel sunken] ] { top fillx }
    bind $w.options.entry <Any-Leave> "focus none"
    bind $w.options.entry <Escape> "focus none"
    bind $w.options.entry <Return> "focus none; $w.buts.update invoke"


    set dd $w.buts; pack append $w [frame $dd] { top fillx }
    pack append $dd [button $dd.update -text "Update" \
      -com "xpEFD.SafeUpdateMedia $dlg"] { left expand fill }
    pack append $dd [button $dd.close -text "Close" \
      -com "$m invoke Zap*"] { left expand fill }
    topWin.BindTree $w <Control-z> "$m invoke Zap"
    return $w
}

#
# Close the previous export media.  Note that we cant use fvars(how)
# to determine which media to close; instead, we must try them all.
# (ideally, we would install a callback func. in fvars, but thats too
# complicated.)
#
proc xpEFD.Close { dlg } {
    upvar #0 $dlg fvars

    if [info exist fvars(orig_opts)] {
        set fvars(options) $fvars(orig_opts)
	unset fvars(orig_opts)
    }
    if { [info exist fvars(agent)] } {
	agent destroy $fvars(agent)
	unset fvars(agent)
    }
    set fvars(autoUpdateB)	0
    if { "$fvars(fileid)" != "" } {
	close $fvars(fileid)
	set fvars(fileid) ""
    }
    if { "$fvars(ipcid)" != "" } {
	$fvars(ipcid) close
	set fvars(ipcid) ""
    }
    if { "$fvars(pageid)" != "" } {
	topWin.SafeDestroy $fvars(pageid)
	set fvars(pageid) ""
    }
}

proc xpEFD.SafeUpdateHow { dlg } {
    upvar #0 $dlg fvars
    global errorInfo

    set fvars(star) [$dlg.l.star.box get [$dlg.l.star.box cursel]]
    set fvars(stardom) [$dlg.l.dom.box get [$dlg.l.dom.box cursel]]

    # Make sure nothing left around
    xpEFD.Close $dlg

    case $fvars(how) {
      file { set fvars(howdesc) "File $fvars(filename)" }
      ipc { set fvars(howdesc) "IPC $fvars(portn)" }
      page { set fvars(howdesc) "Page Window" }
    }
    set ctrlWinB 0
    if { "$fvars(how)" == "page" || "$fvars(how)" == "ipc" } {
	set fvars(autoUpdateB) [expr { $fvars(when) == "change" } ]
	set ctrlWinB 1
    }
    if { "$fvars(when)"=="change" || $ctrlWinB } {
	set ctrlWinB 1
	xpEFD.MakeCtrlWin $dlg
    }

    if [catch {xpEFD.Open $dlg} error] {
	xpStd.DlgErr $dlg "Export Filter (xpEFD.Ok,open)" $error $errorInfo
	return 0
    }

    #
    # Update it for the first time.  Dont care if suceeds or not.
    #
    set r 1
    if [catch {xpEFD.UpdateMedia $dlg} error] {
	xpStd.DlgErr $dlg "Export Filter (xpEFD.Ok,media)" $error $errorInfo
	set r 0
    }

    #
    # If updateable, arrange for future update notifies;
    # otherwise close it down now
    #
    if { $ctrlWinB } {
        set fvars(agent) [lindex \
	  [view agent $fvars(viewspec) "xpEFD.SilarUpdateCB $dlg"] 2]
    } else {
	if [catch {xpEFD.Close $dlg} error] {
	    xpStd.DlgErr $dlg "Export Filter (xpEFD.Ok,close)" $error $errorInfo
	    return 0
	}
	if { $r } {
	    destroy $dlg
	}
    }
    return $r
}

# Filters may be exported in a variety of ways:
# to a text widget, disk file(s), or IPC socket.  This is the "how".
#
# The filter may be represented in a variety of different formats,
# cooresponding to different filter structures and optimizations of
# those structures.  This is the "media", also called the "what".
#
# In the implmentation of the dialog system, there are then two types
# of updates: a how-update, where the previous system must be closed
# down and a new one opened, and a what-update, which just requires
# rewriting the same how-media.
#



proc xpEFD.Ok { dlg } {
    if { ! [xpEFD.SafeUpdateHow $dlg] } {
	return
    }
#    xpEFD.Popdown $dlg
}


#
# We dont just popdown (unmap) it, since there is no way to get it
# back.  Instead, just kill it.
proc xpEFD.Popdown { dlg } {
#    focus none
#    windowunmap $dlg
    destroy $dlg
}

proc xpEFD.HowCB { dlg n1 n2 op } {
    upvar #0 $dlg fvars
    set ddb $dlg.buts
    set ddp $dlg.opts.p
#puts stdout "xpEFD.HowCB: $dlg $fvars(how)"
    case $fvars(how) {
      file {
	$ddp.fn.entry   conf -state normal	-relief sunken
	$ddp.port.entry conf -state disabled	-relief flat
#	$ddb.update	conf -state normal
      }
      ipc {
	$ddp.fn.entry   conf -state disabled	-relief flat
	$ddp.port.entry conf -state normal	-relief sunken
#	$ddb.update	conf -state disabled
      }
      page {
	$ddp.fn.entry   conf -state disabled	-relief flat
	$ddp.port.entry conf -state disabled	-relief flat
#	$ddb.update	conf -state disabled
      }
    }
}

proc xpExportFltrDlg { dlg fltrname viewname } {
    upvar #0 $dlg fvars
    global xpole

    set fltrinfo [filter info $fltrname]
    catch {unset fvars}
    set fvars(fltrname)		$fltrname
    set fvars(viewspec)		$fltrname.$viewname
    set fvars(viewspecfull)	"[lindex $fltrinfo 2].$viewname"
    set fvars(how)		file
    set fvars(when)		once
    set fvars(star)		FIR
    set fvars(filename)		"$fltrname.ptf"
    set fvars(portn)		"2323@aristarchus"
    set fvars(options)		""
    set fvars(autoUpdateB)	0
    set fvars(needUpdateB)	0
    set fvars(warningB)		0

    set fvars(pageid)		""
    set fvars(fileid)		""
    set fvars(ipcid)		""

    menu $dlg.topbar.title.m
    label $dlg.name -text $fvars(viewspecfull)
    pack append $dlg $dlg.name { top }
    
    pack append $dlg [frame $dlg.opts] { top fill }
    set ddb $dlg.opts.b; pack append $dlg.opts [frame $ddb] { left fill }

    set dd $ddb.how
    pack append $ddb [frame $dd -bd 2 -rel raised] { top fillx}
    pack append $dd [radiobutton $dd.file -text "File" -rel flat -anc w \
      -var ${dlg}(how) -val file] {top fill}
    pack append $dd [radiobutton $dd.ipc  -text "IPC" -rel flat -anc w \
      -var ${dlg}(how) -val ipc \
      -state [expr {"[info command tcp]"!=""?"normal":"disabled"}] ] {top fill}
    pack append $dd [radiobutton $dd.window -text "Window" -rel flat -anc w \
      -var ${dlg}(how) -val page] {top fill}
    trace var fvars(how) w "xpEFD.HowCB $dlg"

    set dd $ddb.when
    pack append $ddb [frame $dd -bd 2 -rel raised] { top fillx}
    pack append $dd [radiobutton $dd.once -text "Once" -rel flat -anc w \
      -var ${dlg}(when) -val once] {top fill}
    pack append $dd [radiobutton $dd.change  -text "Change" -rel flat -anc w \
      -var ${dlg}(when) -val change] {top fill}

    set ddp $dlg.opts.p; pack append $dlg.opts [frame $ddp] { left expand fill }

    pack append $ddp [topWin.LbldEntry $ddp.fn \
      [list -text "Filename:" -width 10] \
      [list -textvar ${dlg}(filename)] ] { top fill }
    pack append $ddp [topWin.LbldEntry $ddp.port \
      [list -text "IPC Port:" -width 10] \
      [list -textvar ${dlg}(portn)] ] { top fill }

    set ddl $dlg.l; pack append $dlg [frame $ddl] { top expand fill }

    set dd $ddl.star; pack append $ddl [frame $dd] { left expand fill }
    pack append $dd [label $dd.lbl -text "Star"] { top fill }
    pack append $dd [listbox $dd.box -rel sunken -export 0] { top expand fill }
    eval {$dd.box} insert end $xpole(ptolemy-stars)
    tk_listboxSingleSelect $dd.box
    $dd.box select to 0


    set dd $ddl.dom; pack append $ddl [frame $dd] { left expand fill }
    pack append $dd [label $dd.lbl -text "Domain"] { top fill }
    pack append $dd [listbox $dd.box -rel sunken -export 0] { top expand fill }
    eval {$dd.box} insert end $xpole(ptolemy-domains)
    tk_listboxSingleSelect $dd.box
    $dd.box select to 0

    set dd $dlg.buts; pack append $dlg [frame $dd -bd 2] { top fill }
    button $dd.ok -text "Ok" -com "xpEFD.Ok $dlg"
    pack append $dd $dd.ok { left expand fill }
#    button $dd.update -text "Update" -com "xpEFD.SafeUpdateHow $dlg"
#    pack append $dd $dd.update { left expand fill }
    button $dd.cancel -text "Cancel" -com "xpEFD.Popdown $dlg"
    pack append $dd $dd.cancel { left expand fill }

    # Trigger trace
    set fvars(how) $fvars(how)
    trace var fvars(autoUpdateB) w "xpEFD.AutoUpdateCB $dlg"
}

#
# Note:	The original implementation allowed only one export window per
#	viewspec.  This was not only overly-restrictive, but led to conflicts
#	in the global fvars() array.
#
proc xpExportFltrDlgPopup { {fltrview ""} } {
    global xpole

    if {! [info exist xpole(ptolemy-stars)] } {
	# set xpole(ptolemy-stars) [list FIR IIR Biquad BiquadCascade]
	set xpole(ptolemy-stars) [list FIR IIR Biquad]
    }
    if {! [info exist xpole(ptolemy-domains)] } {
	# set xpole(ptolemy-domains) [list SDF CG56 CG96]
	set xpole(ptolemy-domains) [list SDF]
    }
    if { "$fltrview"=="" } {
	xpViewSelDlg "Export Filter Dialog" "xpExportFltrDlgPopup"
	return
    }
    set viewinfo [xpView.GetByClass $fltrview SilarRep]
    set fltrname [lindex $viewinfo 0]
    set viewname [lindex $viewinfo 1]
    set fltrinfo [filter info $fltrname]
    set domain [lindex $fltrinfo 1]
    if { $domain != "z" } {
	xpStd.DlgErr "" "Export Filter (xpExportFltrDlgPopup)" \
	  "XPole can only export z-domain filters" ""
	return
    }

    set viewspec $fltrname.$viewname
    # The uniqueization of the names is tricky here: the main "popup"
    # window may not exist, but its child control window may still
    # exist.  Thus we must serialize the names ourselves.
    set dlg [xpStdDlgPopup .export[topGenSN] "Export $viewspec" "Export Filter" "(xpole)Export Fltr"]
    xpExportFltrDlg $dlg $fltrname $viewname
    bind $dlg <Destroy> "topWin.Unset %W"
}
