#if !defined(_PIPE_H_)
#define _PIPE_H_ 1
/* 
    Author: Kennard White

    Copyright (c) 1990-93 The Regents of the University of California.
    All rights reserved.  See the file "./copyright.h" for full copyright
    notice, limitation of liability, and disclaimer of warranty provisions.
#include "copyright.h"				COPYRIGHTENDKEY
*/

typedef void (XPPipeNotifyFunc)( XPPipeMod *pPipe, XPViewEvent ev, XPViewSnap *pSrcSnap, XPAgent *pTgtAg);

/**
    Describes a given pipe.  The {CmNotifyFunc} is called whenever
    the src view is modified.  The {PmNotifyFunc} is called whenever
    the tgt view is modified.  As a special case, if {PmNotifyFunc}
    is NULL and notify-type is ParamUpdate, then {CmNotifyFunc} will be
    called with notify-type XPVE_Switch.
**/
struct _XPPipeClassSide {
    char*		ViewClass;
    TOPMask		Flags;
    XPPipeNotifyFunc*	NotifyFunc;
    TOPLogical		(*UpdateFunc) ARGS((XPPipeMod *pPipe, 
			  XPViewSnap *pSrcSnap, XPViewSnap *pTgtSnap));
};

struct _XPPipeClass {
    char*		Name;
    TOPMask		Flags;
    XPPipeClassSide	cSide;
    XPPipeClassSide	pSide;
};

#define XPPipeF_Active		TOP_BIT(0)
#define XPPipeF_Update		TOP_BIT(1)
#define XPPipeF_UpdateTgt	TOP_BIT(2)
#define XPPipeF_UpdateSwitch	TOP_BIT(3)
#define XPPipeF_Alive		TOP_BIT(4)
#define XPPipeF_SimplePC	TOP_BIT(5)
#define XPPipeF_ProdParams	TOP_BIT(6)
#define XPPipeF_MultiPart	TOP_BIT(7)
#define XPPipeF_Update1		TOP_BIT(8)

typedef struct _XPPipeSide XPPipeSide;
struct _XPPipeSide {
    XPPipeClassSide*	Class;
    XPLabel		Lbl;
    XPAgent		Ag;
    TOPPtr		Ext;
};

struct _XPPipeMod {
    XPLabel		Lbl;
    XPPipeClass*	Class;
    int			Flags;
    XPPipeSide		cSide;
    XPPipeSide		pSide;
    TOPMask		WantValsMask;
    /*XPCplxSeqSnapValMask	WantValsMask;*/		/* for RFTF_T */
};


/*
 * pipe.c
 */
extern TOPLogical	xpPipeGetTypeInfo( char *className, 
			  char **pSrcClass, char **pTgtClass);
extern XPLabel*	xpPipeCreate( char *name, char *class, 
			  XPView *pSm, XPView *pTm);
extern void		xpPipeDestroy( XPPipeMod *pPipe);
extern void		xpPipeAddClass( XPPipeClass *pCls);
extern void		xpPipeAddStdClasses();
extern XPPipeMod*	xpPipeFindByName( char *name);

/*
 * pipeTcl.c
 */
extern void		xpPipeFmtTcl ARGS_TCL((Tcl_Interp *ip, 
			  XPPipeMod *pPipe, TOPLogical nestB));
extern void		xpPipeRegisterCmds ARGS_TCL((Tcl_Interp *ip));

#endif /* _PIPE_H_ */
