/* 
    pipeTcl.c :: TCL interface to XPole Pipes

    Author: Kennard White

    Copyright (c) 1990-93 The Regents of the University of California.
    All rights reserved.  See the file "./copyright.h" for full copyright
    notice, limitation of liability, and disclaimer of warranty provisions.
#include "copyright.h"				COPYRIGHTENDKEY
*/
#if !defined(lint) && !defined(SABER)
static char rcsid[] = "$Header: /vol/cooley/cooley1/kennard/src/xpole/RCS/pipeTcl.c,v 1.5 93/08/27 14:28:17 kennard Exp $";
#endif

#include "topFixup.h"
#include <tcl.h>

#include "topStd.h"
#include "topBuf.h"

#include "pole.h"
#include "filter.h"
#include "cplxseq.h"
#include "silar.h"
#include "pipe.h"

extern char _XpPipeSubPackage[];
#define SPKG _XpPipeSubPackage


/**
    Pipe Format: name class tag cSideAg pSideAg
**/
void
xpPipeFmtTcl( Tcl_Interp *ip, XPPipeMod *pPipe, TOPLogical nestB) {
    if ( nestB )		Tcl_AppendResult(ip," {",NULL);
    xpLabelFmtTcl( ip, &pPipe->Lbl, TOP_FALSE);
    Tcl_AppendResult(ip,
      " ",pPipe->cSide.Lbl.Name,
      " ",pPipe->pSide.Lbl.Name," ",NULL);
    if ( nestB )		Tcl_AppendResult(ip," }",NULL);
}

/**
    Note that modifies get turned into switches here.  For now, it
    is to difficult to record all the state to distingush between
    Modify/Switch.  Also, note that in order to use Modify we must
    propagate the modify info at Notify time, not Update time.
    (i.e., downstream notify handlers need good notify state when called.)
**/
static void
_xpPipeSilarTclNotify( XPPipeMod *pPipe, XPViewEvent ev,
  XPViewSnap *pSrcSnap, XPAgent *pTgtAg) {
    /* XPSilarSnap		*pSR = (XPSilarSnap*) pSrcSnap; */

    switch ( ev ) {
    case XPVE_Modify:
    case XPVE_Switch:
	XPVIEWPART_LOOP_FORW_BEGIN( pTgtAg->CurView, pVP) {
	    pVP->Flags &= ~XPViewF_Valid;
	    xpViewPartNotify( pVP, XPNW_Now, XPVE_Switch, pTgtAg);
	} XPVIEWPART_LOOP_FORW_END();
	break;
    default:
	;
    }
}

static TOPLogical
_xpPipeSilarTclUpdate( XPPipeMod *pPipe, 
  XPViewSnap *pSrcSnap, XPViewSnap *pTgtSnap) {
    XPSilarSnap		*pSrcSR = (XPSilarSnap*) pSrcSnap;
    Tcl_Interp		*ip = XPTheMachine.ip;
    int			r;
    char		*lbuf;

    lbuf = topBufLFmt( "{_xpPipe%sNotify} {%s} {%s} {%s}",
      pPipe->Lbl.Class, pPipe->Lbl.Name,
      pSrcSR->PntView->DM.Lbl.Tag, pTgtSnap->PntView->Lbl.Tag);
    r = _xpSilarNotifyTcl( pSrcSnap->PntView, XPVE_Switch, pSrcSnap, ip, lbuf);
    topBufUnlock(lbuf);
    return r == TCL_OK;
}

static int
_xpPipeCmd( ClientData cld, Tcl_Interp *ip, int aC, char **aV) {
    char	*cmd = aV[1], c;
    int		len;
    XPPipeMod	*pPipe;
    XPMachine	*pMach = &XPTheMachine;

    c = cmd[0]; len = strlen(cmd);
    /*IF*/ if ( c=='i' && strncmp(cmd,"info",len)==0 ) {
	if ( aC == 2 ) {
	    DLPTRLIST_LOOP_FORW_BEGIN( pMach->PipeList, XPPipeMod*, pPip) {
		if ( pPip==NULL )	continue;
		xpPipeFmtTcl( ip, pPip, TOP_TRUE);
	    } DLPTRLIST_LOOP_FORW_END();
	} else {
	    pPipe = xpPipeFindByName( aV[2]);
	    if ( pPipe == NULL ) {
		Tcl_AppendResult(ip,aV[0]," ",cmd,": no such pipe ``",
		  aV[2],"''",NULL);
		return TCL_ERROR;
	    }
	    xpPipeFmtTcl( ip, pPipe, TOP_FALSE);
	}
    } else {
	Tcl_AppendResult(ip,aV[0],": invalid command ``",cmd,"''",NULL);
	return TCL_ERROR;
    }
    return TCL_OK;
}

static int
_xpPipeClassCmd( ClientData cld, Tcl_Interp *ip, int aC, char **aV) {
    XPPipeClass		cls;

    if ( aC != 4 ) {
	Tcl_AppendResult(ip,"usage: ",aV[0]," name srcClass dstClass",NULL);
	return TCL_ERROR;
    }
    MEM_CLEAR(&cls);
    cls.Name = memStrSave(aV[1]);
    cls.Flags = 0;
    cls.cSide.ViewClass = memStrSave(aV[2]);
    cls.pSide.ViewClass = memStrSave(aV[3]);
    /*IF*/ if ( strcmp(cls.cSide.ViewClass,"SilarRep")==0 ) {
	cls.cSide.NotifyFunc = _xpPipeSilarTclNotify;
	cls.cSide.UpdateFunc = _xpPipeSilarTclUpdate;
    } else {
	Tcl_AppendResult(ip,aV[0],": source class ``",cls.cSide.ViewClass,
	  "'' is unsupported.", NULL);
	return TCL_ERROR;
    }

    xpPipeAddClass( &cls);
    return TCL_OK;
}

void
xpPipeRegisterCmds( Tcl_Interp *ip) {
    Tcl_CreateCommand( ip, "xppipe", _xpPipeCmd,
      (ClientData)&XPTheMachine, NULL);
    Tcl_CreateCommand( ip, "xppipeclass", _xpPipeClassCmd,
      (ClientData)&XPTheMachine, NULL);
}
