/* 
    Author: Kennard White

    Copyright (c) 1990-93 The Regents of the University of California.
    All rights reserved.  See the file "./copyright.h" for full copyright
    notice, limitation of liability, and disclaimer of warranty provisions.
#include "copyright.h"				COPYRIGHTENDKEY
*/
#if !defined(lint) && !defined(SABER)
static char rcsid[] = "$Header: /vol/cooley/cooley1/kennard/src/xpole/RCS/tkpole.c,v 1.5 93/08/27 14:28:13 kennard Exp $";
#endif

#include "topFixup.h"
#include <tk.h>

#include "topStd.h"
#include "topMisc.h"
#include "topCB.h"

#include "xpSeq.h"
#include "xpPlane.h"

#include "pole.h"
#include "filter.h"
#include "silar.h"
#include "cplxseq.h"
#include "pipe.h"

/*global*/ XPMachine XPTheMachine;


void
xpLabelConstructor( XPLabel *pLbl,
  TOPPtr pMe, int *pSN, char *name, char *class) {
    static int _XpLblSN = 0;
    char		*tagbase;

    pLbl->Me = pMe;
    if ( pSN == NULL )
	pSN = & _XpLblSN;
    pLbl->SN = ++(*pSN);
    if ( name && *name == '\0' )	name = NULL;
    if ( class && *class == '\0' )	class = NULL;
    tagbase = name ? name : ( class ? class : "xxx" );
    pLbl->Tag = memStrSaveFmt("%s%d", tagbase, pLbl->SN);
    pLbl->Name = memStrSave(name ? name : pLbl->Tag);
    pLbl->Class = memStrSave( class);
}

void
xpLabelDestructor( XPLabel *pLbl) {
    pLbl->Me = NULL;
    pLbl->SN = -1;
    MEM_FREE( pLbl->Tag);	pLbl->Tag = NULL;
    MEM_FREE( pLbl->Name);	pLbl->Name = NULL;
    MEM_FREE( pLbl->Class);	pLbl->Class = NULL;
}

static void
xpViewAddStdClasses() {
    xpViewClassAdd( &XpSilarViewClass);
    xpViewClassAdd( &XpCplxSeqViewClass);
}

static void
xpMachineConstructor( XPMachine *pMach) {
    MEM_CLEAR(pMach);
    pMach->FltrList = dlListCreate( "XPFilter", sizeof(XPFilter));
    pMach->AgentList = dlPtrListCreate( "XPAgent");
    pMach->PipeList = dlPtrListCreate( "XPPipe");
}

int main( int argc, char **argv) {
    Tcl_Interp		*ip;
    Tk_Window		win;
    int			r;

    memInitialize();
    topSetAppName( topBasename(argv[0]));
    topMsgSetDbgMode( 1);

    xpMachineConstructor( &XPTheMachine);
    ip = Tcl_CreateInterp();
    XPTheMachine.ip = ip;

    if ( (r=topTkInitialize1( ip, &argc, argv, &win)) != TCL_OK ) {
	fprintf( stderr, "Error: %s\n", topTclGetRetMsg(ip,r));
	exit(1);
    }

    memRegisterCmds( ip);
    topMsgRegisterCmds( ip);
    /* hlpTtyRegisterCmds( ip); */
    eemCplxRegisterCmds( ip);
    xpPoleRegisterCmds( ip);
    xpSeqWdgRegisterCmds( ip, win);
    xpPlaneWdgRegisterCmds( ip, win);

    xpViewAddStdClasses();
    xpPipeAddStdClasses();

    if ( (r=topTkInitialize2( ip, win, argc, argv)) != TCL_OK ) {
	fprintf( stderr, "Error: %s\n", topTclGetRetMsg(ip,r));
	exit(1);
    }

    XPTheMachine.RunEventsB = TOP_TRUE;
    while ( XPTheMachine.RunEventsB ) {
	while ( Tk_DoOneEvent(1) )			/* non-blocking */
	    ;
	while ( topIdleCallbacksCall( NULL) > 0 )	/* non-blocking */
	    ;
	Tk_DoOneEvent(0);				/* blocking */
    }
    exit(0);
}
