/* 
    Author: Kennard White

    Copyright (c) 1990-93 The Regents of the University of California.
    All rights reserved.  See the file "./copyright.h" for full copyright
    notice, limitation of liability, and disclaimer of warranty provisions.
#include "copyright.h"				COPYRIGHTENDKEY
*/
#if !defined(lint) && !defined(SABER)
static char rcsid[] = "$Header: /vol/cooley/cooley1/kennard/src/xpole/RCS/view.c,v 1.6 93/08/27 14:28:11 kennard Exp $";
#endif

#include "topFixup.h"
#include <stdio.h>
#include <tcl.h>

#include "topStd.h"
#include "topCB.h"

#include "pole.h"
#include "filter.h"
#include "cplxseq.h"
#include "silar.h"
#include "pipe.h"

char _XpTrajSubPackage[] = "xp.traj";
#define SPKG _XpTrajSubPackage

/**************************************************************************
 *
 *			 ViewClass
 *
 **************************************************************************/

static DLListHandle _XpViewClassTbl = NULL;

static void
_xpViewClassTblInit() {
    if ( _XpViewClassTbl != NULL )
	return;
    _XpViewClassTbl = dlListCreate( "XPViewClass", sizeof(XPViewClass));
}


void
xpViewClassAdd( XPViewClass *pNewClass) {
    XPViewClass	*pClass;
    _xpViewClassTblInit();
    pClass = (XPViewClass*) dlListCreateAtTail( _XpViewClassTbl);
    *pClass = *pNewClass;
}

XPViewClass*
xpViewClassFindByName( char *name) {
    DLLIST_LOOP_FORW_BEGIN( _XpViewClassTbl, XPViewClass*, pClass) {
	int len = strlen(pClass->Name);
	if ( strncmp( name, pClass->Name, len)==0 ) {
	    return pClass;
	}
    } DLLIST_LOOP_FORW_END();
    return NULL;
}

/**************************************************************************
 *
 *			 View
 *
 **************************************************************************/

static void _xpViewNotifyIdleUndo( XPView *pView);
static void _xpViewPartDestructor( XPViewPart *pVP);

/**
    Initialize the "base" XPView portion of {pView}.  The label {pView.Lbl}
    should have already been initialized.
**/
void
xpViewConstructor( XPView *pView, XPFilter *pFltr, XPXformDomain xfd) {
    pView->PntFltr = pFltr;
    pView->Idx = -1;
    pView->LockCnt = 0;
    pView->PartList = dlListCreate("XPViewPart", sizeof(XPViewPart));
    pView->XfDomain = xfd;
    pView->SampleTime = 1.0;
    pView->Desc = NULL;
}

void
xpViewDestructor( XPView *pView) {
    XPVIEWPART_LOOP_FORW_BEGIN( pView, pVP) {
	_xpViewPartDestructor( pVP);
    } XPVIEWPART_LOOP_FORW_END();
    dlListDestroy( pView->PartList, NULL, NULL);
    pView->PartList = NULL;

    if ( pView->PntFltr ) {
	if ( pView->Idx >= 0 ) {
	    TOPVEC_GET( &pView->PntFltr->Views, pView->Idx, XPView*) = NULL;
	}
	pView->Idx = -1;
        pView->PntFltr = NULL;
    }
    _xpViewNotifyIdleUndo( pView);
    xpLabelDestructor( &pView->Lbl);
}

/**
    Notes: A view is by default up-to-date.  In general, only a view's
    master will mark it out-of-date.
**/
static void
_xpViewPartConstructor( XPViewPart *pVP, XPView *pView, char *partName) {
    MEM_CLEAR(pVP);
    pVP->PntView = pView;
    pVP->Name = memStrSave(partName ? partName : "default");
    pVP->Flags = XPViewF_Valid;
    pVP->AgentList = dlPtrListCreate("XPAgents");
}

static void
_xpViewPartDestructor( XPViewPart *pVP) {
    if ( pVP->LockCnt > 0 ) {
	topProgMsg("ViewPart is locked upon destruction: %s",
	  pVP->PntView->Lbl.Tag);
	return;
    }
    xpViewPartNotifyAgents( pVP, XPVE_Destroy, NULL);
    dlPtrListDestroy( pVP->AgentList, NULL, NULL);
    pVP->AgentList = NULL;
    MEM_FREE(pVP->Name);
    pVP->Name = NULL;
}

/**
    Update view {pView} part {partIdx}.  Returns TRUE if
    update suceeded.  (It may fail if a pipe parameter is invalid
    or if views farther up the tree are unavailable).
**/
TOPLogical
xpViewPartUpdate( XPViewPart *pVP) {
    XPAgent	*pAg = pVP->PntView->MasterAgent;
    TOPLogical	r;

    if ( TOP_IsOn(pVP->Flags,XPViewF_Valid) ) {
	return TOP_TRUE;	/* view is already valid */
    }
    if ( pAg != NULL ) {
	if ( pAg->UpdateFunc == NULL ) {
    	    topProgMsg("Master agent %s doesn't have update function.",
	      pAg->Lbl->Tag);
	} else {
	    if ( TOP_IsOn(pVP->Flags,XPViewF_UpdateLock) ) {
    		topProgMsg("Recursive update in view %s",
		  pVP->PntView->Lbl.Tag);
    		return TOP_FALSE;
	    }
	    pVP->Flags |= XPViewF_UpdateLock;
    	    r = (*(pAg->UpdateFunc))( pAg, pVP);
	    pVP->Flags &= ~XPViewF_UpdateLock;
	    if ( r )
		pVP->Flags |= XPViewF_Valid;
	    return r;
	}
    }
    topProgMsg("Don't know how to update view %s", pVP->PntView->Lbl.Tag);
    return TOP_FALSE;
}

static void
_xpViewDispatch( XPAgent *pAg, XPViewPart *pVP, XPViewEvent ev) {
    TOPMask		not_evm;
    XPNotifyWhen	retryWhen;
    if ( pAg->NotifyFunc == NULL ) {
	/* one-way producers wont have notifyfuncs, but will be on the
	 * agent list.  So its ok for it to be on the list, but it shouldn't
	 * really have any Switch/Modify events being sent to it.
	 */
	return;
    }
    if ( ev == XPVE_Null ) {
	/*IF*/ if ( TOP_IsOn(pAg->Events,XPVEM_Param) ) {
	    not_evm = ~XPVEM_Param;
	    ev = XPVE_Param;
	} else if ( TOP_IsOn(pAg->Events,XPVEM_Switch) ) {
	    not_evm = ~(XPVEM_Switch|XPVEM_Modify);
	    ev = XPVE_Switch;
	} else if ( TOP_IsOn(pAg->Events,XPVEM_Modify) ) {
	    not_evm = ~XPVEM_Modify;
	    ev = XPVE_Modify;
	} else {
	    return;
	}
    } else {
	not_evm = ~ TOP_BIT((int)ev);
    }
    pAg->Events &= not_evm;
    retryWhen = (*(pAg->NotifyFunc))( pAg, pVP, ev);
    switch ( retryWhen ) {
    case XPNW_Null: break;	/* this normal case */
    case XPNW_Now:
    case XPNW_Idle:
	pAg->Events |= ~not_evm;
        xpViewPartNotify( pVP, retryWhen, XPVE_Null, NULL);
	break;
    case XPNW_Defer:
	pAg->Events |= ~not_evm;
	break;
    }
}

/**
    "Clean" viewpart {pVP}.  That means look for NULL agent pointers.
    To avoid problems, we lock down the agentlist and dont allow
    removals while traversing.  This function is called after traversal
    to clean up.
**/
static void
_xpViewPartClean( XPViewPart *pVP) {
    while ( dlPtrListElemRemove( pVP->AgentList, NULL) == 0 )
	;
    pVP->Flags &= ~XPViewF_DoClean;
}

/**
    Notify every agent in {pVP} about event {ev}.
**/
void
xpViewPartNotifyAgents( XPViewPart *pVP, XPViewEvent ev, XPAgent *pAgOmit) {
    ++(pVP->LockCnt);
    DLPTRLIST_LOOP_FORW_BEGIN( pVP->AgentList, XPAgent*, pAgent) {
	if ( pAgent == NULL )	continue;
	if ( pAgent != pAgOmit )
	    _xpViewDispatch( pAgent, pVP, ev);
    } DLPTRLIST_LOOP_FORW_END();
    --(pVP->LockCnt);
    if ( pVP->LockCnt==0 && TOP_IsOn(pVP->Flags,XPViewF_DoClean) )
	_xpViewPartClean( pVP);
}

void
xpViewNotifyAgents( XPView *pView, XPViewEvent ev, XPAgent *pAgOmit) {

    XPVIEWPART_LOOP_FORW_BEGIN( pView, pVP) {
	xpViewPartNotifyAgents( pVP, ev, pAgOmit);
    } XPVIEWPART_LOOP_FORW_END();
}


static TOPStatus
_xpViewNotifyIdleCB( TOPPtr cbClosure, TOPPtr userData) {
    XPView		*pView = (XPView*) cbClosure;

    pView->Flags &= ~XPViewF_IdleCalc;
    xpViewNotify( pView, XPNW_Now, XPVE_Null, NULL);
    return TOP_CB_REMOVE;
}

/**
    Notify all agents in ViewPart {pVP} of event {ev}.  If {pAg} in non-NULL,
    it will not be notifyed.  If {when} is {XPNW_Now}, synchronous notify
    will occur; {XPNW_Idle} causes the notify to occur later (allowing
    multiple updates to be collapsed together.
**/
void
xpViewPartNotify( XPViewPart *pVP, XPNotifyWhen when,
  XPViewEvent ev, XPAgent *pAgOmit) {
    XPView	*pView = pVP->PntView;
    TOPMask	evm = TOP_BIT((int)ev);

    if ( when == XPNW_Now ) {
	if ( pView->Class->NotifyFunc != NULL ) {
	    (*(pView->Class->NotifyFunc))( pVP, ev, pAgOmit);
	} else {
	    xpViewPartNotifyAgents( pVP, ev, pAgOmit);
	}
    } else {
	if ( ev != XPVE_Null ) {
	    XPAGENT_LOOP_FORW_BEGIN( pVP, pAgent) {
		if ( pAgent == NULL )	continue;
		if ( pAgent == pAgOmit ) {
		    if ( TOP_IsOn(pAgOmit->Events,evm) )
			topProgMsg("Possible recursion in %s", pAgent->Lbl->Tag);
		} else {
		    pAgent->Events |= evm;
		}
	    } XPAGENT_LOOP_FORW_END();
	}

	if ( TOP_IsOff(pView->Flags,XPViewF_IdleCalc) ) {
	    pView->Flags |= XPViewF_IdleCalc;
	    topIdleCallbacksAdd( _xpViewNotifyIdleCB, (TOPPtr) pView);
	}
    }
}

void
xpViewNotify( XPView *pView, XPNotifyWhen when,
  XPViewEvent ev, XPAgent *pAgOmit) {
    XPVIEWPART_LOOP_FORW_BEGIN( pView, pVP) {
	xpViewPartNotify( pVP, when, ev, pAgOmit);
    } XPVIEWPART_LOOP_FORW_END();
}

static void
_xpViewNotifyIdleUndo( XPView *pView) {
    if ( TOP_IsOn(pView->Flags,XPViewF_IdleCalc) ) {
	pView->Flags &= ~XPViewF_IdleCalc;
    	topIdleCallbacksRemove( _xpViewNotifyIdleCB, (TOPPtr) pView);
    }
}

XPViewPart*
xpViewPartFindByName( XPView *pView, char *partName) {
    XPVIEWPART_LOOP_FORW_BEGIN( pView, pVP) {
	if ( strcmp(pVP->Name,partName)==0 )
	    return pVP;
    } XPVIEWPART_LOOP_FORW_END();
    return NULL;
}

/**
    If the view supports multiparts and {partName} is non-NULL, then
    {pAgent} becomes the owner of the part.  It is an error for the
    partName to already be owned.
**/
void
xpViewAddAgent( XPView *pView, XPAgent *pAgent, char *partName) {
    XPViewPart	*pVP = NULL;

    pAgent->CurView = pView;

    if ( TOP_IsOff(pView->Flags,XPViewF_MultiPart) ) {
	pVP = (XPViewPart*) dlListGetAtHead(pView->PartList);
    } else {
	if ( partName != NULL ) {
	    pVP = xpViewPartFindByName( pView, partName);
	    if ( pVP != NULL ) {
		if ( pVP->RangeOwner != NULL ) {
		    topProgMsg("%s: View %s, part %s already owned.  Stealing.",
		      SPKG, pView->Lbl.Name, partName);
		}
		pVP->RangeOwner = pAgent;
	    }
	}
    }
    if ( pVP == NULL ) {
	pVP = (XPViewPart*) dlListCreateAtTail(pView->PartList);
	_xpViewPartConstructor( pVP, pView, partName);
	pVP->RangeOwner = pAgent;
    }

    dlPtrListInsertAtTail( pVP->AgentList, pAgent);
    pAgent->ViewPart = pVP;
}

void
xpViewRemoveAgent( XPView *pView, XPAgent *pAgent) {
    TOPPtr	*pAgentRef;
    XPViewPart	*pPV = pAgent->ViewPart;

    if ( pView != pAgent->CurView ) {
	topAbortMsg("xpViewRemoveAgent: inconsistent view");
    }
    if ( pView->MasterAgent == pAgent ) {
	pView->MasterAgent = NULL;
    }
    if ( pPV->RangeOwner == pAgent )
	pPV->RangeOwner = NULL;
    if ( pPV->LockCnt > 0 ) {
	if ( (pAgentRef = dlPtrListGetRef( pPV->AgentList, pAgent)) == NULL ) {
	    topAbortMsg("agent missing from view");
	}
	*pAgentRef = NULL;
	pPV->Flags |= XPViewF_DoClean;
    } else {
        dlPtrListElemRemove( pPV->AgentList, pAgent);
    }
    pAgent->ViewPart = NULL;
    pAgent->CurView = NULL;
}

char*
xpViewGetFullName( XPView *pView) {
    XPFilter		*pFltr = pView->PntFltr;
    return memStrSaveFmt( "%s.%s", pFltr->Name, pView->Lbl.Name);
}

void
xpAgentConstructor( XPAgent *pAgent, XPLabel *pLbl, XPView *pView) {
    MEM_CLEAR(pAgent);
    pAgent->Lbl = pLbl;
    pAgent->Flags = 0;;
    pAgent->DmFlags = 0;
    pAgent->CurView = pView;
    pAgent->CurSet = NULL;
    pAgent->TclCB = NULL;

    dlPtrListInsertAtTail( XPTheMachine.AgentList, (TOPPtr) pAgent);

    if ( pView != NULL ) {
        xpViewAddAgent( pView, pAgent, NULL);
    }
}

void
xpAgentDestructor( XPAgent *pAgent) {
    Tcl_Interp		*ip = XPTheMachine.ip;

    if ( pAgent->TclCB ) {
	Tcl_DeleteCommand( ip, pAgent->Lbl->Tag);
	pAgent->TclCB = NULL;
    }
    if ( pAgent->CurView ) {
	xpViewRemoveAgent( pAgent->CurView, pAgent);
        pAgent->CurView = NULL;
    }
    dlPtrListElemRemove( XPTheMachine.AgentList, (TOPPtr) pAgent);
    pAgent->Lbl = NULL;
    pAgent->NotifyFunc = NULL;
}

XPViewSnap*
xpViewPartGetSnap( XPViewPart *pVP, XPGetSnapHow how) {
     XPView *pView = pVP->PntView;
     if ( pVP->DfltVS == NULL ) {
     	pVP->DfltVS = (*(pView->Class->SnapCreateFunc))( pView);
	pVP->DfltVS->PntVP = pVP;
    }
    if ( how == XPGS_Upd ) {
	if ( ! xpViewPartUpdate( pVP) )
	    return NULL;
    }
    return pVP->DfltVS;
}

XPViewPart*
xpViewGetPrimaryPart( XPView *pView) {
    XPViewPart *pVP;
    if ( pView->PrimaryPart != NULL )
	return pView->PrimaryPart;
    if ( (pVP = (XPViewPart*) dlListGetAtHead(pView->PartList)) == NULL ) {
	pVP = (XPViewPart*) dlListCreateAtTail(pView->PartList);
	_xpViewPartConstructor( pVP, pView, NULL);
    }
    return pVP;
}

XPViewSnap*
xpViewGetSnap( XPView *pView, XPGetSnapHow how) {
    XPViewPart *pVP;
    pVP = xpViewGetPrimaryPart( pView);
    return xpViewPartGetSnap( pVP, how);
}

XPViewPart*
xpAgentGetCurVP( XPAgent *pAg) {
    XPView	*pView = pAg->CurView;
    XPViewPart	*pVP;
    if ( TOP_IsOff(pView->Flags,XPViewF_MultiPart) ) {
	pVP = xpViewGetPrimaryPart( pView);
    } else {
        pVP = pAg->ViewPart;
    }
    (void) xpViewPartGetSnap( pVP, XPGS_AsIs);	/* sanity check */
    return pVP;
}

XPViewSnap*
xpAgentGetSnap( XPAgent *pAg, XPGetSnapHow how) {
    XPViewPart	*pVP = xpAgentGetCurVP( pAg);
    return xpViewPartGetSnap( pVP, how);
}

void
xpViewSetMaster( XPView *pView, XPAgent *pAg) {
    pAg->CurView = pView;
    if ( pView->MasterAgent != NULL ) {
	topWarnMsg("View already has master.");
    }
    pView->MasterAgent = pAg;
    pView->PrimaryPart = pAg->ViewPart;
}

XPAgent*
xpAgentFindByName( char *name, char *class) {
    DLPTRLIST_LOOP_FORW_BEGIN( XPTheMachine.AgentList, XPAgent*, pAgent) {
	if ( pAgent == NULL )           continue;
	if ( class && strcmp(pAgent->Lbl->Class,class)!=0 )	continue;
	if ( name && (strcmp(pAgent->Lbl->Tag,name)==0
	  || strcmp(pAgent->Lbl->Name,name)==0) )
	    return pAgent;
    } DLPTRLIST_LOOP_FORW_END();
    return NULL;
}

void
xpAgentNotify( XPAgent *pAg, XPNotifyWhen when, XPViewEvent ev) {
    XPViewPart	*pVP = xpAgentGetCurVP( pAg);
    pAg->Events |= TOP_BIT((int)ev);
    xpViewPartNotify( pVP, when, XPVE_Null, NULL);
}

/*****************************************************************************

	data mod stuff

*****************************************************************************/

XPView*
xpViewCreateByClass( XPFilter *pFltr, char *name, char *class) {
    XPView		*pView;
    XPViewClass		*pClass;

    if ( (pClass = xpViewClassFindByName( class)) == NULL ) {
	topErrMsg("Unknown view class ``%s''", class);
	return NULL;
    }
    pView = (XPView*) memMalloc(pClass->byteSize);
    memset( (TOPPtr)pView, '\0', pClass->byteSize);
    pView->Class = pClass;
    xpLabelConstructor( &pView->Lbl, (TOPPtr)pView, pClass->SerialPtr,
      name, class);
    xpViewConstructor( pView, pFltr, pFltr->XfDomain);
    (*(pClass->ConstructorFunc))( pView, class);
    xpFltrAddView( pFltr, pView);
    return pView;
}

void
xpViewDestroy( XPView *pView) {
    (*(pView->Class->DestructorFunc))( pView);
    xpViewDestructor( pView);
    xpLabelDestructor( &pView->Lbl);
    MEM_FREE( pView);
    xpFltrListChange();
}

XPView*
xpViewFindByName( XPFilter *pFltr, char *name, char *class) {
    XPVIEW_LOOP_FORW_BEGIN( pFltr, pView) {
	if ( class && strcasecmp( pView->Lbl.Class, class)!=0 )
	    continue;
	if ( name && strcasecmp( pView->Lbl.Name, name)!=0 )
	    continue;
	return pView;
    } XPVIEW_LOOP_FORW_END();
    return NULL;
}

/**
    Find or create a view in {pFltr}.  The view will be of class {class}.
    The {name} may take one of the forms:
	viewName
	viewName=pipeClass/sourceViewSpec
**/
XPView*
xpViewGetByName( XPFilter *pFltr, char *name, char *class) {
    XPView		*pView, *pSrcView;
    char		*viewName, *pipeClass, *pntName;
    char		*srcClass, *tgtClass;

    viewName = memStrSave( name);
    if ( (pipeClass=strchr( viewName, '=')) != NULL ) {
	*pipeClass = '\0'; ++pipeClass;
    }
    if ( (pView=xpViewFindByName( pFltr, viewName, NULL)) != NULL ) {
	memFree(viewName);
	/* we got it exactly, yeah!! */
#ifdef notdef
	if ( pipeClass != NULL ) {
	    topErrMsg("View ``%s'' already exists", viewName);
	    return NULL;
	}
#endif
	if ( class
	  && strncasecmp( pView->Lbl.Class, class, strlen(class))!=0 ) {
	    topErrMsg("Class-type mismatch in ``%s.%s'' (wanted %s, got %s)",
	      pFltr->Name, viewName, class, pView->Lbl.Class);
	    return NULL;
	}
	return pView;
    }

    /*
     * No pipespec: see if its a built-in type that we can auto-create
     */
    if ( pipeClass == NULL ) {
	memFree(viewName);
	if ( class != NULL ) {
	    pView = xpViewCreateByClass( pFltr, name, class);
	} else {
	    topErrMsg("No such view ``%s.%s'' (and no class given)",
	      pFltr->Name, name);
	    return NULL;
	}
	return pView;
    }

    /*
     * Build the pipe, recursively if needed
     */
    if ( (pntName = strchr( pipeClass, '/')) == NULL ) {
	topErrMsg("Malformed auto-create viewspec, should be %s",
	  "fltrSpec.viewSpec=pipeClass/srcViewSpec");
	memFree( viewName);
	return NULL;
    }
    *pntName = '\0'; ++pntName;

    if ( xpPipeGetTypeInfo( pipeClass, &srcClass, &tgtClass) == TOP_FALSE ) {
	topErrMsg("No such pipe class ``%s''", pipeClass);
	goto bad;
    }
    if ( (pSrcView=xpViewGetBySpec( pntName, srcClass, pFltr, TOP_TRUE))
      == NULL ) {
	topErrMsg("Missing source view for ``%s''", viewName);
	goto bad;
    }
    if ( (pView=xpViewCreateByClass( pFltr, viewName, tgtClass)) == NULL ) {
	topErrMsg("Cant create target view %s", name);
	goto bad;
    }
    if ( (xpPipeCreate( NULL, pipeClass, pSrcView, pView)) == NULL ) {
	topErrMsg("pipe create failed on %s", name);
        /* xpViewDestroy( pView); */
	goto bad;
    }
    memFree( viewName);
    return pView;
bad:
    memFree( viewName);
    return NULL;
}

/**
    A viewSpec has several different forms.  The most general is:
	(1) fltrSpec.viewName[=pipeinfo]
    where fltrSpec must exist; see xpFltrFindBySpec().
    If {pFltr} is non-NULL, then fltrSpec may be ommitted:
	(2) viewName[=pipeinfo]

    If both {*ppFltr} is non-null and {spec} specifies a filter,
    they may conflict.  In the case of confict, {hintB} indicates
    which should happen:
      if true, then {*ppFltr} considered only a hint and {spec} will dominate.  
      If false, then an error is signaled.

    Upon return, {*ppFltr} will be set to the specified filter and
    {*ppViewName} will point to a dynamically allocated string
    containing "viewName=pipeinfo".  The =pipeinfo is optional and ignored
    by this function.
**/
TOPLogical
xpViewParseSpec( char *spec, XPFilter **ppFltr, TOPLogical hintB,
  char **ppViewName) {
    char	*fltrSpec = NULL;
    char	*tmpName, *viewName, *pipeName;

    if ( spec == NULL || spec[0] == NULL ) {
	topErrMsg("Empty (null) view spec.");
	return TOP_FALSE;
    }

    tmpName = memStrSave(spec);
    if ( (pipeName = strchr( tmpName, '=')) != NULL )
	*pipeName = '\0';
    if ( (viewName = strrchr( tmpName, '.')) == NULL ) {
	fltrSpec = NULL; viewName = tmpName; /* form (2) */
    } else {
	fltrSpec = tmpName; *viewName = '\0'; ++viewName; /* form (1) */
    }
    if ( viewName == NULL || *viewName == '\0' ) {
	topErrMsg("Missing view name in ``%s''", tmpName);
	goto bad;
    }
    if ( pipeName )
        *pipeName = '=';
    if ( xpFltrFindBySpec( fltrSpec, ppFltr, hintB) == TOP_FALSE
      || *ppFltr == NULL ) {
	topErrMsg("Bad filter ``%s'' in ``%s''", fltrSpec, tmpName);
	goto bad;
    }
    *ppViewName = memStrSave( viewName);
    memFree( tmpName);
    return TOP_TRUE;
bad:
    *ppFltr = NULL;
    *ppViewName = NULL;
    memFree( tmpName);
    return TOP_FALSE;
}

/**
    Given a viewspec {spec} of the form described by xpViewParseSpec(),
    and a possibly null class, return the view.  If {pFltr} is non-null,
    it specifies at default (hint) filter for {spec}.  If {hintB} is true,
    then the filter in {spec} must agree with {pFltr}.
**/
XPView*
xpViewFindBySpec( char *spec, char *class, XPFilter *pFltr, TOPLogical hintB) {
    char		*viewName;
    XPView		*pView;
    XPFilter		*pInFltr = pFltr;

    if ( ! xpViewParseSpec( spec, &pInFltr, hintB, &viewName) )
	return NULL;
    pView = xpViewFindByName( pInFltr, viewName, class);
    memFree( viewName);
    return pView;
}

/**
    Same as xpViewGetByName(), except that it will create the view if
    it doesnt already exist.
**/
XPView*
xpViewGetBySpec( char *spec, char *class, XPFilter *pFltr, TOPLogical hintB) {
    XPView		*pView;
    char		*viewName;
    XPFilter		*pInFltr = pFltr;

    if ( ! xpViewParseSpec( spec, &pInFltr, hintB, &viewName) )
	return NULL;
    pView = xpViewGetByName( pInFltr, viewName, class);
    memFree( viewName);
    return pView;
}

#ifdef notdef
XPAgent*
xpAgentFindByName( XPView *pView, char *name, char *class) {
    DLPTRLIST_LOOP_FORW_BEGIN( pView->AgentList, XPAgent*, pAgent) {
	if ( pAgent == NULL )           continue;
	if ( class && strcmp(pAgent->Lbl->Class,class)!=0 )	continue;
	if ( strcmp(pAgent->Lbl->Tag,name)==0
	  || strcmp(pAgent->Lbl->Name,name)==0 )
	    return pAgent;
    } DLPTRLIST_LOOP_FORW_END();
    return NULL;
}

XPAgent*
xpAgentFindBySpec( char *spec, char *class) {
    char	*s, *tmpSpec;
    XPView	*pView;
    XPAgent	*pAgent;

    if ( (s=strrchr( spec, '.'))==NULL )
	return NULL;
    tmpSpec = memStrSave(spec);
    tmpSpec[s-spec] = '\0';
    if ( (pView = xpViewFindBySpec( tmpSpec, NULL, NULL, NULL)) == NULL ) {
	memFree(tmpSpec);
	return NULL;
    }
    pAgent = xpAgentFindByName( pView, s+1, class);
    memFree(tmpSpec);
    return pAgent;
}

#endif
